@@ -28,6 +28,7 @@
 #include "gtk_common.h"
 #include "gtk_ruler.h"
 #include "mathfn.h"
+#include "x11menu.h"
 
 #include <math.h>
 #include <string.h>
@@ -39,7 +40,7 @@
 #define MAXIMUM_SCALES        10
 
 typedef struct _Nruler {
-  int orientation;
+  int orientation, ofst, length, size;
 #if GTK_CHECK_VERSION(3, 0, 0)
   cairo_surface_t *backing_store;
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
@@ -47,6 +48,7 @@ typedef struct _Nruler {
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
   double lower, upper, position;
   double save_l, save_u;
+  GtkWidget *widget, *parent;
 } Nruler;
 
 struct _NrulerMetric
@@ -66,30 +68,26 @@ static const struct _NrulerMetric Metric = {
 
 #define RULER_DATA_KEY "nruler"
 
-static void nruler_make_pixmap(Nruler *ruler, GtkWidget *widget);
+static void nruler_make_pixmap(Nruler *ruler, GtkWidget *widget, GtkWidget *parent);
 static void nruler_draw_ticks(Nruler *ruler, GtkWidget *widget);
 static void nruler_realize(GtkWidget *widget, gpointer user_data);
 static void nruler_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data);
-static void nruler_size_request(GtkWidget *widget, GtkRequisition *requisition, gpointer user_data);
 static gboolean nruler_destroy(GtkWidget *widget, gpointer user_data);
 #if GTK_CHECK_VERSION(3, 0, 0)
 static void nruler_draw_pos(Nruler *ruler, GtkWidget *widget, cairo_t *cr);
 static gboolean nruler_expose(GtkWidget *widget, cairo_t *cr, gpointer user_data);
-static void nruler_get_preferred_width(GtkWidget *widget, gint *minimal_width, gint *natural_width);
-static void nruler_get_preferred_height(GtkWidget *widget, gint *minimal_height, gint *natural_height);
+static void nruler_get_color(Nruler *ruler, GdkRGBA *fg, GdkRGBA *bg);
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
 static void nruler_draw_pos(Nruler *ruler, GtkWidget *widget);
 static gboolean nruler_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
+static void nruler_get_color(Nruler *ruler, GdkColor *fg, GdkColor *bg);
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 
-static GtkWidget *
-ruler_new(int orientation)
+GtkWidget *
+nruler_new(GtkOrientation orientation)
 {
   Nruler *ruler;
-  GtkWidget *w;
-#if GTK_CHECK_VERSION(3, 0, 0)
-  GtkWidgetClass *widget_class;
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
+  GtkWidget *w, *frame;
 
   ruler = g_malloc0(sizeof(*ruler));
   if (ruler == NULL) {
@@ -97,49 +95,45 @@ ruler_new(int orientation)
   }
 
   w = gtk_drawing_area_new();
-  ruler->orientation = orientation;
+  if (orientation == GTK_ORIENTATION_VERTICAL) {
+    gtk_widget_set_size_request(w, RULER_WIDTH, -1);
+  } else {
+    gtk_widget_set_size_request(w, -1, RULER_WIDTH);
+  }
 
-  g_object_set_data(G_OBJECT(w), RULER_DATA_KEY, ruler);
+  frame = gtk_frame_new(NULL);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_OUT);
+  gtk_container_add(GTK_CONTAINER(frame), w);
+
+  ruler->orientation = orientation;
+  ruler->widget = w;
+  ruler->parent = frame;
+
+  g_object_set_data(G_OBJECT(frame), RULER_DATA_KEY, ruler);
 
 #if GTK_CHECK_VERSION(3, 0, 0)
   g_signal_connect(w, "draw", G_CALLBACK(nruler_expose), ruler);
-
-  widget_class = GTK_WIDGET_GET_CLASS(w);
-  widget_class->get_preferred_width = nruler_get_preferred_width;
-  widget_class->get_preferred_height = nruler_get_preferred_height;
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
   g_signal_connect(w, "expose-event", G_CALLBACK(nruler_expose), ruler);
-  g_signal_connect(w, "size-request", G_CALLBACK(nruler_size_request), ruler);
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
+
   g_signal_connect(w, "realize", G_CALLBACK(nruler_realize), ruler);
   g_signal_connect(w, "size-allocate", G_CALLBACK(nruler_size_allocate), ruler);
-  g_signal_connect(w, "unrealize", G_CALLBACK(nruler_destroy), ruler);
+  g_signal_connect(frame, "unrealize", G_CALLBACK(nruler_destroy), ruler);
 
-  return w;
-}
-
-GtkWidget *
-hruler_new(void)
-{
-  return ruler_new(GTK_ORIENTATION_HORIZONTAL);
-}
-
-GtkWidget *
-vruler_new(void)
-{
-  return ruler_new(GTK_ORIENTATION_VERTICAL);
+  return frame;
 }
 
 void
-nruler_set_range(GtkWidget *w, double lower, double upper)
+nruler_set_range(GtkWidget *frame, double lower, double upper)
 {
   Nruler *ruler;
 
-  if (w == NULL) {
+  if (frame == NULL) {
     return;
   }
 
-  ruler = g_object_get_data(G_OBJECT(w), RULER_DATA_KEY);
+  ruler = g_object_get_data(G_OBJECT(frame), RULER_DATA_KEY);
   if (ruler == NULL) {
     return;
   }
@@ -147,29 +141,29 @@ nruler_set_range(GtkWidget *w, double lower, double upper)
   ruler->lower = lower;
   ruler->upper = upper;
 
-  if (gtk_widget_is_drawable(w)) {
-    gtk_widget_queue_draw(w);
+  if (gtk_widget_is_drawable(ruler->widget)) {
+    gtk_widget_queue_draw(ruler->widget);
   }
 }
 
 void
-nruler_set_position(GtkWidget *w, double position)
+nruler_set_position(GtkWidget *frame, double position)
 {
   Nruler *ruler;
 
-  if (w == NULL) {
+  if (frame == NULL) {
     return;
   }
 
-  ruler = g_object_get_data(G_OBJECT(w), RULER_DATA_KEY);
+  ruler = g_object_get_data(G_OBJECT(frame), RULER_DATA_KEY);
   if (ruler == NULL) {
     return;
   }
 
   ruler->position = position;
 
-  if (gtk_widget_is_drawable(w)) {
-    gtk_widget_queue_draw(w);
+  if (gtk_widget_is_drawable(ruler->widget)) {
+    gtk_widget_queue_draw(ruler->widget);
   }
 }
 
@@ -182,103 +176,19 @@ nruler_destroy(GtkWidget *widget, gpointer user_data)
 
   ruler = (Nruler *) user_data;
   if (ruler) {
+    if (ruler->backing_store) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+      cairo_surface_destroy(ruler->backing_store);
+#else
+      g_object_unref(ruler->backing_store);
+#endif
+    }
     g_free(ruler);
   }
 
   return FALSE;
 }
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-static void
-nruler_get_preferred_width(GtkWidget *widget, gint *minimal_width, gint *natural_width)
-{
-  GtkRequisition requisition;
-  Nruler *ruler;
-
-  ruler = g_object_get_data(G_OBJECT(widget), RULER_DATA_KEY);
-  if (ruler == NULL) {
-    return;
-  }
-
-  nruler_size_request(widget, &requisition, ruler);
-  *minimal_width = *natural_width = requisition.width;
-}
-
-static void
-nruler_get_preferred_height(GtkWidget *widget, gint *minimal_height, gint *natural_height)
-{
-  GtkRequisition requisition;
-  Nruler *ruler;
-
-  ruler = g_object_get_data(G_OBJECT(widget), RULER_DATA_KEY);
-  if (ruler == NULL) {
-    return;
-  }
-
-  nruler_size_request(widget, &requisition, ruler);
-  *minimal_height = *natural_height = requisition.height;
-}
-
-static void
-get_thickness(GtkWidget *widget, GtkStyleContext *style, gint *xt, gint *yt)
-{
-  GtkBorder border;
-  gint xthickness, ythickness;
-
-  gtk_style_context_get_border(style, gtk_widget_get_state_flags(widget), &border);
-  xthickness = (border.left + border.right);
-  ythickness = (border.top + border.bottom);
-
-  gtk_style_context_get_padding(style, gtk_widget_get_state_flags(widget), &border);
-  xthickness += (border.left + border.right);
-  ythickness += (border.top + border.bottom);
-
-  gtk_style_context_get_margin(style, gtk_widget_get_state_flags(widget), &border);
-  xthickness += (border.left + border.right);
-  ythickness += (border.top + border.bottom);
-
-  *xt = xthickness;
-  *yt = ythickness;
-}
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
-
-static void
-nruler_size_request(GtkWidget *widget, GtkRequisition *requisition, gpointer user_data)
-{
-#if GTK_CHECK_VERSION(3, 0, 0)
-  Nruler *ruler;
-  gint xthickness, ythickness;
-  GtkStyleContext *style;
-
-  ruler = (Nruler *) user_data;
-
-  style = gtk_widget_get_style_context(widget);
-  get_thickness(widget, style, &xthickness, &ythickness);
-
-  if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    requisition->width  = xthickness + 1;
-    requisition->height = ythickness + RULER_WIDTH;
-  } else {
-    requisition->width  = xthickness + RULER_WIDTH;
-    requisition->height = ythickness + 1;
-  }
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  Nruler *ruler;
-  GtkStyle *style;
-
-  ruler = (Nruler *) user_data;
-  style = gtk_widget_get_style(widget);
-
-  if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    requisition->width  = style->xthickness * 2 + 1;
-    requisition->height = style->ythickness * 2 + RULER_WIDTH;
-  } else {
-    requisition->width  = style->xthickness * 2 + RULER_WIDTH;
-    requisition->height = style->ythickness * 2 + 1;
-  }
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
-}
-
 static void
 nruler_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
 {
@@ -286,7 +196,7 @@ nruler_size_allocate(GtkWidget *widget, GtkAllocation *allocation, gpointer user
 
   ruler = (Nruler *) user_data;
 
-  nruler_make_pixmap(ruler, widget);
+  nruler_make_pixmap(ruler, widget, ruler->parent);
 }
 
 static void
@@ -296,38 +206,60 @@ nruler_realize(GtkWidget *widget, gpointer user_data)
 
   ruler = (Nruler *) user_data;
 
-  nruler_make_pixmap(ruler, widget);
+  nruler_make_pixmap(ruler, widget, ruler->parent);
 }
 
-static gboolean
 #if GTK_CHECK_VERSION(3, 0, 0)
+static gboolean
 nruler_expose(GtkWidget *widget, cairo_t *cr, gpointer user_data)
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-nruler_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 {
   if (gtk_widget_is_drawable(widget)) {
     Nruler *ruler = (Nruler *) user_data;
 
     nruler_draw_ticks(ruler, widget);
-#if GTK_CHECK_VERSION(3, 0, 0)
     nruler_draw_pos(ruler, widget, cr);
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-    nruler_draw_pos(ruler, widget);
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
   }
 
   return FALSE;
 }
+#else	/* GTK_CHECK_VERSION(3, 0, 0) */
+static gboolean
+nruler_expose(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
+{
+  if (gtk_widget_is_drawable(widget)) {
+    Nruler *ruler = (Nruler *) user_data;
+
+    nruler_draw_ticks(ruler, widget);
+    nruler_draw_pos(ruler, widget);
+  }
+
+  return FALSE;
+}
+#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 
 static void
-nruler_make_pixmap(Nruler *ruler, GtkWidget *widget)
+nruler_make_pixmap(Nruler *ruler, GtkWidget *widget, GtkWidget *parent)
 {
   gint width;
   gint height;
-  GtkAllocation allocation;
+  GtkAllocation allocation, parent_allocation;
+
+  if (! gtk_widget_is_drawable(widget)) {
+    return;
+  }
 
   gtk_widget_get_allocation(widget, &allocation);
+  gtk_widget_get_allocation(parent, &parent_allocation);
+
+  if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
+    ruler->length = parent_allocation.width;
+    ruler->size = allocation.height;
+    ruler->ofst = (parent_allocation.width - allocation.width) / 2;
+  } else {
+    ruler->length = parent_allocation.height;
+    ruler->size = allocation.width;
+    ruler->ofst = (parent_allocation.height - allocation.height) / 2;
+  }
 
   if (ruler->backing_store) {
 #if GTK_CHECK_VERSION(3, 0, 0)
@@ -370,9 +302,6 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
 {
   cairo_t *cr;
   gint i, j, len;
-  gint width, height;
-  gint xthickness;
-  gint ythickness;
   gint length, ideal_length;
   gdouble lower, upper;		/* Upper and lower limits, in ruler units */
   gdouble increment;		/* Number of pixels per unit */
@@ -389,12 +318,10 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
   PangoFontDescription *fs;
   GtkAllocation allocation;
 #if GTK_CHECK_VERSION(3, 0, 0)
-  GtkStyleContext *style;
-  GdkRGBA color;
+  GdkRGBA bg, fg;
 #else
-  GtkStyle *style;
+  GdkColor bg, fg;
 #endif
-  GtkStateType state;
 
   if (! gtk_widget_is_drawable(widget)) {
     return;
@@ -405,17 +332,6 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
   }
 
   gtk_widget_get_allocation(widget, &allocation);
-  state = gtk_widget_get_state(widget);
-#if GTK_CHECK_VERSION(3, 0, 0)
-  style = gtk_widget_get_style_context(widget);
-  get_thickness(widget, style, &xthickness, &ythickness);
-  xthickness /= 2;
-  ythickness /= 2;
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  style = gtk_widget_get_style(widget);
-  xthickness = style->xthickness;
-  ythickness = style->ythickness;
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 
   layout = gtk_widget_create_pango_layout(widget, "012456789");
   fs = pango_font_description_new();
@@ -427,52 +343,28 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
 
   digit_height = PANGO_PIXELS(ink_rect.height) + 2;
 
-  if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    width = allocation.width;
-    height = allocation.height - ythickness * 2;
-  } else {
-    width = allocation.height;
-    height = allocation.width - ythickness * 2;
-  }
-
+  nruler_get_color(ruler, &fg, &bg);
 #if GTK_CHECK_VERSION(3, 0, 0)
   cr = cairo_create(ruler->backing_store);
-  gtk_render_background(style,
-			cr, 0, 0, allocation.width, allocation.height);
-  gtk_render_frame(style,
-		   cr, 0, 0, allocation.width, allocation.height);
+  gdk_cairo_set_source_rgba(cr, &bg);
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
   cr = gdk_cairo_create(ruler->backing_store);
-  gtk_paint_box(style, ruler->backing_store,
-		GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		NULL, widget,
-		ruler->orientation == GTK_ORIENTATION_HORIZONTAL ?
-		"hruler" : "vruler",
-		0, 0,
-		allocation.width, allocation.height);
+  gdk_cairo_set_source_color(cr, &bg);
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
+  cairo_rectangle(cr, 0, 0, allocation.width, allocation.height);
+  cairo_fill(cr);
 
 #if GTK_CHECK_VERSION(3, 0, 0)
-  gtk_style_context_get_color(style, state, &color);
-  gdk_cairo_set_source_rgba(cr, &color);
+  gdk_cairo_set_source_rgba(cr, &fg);
 #else
-  gdk_cairo_set_source_color(cr, &style->fg[state]);
+  gdk_cairo_set_source_color(cr, &fg);
 #endif
 
   if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    cairo_rectangle(cr,
-		    xthickness,
-		    height + ythickness,
-		    allocation.width - 2 * xthickness,
-		    1);
+    cairo_rectangle(cr, 0, allocation.height - 1, allocation.width, 1);
   } else {
-    cairo_rectangle(cr,
-		    height + xthickness,
-		    ythickness,
-		    1,
-		    allocation.height - 2 * ythickness);
+    cairo_rectangle(cr, allocation.width - 1, 0, 1, allocation.height);
   }
-
   upper = ruler->upper / Metric.pixels_per_unit;
   lower = ruler->lower / Metric.pixels_per_unit;;
 
@@ -480,7 +372,7 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
     goto out;
   }
 
-  increment = (gdouble) width / (upper - lower);
+  increment = (gdouble) ruler->length / (upper - lower);
 
   /* determine the scale H
    *  We calculate the text size as for the vruler instead of using
@@ -530,7 +422,7 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
     /* Calculate the length of the tickmarks. Make sure that
      * this length increases for each set of ticks
      */
-    ideal_length = height / (i + 1) - 1;
+    ideal_length = ruler->size / (i + 1) - 1;
     if (ideal_length > ++length) {
       length = ideal_length;
     }
@@ -544,15 +436,15 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
     }
 
     for (cur = start; cur <= end; cur += subd_incr) {
-      pos = nround((cur - lower) * increment);
+      pos = nround((cur - lower) * increment) - ruler->ofst;
 
       if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
 	cairo_rectangle(cr,
-			pos, height + ythickness - length,
+			pos, ruler->size - length,
 			1, length);
       } else {
 	cairo_rectangle(cr,
-			height + xthickness - length, pos,
+			ruler->size - length, pos,
 			length, 1);
       }
 
@@ -567,13 +459,13 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
 	  pango_layout_set_text(layout, unit_str, -1);
 	  cairo_move_to(cr,
 			pos + 2,
-			ythickness + ofst);
+			ofst + 1);
 	  pango_cairo_show_layout(cr, layout);
 	} else {
 	  for (j = 0; j < len; j++) {
 	    pango_layout_set_text(layout, unit_str + j, 1);
 	    cairo_move_to(cr,
-			  xthickness + 1,
+			  1,
 			  pos + digit_height * j + 2 + ofst);
 	    pango_cairo_show_layout(cr, layout);
 	  }
@@ -592,6 +484,57 @@ nruler_draw_ticks(Nruler *ruler, GtkWidget *widget)
   g_object_unref(layout);
 }
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+static void
+nruler_get_color(Nruler *ruler, GdkRGBA *fg, GdkRGBA *bg)
+{
+  GdkRGBA color;
+  GtkStyleContext *style;
+
+  style = gtk_widget_get_style_context(TopLevel);
+  gtk_style_context_get_background_color(style, GTK_STATE_FLAG_NORMAL, &color);
+  color.alpha = 1.0;
+
+  if (bg) {
+    *bg = color;
+  }
+
+  if (fg) {
+#if 0
+    if (color.red + color.green + color.blue > 1.5) {
+      fg->red = 0.0;
+      fg->green = 0.0;
+      fg->blue = 0.0;
+    } else {
+      fg->red = 1.0;
+      fg->green = 1.0;
+      fg->blue = 1.0;
+    }
+    fg->alpha = 1.0;
+#else
+    gtk_style_context_get_color(style, GTK_STATE_FLAG_NORMAL, fg);
+#endif
+  }
+}
+#else
+static void
+nruler_get_color(Nruler *ruler, GdkColor *fg, GdkColor *bg)
+{
+  GtkStateType state;
+  GtkStyle *style;
+
+  state = gtk_widget_get_state(ruler->parent);
+  style = gtk_widget_get_style(ruler->parent);
+  if (fg) {
+    *fg = style->fg[state];
+  }
+
+  if (bg) {
+    *bg = style->bg[state];
+  }
+}
+#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
+
 static void
 #if GTK_CHECK_VERSION(3, 0, 0)
 nruler_draw_pos(Nruler *ruler, GtkWidget *widget, cairo_t *cr)
@@ -602,64 +545,35 @@ nruler_draw_pos(Nruler *ruler, GtkWidget *widget)
   gint x, y;
   gint width, height;
   gint bs_width, bs_height;
-  gint xthickness;
-  gint ythickness;
   gdouble increment;
   GtkAllocation allocation;
 #if GTK_CHECK_VERSION(3, 0, 0)
-  GtkStyleContext *style;
-  GdkRGBA color;
+  GdkRGBA fg;
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  GtkStyle *style;
+  GdkColor fg;
   cairo_t *cr;
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
-  GtkStateType state;
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-  style = gtk_widget_get_style_context(widget);
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  style = gtk_widget_get_style(widget);
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
-  state = gtk_widget_get_state(widget);
   gtk_widget_get_allocation(widget, &allocation);
 
   if (! gtk_widget_is_drawable(widget)) {
     return;
   }
 
-  width = allocation.width;
-  height = allocation.height;
-
-#if GTK_CHECK_VERSION(3, 0, 0)
-  get_thickness(widget, style, &xthickness, &ythickness);
 
   if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    height -= ythickness;
+    width = ruler->length;
+    height = ruler->size;
     bs_width = height / 2 + 2;
     bs_width |= 1;  /* make sure it's odd */
     bs_height = bs_width / 2 + 1;
   } else {
-    width -= xthickness;
+    width = ruler->size;
+    height = ruler->length;
     bs_height = width / 2 + 2;
     bs_height |= 1;  /* make sure it's odd */
     bs_width = bs_height / 2 + 1;
   }
-#else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  xthickness = style->xthickness;
-  ythickness = style->ythickness;
-
-  if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
-    height -= ythickness * 2;
-    bs_width = height / 2 + 2;
-    bs_width |= 1;  /* make sure it's odd */
-    bs_height = bs_width / 2 + 1;
-  } else {
-    width -= xthickness * 2;
-    bs_height = width / 2 + 2;
-    bs_height |= 1;  /* make sure it's odd */
-    bs_width = bs_height / 2 + 1;
-  }
-#endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 
   if (bs_width <= 0 || bs_height <= 0) {
     return;
@@ -683,20 +597,20 @@ nruler_draw_pos(Nruler *ruler, GtkWidget *widget)
   if (ruler->orientation == GTK_ORIENTATION_HORIZONTAL) {
     increment = (gdouble) width / (ruler->upper - ruler->lower);
 
-    x = nround((ruler->position - ruler->lower) * increment) - bs_width / 2;
-    y = (height + bs_height) / 2;
+    x = nround((ruler->position - ruler->lower) * increment) - bs_width / 2 - ruler->ofst;
+    y = (height + bs_height) / 2 - 1;
   } else {
     increment = (gdouble) height / (ruler->upper - ruler->lower);
 
-    x = (width + bs_width) / 2;
-    y = nround((ruler->position - ruler->lower) * increment) - bs_height / 2;
+    x = (width + bs_width) / 2 - 1;
+    y = nround((ruler->position - ruler->lower) * increment) - bs_height / 2 - ruler->ofst;
   }
 
+  nruler_get_color(ruler, &fg, NULL);
 #if GTK_CHECK_VERSION(3, 0, 0)
-  gtk_style_context_get_color(style, state, &color);
-  gdk_cairo_set_source_rgba(cr, &color);
+  gdk_cairo_set_source_rgba(cr, &fg);
 #else	/* GTK_CHECK_VERSION(3, 0, 0) */
-  gdk_cairo_set_source_color(cr, &style->fg[state]);
+  gdk_cairo_set_source_color(cr, &fg);
 #endif	/* GTK_CHECK_VERSION(3, 0, 0) */
 
   cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);
