@@ -199,7 +199,6 @@ get_parent_window(GtkWidget *w)
 }
 
 
-#if USE_ENTRY_ICON
 static void
 entry_icon_file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
@@ -222,9 +221,6 @@ entry_icon_file_select(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *eve
     g_free(file);
   }
 }
-#else
-GCallback entry_icon_file_select = NULL;
-#endif
 
 GtkWidget *
 create_file_entry_with_cb(GCallback cb, gpointer data)
@@ -233,10 +229,8 @@ create_file_entry_with_cb(GCallback cb, gpointer data)
 
   w = create_text_entry(TRUE, TRUE);
 
-#if USE_ENTRY_ICON
   gtk_entry_set_icon_from_stock(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_OPEN);
   g_signal_connect(w, "icon-release", cb, data);
-#endif
 
   return w;
 }
@@ -247,7 +241,6 @@ create_file_entry(struct objlist *obj)
   return create_file_entry_with_cb(G_CALLBACK(entry_icon_file_select), obj);
 }
 
-#if USE_ENTRY_ICON
 static void
 direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *event, gpointer user_data)
 {
@@ -284,14 +277,12 @@ direction_icon_released(GtkEntry *entry, GtkEntryIconPosition pos, GdkEvent *eve
 
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(entry), val);
 }
-#endif
 
 GtkWidget *
 create_direction_entry(void)
 {
   GtkWidget *w;
 
-#if USE_ENTRY_ICON
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_ANGLE, FALSE, TRUE);
 #if GTK_CHECK_VERSION(3, 4, 0)
   gtk_widget_set_size_request(w, NUM_ENTRY_WIDTH * 1.2, -1);
@@ -301,9 +292,6 @@ create_direction_entry(void)
   gtk_entry_set_icon_from_stock(GTK_ENTRY(w), GTK_ENTRY_ICON_SECONDARY, GTK_STOCK_GO_UP);
   gtk_entry_set_icon_from_stock(GTK_ENTRY(w), GTK_ENTRY_ICON_PRIMARY, GTK_STOCK_GO_DOWN);
   g_signal_connect(w, "icon-release", G_CALLBACK(direction_icon_released), NULL);
-#else
-  w = create_spin_entry_type(SPIN_BUTTON_TYPE_ANGLE, TRUE, TRUE);
-#endif
 
   return w;
 }
@@ -879,9 +867,13 @@ create_text_view_with_line_number(GtkWidget **v)
 {
   GtkWidget *view, *ln, *swin, *hs, *vs;
   GtkAdjustment *hadj, *vadj;
+  GtkTextBuffer *buf;
 
-  view = gtk_text_view_new_with_buffer(NULL);
-  ln = gtk_text_view_new_with_buffer(NULL);
+  buf = gtk_text_buffer_new(NULL);
+  view = gtk_text_view_new_with_buffer(buf);
+
+  buf = gtk_text_buffer_new(NULL);
+  ln = gtk_text_view_new_with_buffer(buf);
 
   set_linumber_color(ln, 0xCC00, 0xCC00, 0xCC00);
 
@@ -1031,3 +1023,179 @@ text_view_with_line_number_set_font(GtkWidget *view, const gchar *font)
 
   pango_font_description_free(desc);
 }
+
+enum SELECT_OBJ_COLOR_RESULT
+select_obj_color(struct objlist *obj, int id, enum OBJ_FIELD_COLOR_TYPE type)
+{
+  GtkWidget *dlg;
+  int r, g, b, a, rr ,gg, bb, aa, response;
+#if GTK_CHECK_VERSION(3, 4, 0)
+  GdkRGBA color;
+#else
+  GtkWidget *sel;
+  GdkColor color;
+#endif
+  char *title;
+
+  switch (type) {
+  case OBJ_FIELD_COLOR_TYPE_STROKE:
+    title = _("Stroke Color");
+    getobj(obj, "stroke_R", id, 0, NULL, &r);
+    getobj(obj, "stroke_G", id, 0, NULL, &g);
+    getobj(obj, "stroke_B", id, 0, NULL, &b);
+    getobj(obj, "stroke_A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_FILL:
+    title = _("Fill Color");
+    getobj(obj, "fill_R", id, 0, NULL, &r);
+    getobj(obj, "fill_G", id, 0, NULL, &g);
+    getobj(obj, "fill_B", id, 0, NULL, &b);
+    getobj(obj, "fill_A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_0:
+    title = _("Color");
+    getobj(obj, "R", id, 0, NULL, &r);
+    getobj(obj, "G", id, 0, NULL, &g);
+    getobj(obj, "B", id, 0, NULL, &b);
+    getobj(obj, "A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_1:
+    title = _("Color 1");
+    getobj(obj, "R", id, 0, NULL, &r);
+    getobj(obj, "G", id, 0, NULL, &g);
+    getobj(obj, "B", id, 0, NULL, &b);
+    getobj(obj, "A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_2:
+    title = _("Color 2");
+    getobj(obj, "R2", id, 0, NULL, &r);
+    getobj(obj, "G2", id, 0, NULL, &g);
+    getobj(obj, "B2", id, 0, NULL, &b);
+    getobj(obj, "A2", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_AXIS_BASE:
+    title = _("Axis baseline color");
+    getobj(obj, "R", id, 0, NULL, &r);
+    getobj(obj, "G", id, 0, NULL, &g);
+    getobj(obj, "B", id, 0, NULL, &b);
+    getobj(obj, "A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_AXIS_GAUGE:
+    title = _("Axis gauge color");
+    getobj(obj, "gauge_R", id, 0, NULL, &r);
+    getobj(obj, "gauge_G", id, 0, NULL, &g);
+    getobj(obj, "gauge_B", id, 0, NULL, &b);
+    getobj(obj, "gauge_A", id, 0, NULL, &a);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_AXIS_NUM:
+    title = _("Axis numbering color");
+    getobj(obj, "num_R", id, 0, NULL, &r);
+    getobj(obj, "num_G", id, 0, NULL, &g);
+    getobj(obj, "num_B", id, 0, NULL, &b);
+    getobj(obj, "num_A", id, 0, NULL, &a);
+    break;
+  default:
+    return SELECT_OBJ_COLOR_ERROR;
+  }
+
+  if (! Menulocal.use_opacity) {
+    a = 255;
+  }
+
+#if GTK_CHECK_VERSION(3, 4, 0)
+  color.red = r / 255.0;
+  color.green = g / 255.0;
+  color.blue = b / 255.0;
+  color.alpha = a / 255.0;
+
+  dlg = gtk_color_chooser_dialog_new(title, GTK_WINDOW(NgraphApp.FileWin.Win));
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(dlg), &color);
+  gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(dlg), Menulocal.use_opacity);
+
+  response = ndialog_run(dlg);
+  gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(dlg), &color);
+
+  gtk_widget_destroy(dlg);
+
+  if (response != GTK_RESPONSE_OK) {
+    return SELECT_OBJ_COLOR_CANCEL;
+  }
+
+  rr = nround(color.red * 255);
+  gg = nround(color.green * 255);
+  bb = nround(color.blue * 255);
+  aa = nround(color.alpha * 255);
+#else
+  color.red = (r & 0xffU) * 257;
+  color.green = (g & 0xffU) * 257;
+  color.blue = (b & 0xffU) * 257;
+
+  dlg = gtk_color_selection_dialog_new(title);
+  sel = gtk_color_selection_dialog_get_color_selection(GTK_COLOR_SELECTION_DIALOG(dlg));
+
+  gtk_color_selection_set_has_palette(GTK_COLOR_SELECTION(sel), TRUE);
+  gtk_color_selection_set_has_opacity_control(GTK_COLOR_SELECTION(sel), Menulocal.use_opacity);
+
+  gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(sel), &color);
+  gtk_color_selection_set_current_alpha(GTK_COLOR_SELECTION(sel), (a & 0xffU) * 257);
+
+  response = ndialog_run(dlg);
+  gtk_color_selection_get_current_color(GTK_COLOR_SELECTION(sel), &color);
+  aa = gtk_color_selection_get_current_alpha(GTK_COLOR_SELECTION(sel));
+  gtk_widget_destroy(dlg);
+
+  if (response != GTK_RESPONSE_OK) {
+    return SELECT_OBJ_COLOR_CANCEL;
+  }
+
+  rr = (color.red >> 8);
+  gg = (color.green >> 8);
+  bb = (color.blue >> 8);
+  aa >>= 8;
+#endif
+
+  switch (type) {
+  case OBJ_FIELD_COLOR_TYPE_STROKE:
+    putobj(obj, "stroke_R", id, &rr);
+    putobj(obj, "stroke_G", id, &gg);
+    putobj(obj, "stroke_B", id, &bb);
+    putobj(obj, "stroke_A", id, &aa);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_FILL:
+    putobj(obj, "fill_R", id, &rr);
+    putobj(obj, "fill_G", id, &gg);
+    putobj(obj, "fill_B", id, &bb);
+    putobj(obj, "fill_A", id, &aa);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_0:
+  case OBJ_FIELD_COLOR_TYPE_1:
+  case OBJ_FIELD_COLOR_TYPE_AXIS_BASE:
+    putobj(obj, "R", id, &rr);
+    putobj(obj, "G", id, &gg);
+    putobj(obj, "B", id, &bb);
+    putobj(obj, "A", id, &aa);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_2:
+    putobj(obj, "R2", id, &rr);
+    putobj(obj, "G2", id, &gg);
+    putobj(obj, "B2", id, &bb);
+    putobj(obj, "A2", id, &aa);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_AXIS_GAUGE:
+    putobj(obj, "gauge_R", id, &rr);
+    putobj(obj, "gauge_G", id, &gg);
+    putobj(obj, "gauge_B", id, &bb);
+    putobj(obj, "gauge_A", id, &aa);
+    break;
+  case OBJ_FIELD_COLOR_TYPE_AXIS_NUM:
+    putobj(obj, "num_R", id, &rr);
+    putobj(obj, "num_G", id, &gg);
+    putobj(obj, "num_B", id, &bb);
+    putobj(obj, "num_A", id, &aa);
+    break;
+  default:
+    return SELECT_OBJ_COLOR_ERROR;
+  }
+
+  return (rr == r && gg == g && bb == b && aa == a) ? SELECT_OBJ_COLOR_SAME : SELECT_OBJ_COLOR_DIFFERENT;
+}
