@@ -52,7 +52,7 @@ char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *CONFDIR, *LOCALEDIR, *PIXMAPDIR;
 #include "nconfig.h"
 #include "shell.h"
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
 #include <readline/history.h>
 #include "ox11menu.h"
@@ -63,7 +63,7 @@ char *DOCDIR, *NDATADIR, *ADDINDIR, *LIBDIR, *CONFDIR, *LOCALEDIR, *PIXMAPDIR;
 static char **attempt_shell_completion(char *text, int start, int end);
 #define HIST_SIZE 100
 #define HIST_FILE "shell_history"
-#endif	/* HAVE_LIBREADLINE */
+#endif	/* HAVE_READLINE_READLINE_H */
 
 #define SYSCONF "[Ngraph]"
 
@@ -111,6 +111,9 @@ void *addpath(void);
 void *addgra2gtk(void);
 void *addmenu(void);
 void *adddialog(void);
+#ifdef WINDOWS
+void *addgra2emf(void);
+#endif	/* WINDOWS */
 
 void resizeconsole(int col, int row);
 
@@ -190,6 +193,9 @@ static void * ( * obj_add_func_ary[]) (void) = {
   addtext,
   addmenu,
   adddialog,
+#ifdef WINDOWS
+  addgra2emf,
+#endif
 };
 
 int
@@ -229,12 +235,6 @@ printfconsole(char *fmt, ...)
   return len;
 }
 
-static int
-interruptconsole(void)
-{
-  return FALSE;
-}
-
 static int
 inputynconsole(const char *mes)
 {
@@ -275,28 +275,12 @@ displaystatusconsole(const char *str)
 {
 }
 
-static char *terminal = NULL;
-static struct savedstdio consolesave;
-static int consolefd[3];
-
 void
 resizeconsole(int col, int row)
 {
 }
 
-static void
-reset_fifo(char *fifo_in, char *fifo_out)
-{
-  int fdi, fdo;
-
-  fdi = nopen(fifo_in, O_WRONLY, 0);
-  fdo = nopen(fifo_out, O_RDONLY, 0);
-  if (fdi >= 0)
-    nclose(fdi);
-
-  if (fdo >= 0)
-    nclose(fdo);
-}
+static char *terminal = NULL;
 
 #ifdef WINDOWS
 static HWND ConsoleHandle = NULL;
@@ -304,7 +288,6 @@ static HWND ConsoleHandle = NULL;
 static int
 check_console(int allocconsole)
 {
-  HMENU menu;
   DWORD console_pid;
   pid_t pid;
 
@@ -378,6 +361,29 @@ resotre_console(void)
 }
 #else  /* WINDOWS */
 static pid_t consolepid = -1;
+static struct savedstdio consolesave;
+static int consolefd[3];
+
+static int
+interruptconsole(void)
+{
+  return FALSE;
+}
+
+static void
+reset_fifo(char *fifo_in, char *fifo_out)
+{
+  int fdi, fdo;
+
+  fdi = nopen(fifo_in, O_WRONLY, 0);
+  fdo = nopen(fifo_out, O_RDONLY, 0);
+  if (fdi >= 0)
+    nclose(fdi);
+
+  if (fdo >= 0)
+    nclose(fdo);
+}
+
 
 void
 hide_console(void)
@@ -605,7 +611,7 @@ nforegroundconsole()
 }
 
 static void
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
 load_config(struct objlist *sys, N_VALUE *inst, int *allocconsole, int *history_size)
 #else
 load_config(struct objlist *sys, N_VALUE *inst, int *allocconsole)
@@ -653,7 +659,7 @@ load_config(struct objlist *sys, N_VALUE *inst, int *allocconsole)
 	if (endptr[0] == '\0')
 	  consolerow = val;
 	g_free(f1);
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
       } else if (strcmp(tok, "history_size") == 0) {
 	f1 = getitok2(&s2, &len, " \t,");
 	val = strtol(f1, &endptr, 10);
@@ -753,7 +759,7 @@ main(int argc, char **argv)
   int allocnow, allocconsole = FALSE;
   struct narray iarray;
   char *arg;
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
   int history_size = HIST_SIZE;
   char *history_file = NULL;
 #endif
@@ -765,8 +771,6 @@ main(int argc, char **argv)
 #if ! GLIB_CHECK_VERSION(2, 32, 0)
   g_thread_init(NULL);
 #endif
-  gdk_threads_init();
-  gdk_threads_enter();
 
 #if EOF == -1
   char_type_buf_init();
@@ -902,7 +906,7 @@ main(int argc, char **argv)
   }
 
   loginshell = NULL;
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
   load_config(sys, inst, &allocconsole, &history_size);
   rl_readline_name = "ngraph";
   rl_completer_word_break_characters = " \t\n\"'@><;|&({}`";
@@ -960,23 +964,25 @@ main(int argc, char **argv)
     exit(1);
   }
 
-  for (i = 1; i < argc; i++) {
-    if (argv[i][0] != '-' || argv[i][1] != 'i' || i >= argc - 1) {
-      break;
-    }
+  i = 1;
+  if (argc > 1 && strcmp(argv[1], "-i") == 0) {
     i++;
-    inifile = g_strdup(argv[i]);
-    if (inifile == NULL) {
-      exit(1);
+    if (argc > 2) {
+      inifile = g_strdup(argv[2]);
+      if (inifile == NULL) {
+	exit(1);
+      }
+      changefilename(inifile);
+      i++;
+    } else {
+      inifile = NULL;
     }
-    changefilename(inifile);
-  }
-
-  if (inifile == NULL) {
-    if (findfilename(homedir, CONFTOP, INIT_SCRIPT))
+  } else {
+    if (findfilename(homedir, CONFTOP, INIT_SCRIPT)) {
       inifile = getfilename(homedir, CONFTOP, INIT_SCRIPT);
-    else if (findfilename(confdir, CONFTOP, INIT_SCRIPT))
+    } else if (findfilename(confdir, CONFTOP, INIT_SCRIPT)) {
       inifile = getfilename(confdir, CONFTOP, INIT_SCRIPT);
+    }
   }
 
   if (inifile) {
@@ -1031,7 +1037,7 @@ main(int argc, char **argv)
       exit(1);
     }
   } while (loginshell != NULL);
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
   if (history_file != NULL) {
     write_history(history_file);
     g_free(history_file);
@@ -1049,7 +1055,7 @@ main(int argc, char **argv)
   return 0;
 }
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
 struct mylist
 {
   struct mylist *next;
