@@ -75,14 +75,14 @@ static struct compatible_font_info CompatibleFont[] = {
   {"SymbolBold",           GRA_FONT_STYLE_BOLD,                         TRUE,  "Serif"},
   {"SymbolItalic",         GRA_FONT_STYLE_ITALIC,                       TRUE,  "Serif"},
   {"SymbolBoldItalic",     GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, TRUE,  "Serif"},
-  {"Mincho",               GRA_FONT_STYLE_NORMAL,                       FALSE, "Sans-serif"},
-  {"MinchoBold",           GRA_FONT_STYLE_BOLD,                         FALSE, "Sans-serif"},
-  {"MinchoItalic",         GRA_FONT_STYLE_ITALIC,                       FALSE, "Sans-serif"},
-  {"MinchoBoldItalic",     GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Sans-serif"},
-  {"Gothic",               GRA_FONT_STYLE_NORMAL,                       FALSE, "Serif"},
-  {"GothicBold",           GRA_FONT_STYLE_BOLD,                         FALSE, "Serif"},
-  {"GothicItalic",         GRA_FONT_STYLE_ITALIC,                       FALSE, "Serif"},
-  {"GothicBoldItalic",     GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Serif"},
+  {"Mincho",               GRA_FONT_STYLE_NORMAL,                       FALSE, "Serif"},
+  {"MinchoBold",           GRA_FONT_STYLE_BOLD,                         FALSE, "Serif"},
+  {"MinchoItalic",         GRA_FONT_STYLE_ITALIC,                       FALSE, "Serif"},
+  {"MinchoBoldItalic",     GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Serif"},
+  {"Gothic",               GRA_FONT_STYLE_NORMAL,                       FALSE, "Sans-serif"},
+  {"GothicBold",           GRA_FONT_STYLE_BOLD,                         FALSE, "Sans-serif"},
+  {"GothicItalic",         GRA_FONT_STYLE_ITALIC,                       FALSE, "Sans-serif"},
+  {"GothicBoldItalic",     GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Sans-serif"},
   {"Tim",                  GRA_FONT_STYLE_NORMAL,                       FALSE, "Serif"},
   {"TimB",                 GRA_FONT_STYLE_BOLD,                         FALSE, "Serif"},
   {"TimI",                 GRA_FONT_STYLE_ITALIC,                       FALSE, "Serif"},
@@ -103,14 +103,14 @@ static struct compatible_font_info CompatibleFont[] = {
   {"SymB",                 GRA_FONT_STYLE_BOLD,                         TRUE,  "Serif"},
   {"SymI",                 GRA_FONT_STYLE_ITALIC,                       TRUE,  "Serif"},
   {"SymBI",                GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, TRUE,  "Serif"},
-  {"Min",                  GRA_FONT_STYLE_NORMAL,                       FALSE, "Sans-serif"},
-  {"MinB",                 GRA_FONT_STYLE_BOLD,                         FALSE, "Sans-serif"},
-  {"MinI",                 GRA_FONT_STYLE_ITALIC,                       FALSE, "Sans-serif"},
-  {"MinBI",                GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Sans-serif"},
-  {"Goth",                 GRA_FONT_STYLE_NORMAL,                       FALSE, "Serif"},
-  {"GothB",                GRA_FONT_STYLE_BOLD,                         FALSE, "Serif"},
-  {"GothI",                GRA_FONT_STYLE_ITALIC,                       FALSE, "Serif"},
-  {"GothBI",               GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Serif"},
+  {"Min",                  GRA_FONT_STYLE_NORMAL,                       FALSE, "Serif"},
+  {"MinB",                 GRA_FONT_STYLE_BOLD,                         FALSE, "Serif"},
+  {"MinI",                 GRA_FONT_STYLE_ITALIC,                       FALSE, "Serif"},
+  {"MinBI",                GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Serif"},
+  {"Goth",                 GRA_FONT_STYLE_NORMAL,                       FALSE, "Sans-serif"},
+  {"GothB",                GRA_FONT_STYLE_BOLD,                         FALSE, "Sans-serif"},
+  {"GothI",                GRA_FONT_STYLE_ITALIC,                       FALSE, "Sans-serif"},
+  {"GothBI",               GRA_FONT_STYLE_BOLD | GRA_FONT_STYLE_ITALIC, FALSE, "Sans-serif"},
 };
 
 #define FONT_FACE_NUM ((int) (sizeof(FontFaceStr) / sizeof(*FontFaceStr)))
@@ -185,8 +185,25 @@ free_font_map(struct fontmap *fcur)
   g_free(fcur);
 }
 
+static void
+add_font_map(struct fontmap *fmap)
+{
+  struct fontmap *cur;
+
+  if (Gra2cairoConf->fontmap_list_root == NULL) {
+    Gra2cairoConf->fontmap_list_root = fmap;
+    return;
+  }
+
+  cur = Gra2cairoConf->fontmap_list_root;
+  while (cur->next) {
+    cur = cur->next;
+  }
+  cur->next = fmap;
+}
+
 static struct fontmap *
-create_font_map(const char *fontalias, const char *fontname, const char *alternative, struct fontmap *fprev)
+create_font_map(const char *fontalias, const char *fontname, const char *alternative)
 {
   struct fontmap *fnew;
 
@@ -212,14 +229,7 @@ create_font_map(const char *fontalias, const char *fontname, const char *alterna
   fnew->alternative = g_strdup(alternative);
   fnew->font = NULL;
   fnew->next = NULL;
-
-  if (fprev) {
-    fprev->next = fnew;
-  } else {
-    fnew->next = Gra2cairoConf->fontmap_list_root;
-    Gra2cairoConf->fontmap_list_root = fnew;
-  }
-
+  add_font_map(fnew);
   Gra2cairoConf->font_num++;
 
   return fnew;
@@ -276,13 +286,12 @@ loadconfig(void)
   char *tok, *str, *s2;
   char *f1, *f2, *f3;
   int len;
-  struct fontmap *fnew, *fprev;
+  struct fontmap *fnew;
 
   fp = openconfig(CAIROCONF);
   if (fp == NULL)
     return 0;
 
-  fprev = NULL;
   while ((tok = getconfig(fp, &str))) {
     s2 = str;
     if (strcmp(tok, "font") == 0) {
@@ -291,7 +300,7 @@ loadconfig(void)
       for (; (s2[0] != '\0') && (strchr(" \x09,", s2[0])); s2++);
       f3 = getitok2(&s2, &len, "");
       if (f1 && f2) {
-	fnew = create_font_map(f1, f2, f3, fprev);
+	fnew = create_font_map(f1, f2, f3);
 	g_free(f1);
 	g_free(f2);
 	if (fnew == NULL) {
@@ -299,8 +308,6 @@ loadconfig(void)
 	  closeconfig(fp);
 	  return 1;
 	}
-
-	fprev = fnew;
       } else {
 	g_free(f1);
 	g_free(f2);
@@ -331,7 +338,7 @@ loadconfig(void)
 	    g_free(f1);
 	    f1 = g_strdup(info->name);
 	    if (f1) {
-	      fnew = create_font_map(f1, f2, NULL, fprev);
+	      fnew = create_font_map(f1, f2, NULL);
 	    }
 	    if (fnew == NULL) {
 	      g_free(tok);
@@ -340,7 +347,6 @@ loadconfig(void)
 	      closeconfig(fp);
 	      return 1;
 	    }
-	    fprev = fnew;
 	  } else {
 	    if (two_byte) {
 	      gra2cairo_set_alternative_font(info->name, f2);
@@ -421,7 +427,7 @@ free_conf(void)
 }
 
 struct fontmap *
-gra2cairo_get_fontmap(char *fontalias)
+gra2cairo_get_fontmap(const char *fontalias)
 {
   struct fontmap *fnew;
 
@@ -443,7 +449,7 @@ gra2cairo_get_fontmap_num(void)
 }
 
 void
-gra2cairo_remove_fontmap(char *fontalias)
+gra2cairo_remove_fontmap(const char *fontalias)
 {
   struct fontmap *fnew;
 
@@ -483,7 +489,7 @@ gra2cairo_add_fontmap(const char *fontalias, const char *fontname)
   if (nhash_get_ptr(Gra2cairoConf->fontmap, fontalias, (void *) &fnew) == 0) {
     free_font_map(fnew);
   }
-  create_font_map(fontalias, fontname, NULL, NULL);
+  create_font_map(fontalias, fontname, NULL);
 }
 
 void
@@ -569,7 +575,7 @@ gra2cairo_init(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
 void
 gra2cairo_draw_path(struct gra2cairo_local *local)
 {
-  if (local->linetonum) {
+  if (local->cairo && local->linetonum) {
     double x, y;
 
     cairo_get_current_point(local->cairo, &x, &y);
@@ -749,7 +755,7 @@ set_antialias(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
 }
 
 struct compatible_font_info *
-gra2cairo_get_compatible_font_info(char *name)
+gra2cairo_get_compatible_font_info(const char *name)
 {
   int i;
 
@@ -934,6 +940,7 @@ static int
 gra2cairo_flush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct gra2cairo_local *local;
+  cairo_surface_t *surface;
 
   _getobj(obj, "_local", inst, &local);
 
@@ -942,16 +949,72 @@ gra2cairo_flush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, cha
 
   gra2cairo_draw_path(local);
 
+  surface = cairo_get_target(local->cairo);
+  if (surface) {
+    cairo_surface_flush(surface);
+  }
+
+
   return check_cairo_status(local->cairo);
 }
 
+char *
+gra2cairo_get_utf8_str(const char *cstr, int symbol)
+{
+  char *tmp;
+  size_t l, i, j;
+
+  l = strlen(cstr);
+  tmp = g_malloc(l * 6 + 1);
+  if (tmp == NULL) {
+    return NULL;
+  }
+
+  for (j = i = 0; i <= l; i++) {
+    if (cstr[i] == '\\') {
+      if (cstr[i + 1] == 'x' &&
+	  g_ascii_isxdigit(cstr[i + 2]) &&
+	  g_ascii_isxdigit(cstr[i + 3])) {
+	char buf[8];
+	int len, k;
+	gunichar wc;
+
+	wc = g_ascii_xdigit_value(cstr[i + 2]) * 16 + g_ascii_xdigit_value(cstr[i + 3]);
+	len = g_unichar_to_utf8(wc, buf);
+	for (k = 0; k < len; k++) {
+	  tmp[j++] = buf[k];
+	}
+	i += 3;
+      } else {
+	i += 1;
+	tmp[j++] = cstr[i];
+      }
+    } else {
+      tmp[j++] = cstr[i];
+    }
+    tmp[j] = '\0';
+  }
+
+  if (symbol) {
+    char *ptr;
+
+    ptr = ascii2greece(tmp);
+    if (ptr) {
+      g_free(tmp);
+      tmp = ptr;
+    }
+  }
+
+  return tmp;
+}
+
 static int
 gra2cairo_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
                  int argc, char **argv)
 {
-  char code, *cstr, *tmp, *tmp2;
-  int *cpar, i, j, r, font_style;
-  double x, y, w, h, l, fontsize,
+  char code, *cstr, *tmp;
+  int *cpar, i, r, font_style;
+  double x, y, w, h, fontsize,
     fontspace, fontdir, fontsin, fontcos, a1, a2;
   cairo_line_join_t join;
   cairo_line_cap_t cap;
@@ -1204,57 +1267,21 @@ gra2cairo_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval,
     if (local->loadfont == NULL)
       break;
 
-    l = strlen(cstr);
-    tmp = g_malloc(l * 6 + 1);
-    if (tmp == NULL)
-      break;
-
-    for (j = i = 0; i <= l; i++) {
-      if (cstr[i] == '\\') {
-	if (cstr[i + 1] == 'x' &&
-	    g_ascii_isxdigit(cstr[i + 2]) &&
-	    g_ascii_isxdigit(cstr[i + 3])) {
-	  char buf[8];
-	  int len, k;
-	  gunichar wc;
-
-	  wc = g_ascii_xdigit_value(cstr[i + 2]) * 16 + g_ascii_xdigit_value(cstr[i + 3]);
-	  len = g_unichar_to_utf8(wc, buf);
-	  for (k = 0; k < len; k++) {
-	    tmp[j++] = buf[k];
-	  }
-	  i += 3;
-	} else {
-	  i += 1;
-	  tmp[j++] = cstr[i];
-	}
-      } else {
-	tmp[j++] = cstr[i];
-      }
-      tmp[j] = '\0';
+    tmp = gra2cairo_get_utf8_str(cstr, local->symbol);
+    if (tmp) {
+      draw_str(local, TRUE, tmp, local->loadfont, local->fontsize, local->fontspace, NULL, NULL, NULL);
+      g_free(tmp);
     }
-
-    if (local->symbol) {
-      char *ptr;
-
-      ptr = ascii2greece(tmp);
-      if (ptr) {
-	g_free(tmp);
-	tmp = ptr;
-      }
-    }
-    draw_str(local, TRUE, tmp, local->loadfont, local->fontsize, local->fontspace, NULL, NULL, NULL);
-    g_free(tmp);
     break;
   case 'K':
     if (local->loadfont == NULL)
       break;
 
-    tmp2 = sjis_to_utf8(cstr);
-    if (tmp2 == NULL)
-      break;
-    draw_str(local, TRUE, tmp2, local->loadfont, local->fontsize, local->fontspace, NULL, NULL, NULL);
-    g_free(tmp2);
+    tmp = sjis_to_utf8(cstr);
+    if (tmp) {
+      draw_str(local, TRUE, tmp, local->loadfont, local->fontsize, local->fontspace, NULL, NULL, NULL);
+      g_free(tmp);
+    }
     break;
   default:
     break;
@@ -1430,59 +1457,26 @@ void *
 addgra2cairo()
 /* addgra2cairoile() returns NULL on error */
 {
-  int errcode[] = {
-    CAIRO_STATUS_SUCCESS,
-    CAIRO_STATUS_NO_MEMORY,
-    CAIRO_STATUS_INVALID_RESTORE,
-    CAIRO_STATUS_INVALID_POP_GROUP,
-    CAIRO_STATUS_NO_CURRENT_POINT,
-    CAIRO_STATUS_INVALID_MATRIX,
-    CAIRO_STATUS_INVALID_STATUS,
-    CAIRO_STATUS_NULL_POINTER,
-    CAIRO_STATUS_INVALID_STRING,
-    CAIRO_STATUS_INVALID_PATH_DATA,
-    CAIRO_STATUS_READ_ERROR,
-    CAIRO_STATUS_WRITE_ERROR,
-    CAIRO_STATUS_SURFACE_FINISHED,
-    CAIRO_STATUS_SURFACE_TYPE_MISMATCH,
-    CAIRO_STATUS_PATTERN_TYPE_MISMATCH,
-    CAIRO_STATUS_INVALID_CONTENT,
-    CAIRO_STATUS_INVALID_FORMAT,
-    CAIRO_STATUS_INVALID_VISUAL,
-    CAIRO_STATUS_FILE_NOT_FOUND,
-    CAIRO_STATUS_INVALID_DASH,
-    CAIRO_STATUS_INVALID_DSC_COMMENT,
-    CAIRO_STATUS_INVALID_INDEX,
-    CAIRO_STATUS_CLIP_NOT_REPRESENTABLE,
-#ifdef CAIRO_STATUS_TEMP_FILE_ERROR
-    CAIRO_STATUS_TEMP_FILE_ERROR,
-#endif
-#ifdef CAIRO_STATUS_INVALID_STRIDE
-    CAIRO_STATUS_INVALID_STRIDE,
-#endif
-  };
-  unsigned int i, n;
+  int i;
 
   if (CompatibleFontHash == NULL) {
     CompatibleFontHash = nhash_new();
     if (CompatibleFontHash == NULL) {
       return NULL;
     }
-    for (i = 0; i < sizeof(CompatibleFont) / sizeof(*CompatibleFont); i++) {
+    for (i = 0; i < (int) (sizeof(CompatibleFont) / sizeof(*CompatibleFont)); i++) {
       nhash_set_int(CompatibleFontHash, CompatibleFont[i].old_name, i);
     }
   }
 
   if (Gra2CairoErrMsgs == NULL) {
-    n = sizeof(errcode) / sizeof(*errcode);
+    Gra2CairoErrMsgs = g_malloc(sizeof(*Gra2CairoErrMsgs) * CAIRO_STATUS_LAST_STATUS);
+    Gra2CairoErrMsgNum = CAIRO_STATUS_LAST_STATUS;
 
-    Gra2CairoErrMsgs = g_malloc(sizeof(*Gra2CairoErrMsgs) * n);
-    Gra2CairoErrMsgNum = n;
-
-    for (i = 0; i < n; i++) {
-      Gra2CairoErrMsgs[i] = g_strdup(cairo_status_to_string(errcode[i]));
+    for (i = 0; i < CAIRO_STATUS_LAST_STATUS; i++) {
+      Gra2CairoErrMsgs[i] = g_strdup(cairo_status_to_string(i));
     }
   }
 
-  return addobject(NAME, NULL, PARENT, OVERSION, TBLNUM, gra2cairo, n, Gra2CairoErrMsgs, NULL, NULL);
+  return addobject(NAME, NULL, PARENT, OVERSION, TBLNUM, gra2cairo, Gra2CairoErrMsgNum, Gra2CairoErrMsgs, NULL, NULL);
 }
