@@ -68,18 +68,30 @@ static char *gtkerrorlist[] = {
 
 #define ERRNUM (sizeof(gtkerrorlist)/sizeof(*gtkerrorlist))
 
+struct action_type
+{
+  enum {
+    ACTION_TYPE_NONE,
+    ACTION_TYPE_KEY,
+    ACTION_TYPE_BUTTON,
+    ACTION_TYPE_SCROLL,
+  } type;
+  int val;
+};
 
 struct gtklocal
 {
   struct objlist *obj;
+  GdkCursor *blank_cursor;
   N_VALUE *inst;
   GtkWidget *mainwin, *View;
   cairo_surface_t *surface;
   GdkWindow *window;
   char *title;
-  int redraw;
+  int redraw, fit, frame;
   unsigned int windpi;
   int PaperWidth, PaperHeight;
+  struct action_type action;
   double bg_r, bg_g, bg_b;
   struct gra2cairo_local *local;
 };
@@ -199,11 +211,13 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
 
   e = (GdkEventKey *)event;
 
+  gtklocal->action.type = ACTION_TYPE_KEY;
+  gtklocal->action.val = e->keyval;
+
   switch (e->keyval) {
   case GDK_KEY_w:
     if (e->state & GDK_CONTROL_MASK) {
-      gtk_widget_destroy(gtklocal->mainwin);
-      gtklocal->mainwin = NULL;
+      gtkclose(w, NULL, NULL);
       return TRUE;
     }
     return FALSE;
@@ -211,12 +225,110 @@ ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   return FALSE;
 }
 
+void
+size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data)
+{
+  struct gtklocal *local;
+  double w, h, rh, rw, ratio;
+  int pw, ph, dpi;
+
+  local = (struct gtklocal *) user_data;
+  if (local == NULL) {
+    return;
+  }
+
+  if (! local->fit) {
+    return;
+  }
+
+  w = allocation->width;
+  h = allocation->height;
+
+  pw = local->PaperWidth;
+  ph = local->PaperHeight;
+
+  if (pw < 1 || ph < 1) {
+    return;
+  }
+
+  rw = w / pw;
+  rh = h / ph;
+
+  ratio = (rh > rw) ? rw : rh;
+  dpi = ratio * DPI_MAX;
+
+  local->windpi = dpi;
+  local->local->pixel_dot_x =
+    local->local->pixel_dot_y = ratio;
+
+  _putobj(local->obj, "dpi", local->inst, &dpi);
+
+  gtkchangedpi(local);
+
+}
+
+static gboolean
+cursor_moved(GtkWidget *widget, GdkEvent  *event, gpointer user_data)
+{
+  struct gtklocal *gtklocal;
+
+  gtklocal = (struct gtklocal *) user_data;
+
+  if (gtklocal->blank_cursor) {
+    gdk_window_set_cursor(gtk_widget_get_window(gtklocal->mainwin), NULL);
+#if GTK_CHECK_VERSION(3, 0, 0)
+    g_object_unref(gtklocal->blank_cursor);
+#else
+    gdk_cursor_unref(gtklocal->blank_cursor);
+#endif
+    gtklocal->blank_cursor = NULL;
+  }
+
+  return FALSE;
+}
+
+static gboolean
+button_released(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
+{
+  struct gtklocal *gtklocal;
+
+  gtklocal = (struct gtklocal *) user_data;
+
+  gtklocal->action.type = ACTION_TYPE_BUTTON;
+  gtklocal->action.val = event->button;
+
+  return FALSE;
+}
+
+static gboolean
+scrolled(GtkWidget *widget, GdkEventScroll *event, gpointer user_data)
+{
+  struct gtklocal *gtklocal;
+
+  gtklocal = (struct gtklocal *) user_data;
+
+  switch (event->direction) {
+  case GDK_SCROLL_UP:
+  case GDK_SCROLL_DOWN:
+  case GDK_SCROLL_LEFT:
+  case GDK_SCROLL_RIGHT:
+    gtklocal->action.type = ACTION_TYPE_SCROLL;
+    gtklocal->action.val = event->direction;
+    break;
+#if GTK_CHECK_VERSION(3, 4, 0)
+  case GDK_SCROLL_SMOOTH:
+    break;
+#endif
+  }
+
+  return FALSE;
+}
+
 static int
 gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
   struct gtklocal *gtklocal;
   struct gra2cairo_local *local;
-  GdkWindow *win;
   struct objlist *robj;
   int idn, oid, width, height;
   GtkWidget *scrolled_window = NULL;
@@ -232,6 +344,11 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtklocal->inst = inst;
   gtklocal->title = NULL;
   gtklocal->mainwin = NULL;
+  gtklocal->fit = FALSE;
+  gtklocal->frame = TRUE;
+  gtklocal->blank_cursor = NULL;
+  gtklocal->action.type = ACTION_TYPE_NONE;
+  gtklocal->action.val = 0;
 
   if (_putobj(obj, "_gtklocal", inst, gtklocal))
     goto errexit;
@@ -242,6 +359,12 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   if (_getobj(obj, "_local", inst, &local))
     goto errexit;
 
+  if (_getobj(obj, "fit", inst, &gtklocal->fit))
+    goto errexit;
+
+  if (_getobj(obj, "frame", inst, &gtklocal->frame))
+    goto errexit;
+
   gtklocal->PaperWidth = 0;
   gtklocal->PaperHeight = 0;
   gtklocal->windpi = DEFAULT_DPI / 2;
@@ -299,6 +422,10 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   gtk_container_add(GTK_CONTAINER(gtklocal->mainwin), scrolled_window);
 
   gtklocal->View = gtk_drawing_area_new();
+#if GTK_CHECK_VERSION(3, 0, 0)
+  gtk_widget_set_halign(gtklocal->View, GTK_ALIGN_CENTER);
+  gtk_widget_set_valign(gtklocal->View, GTK_ALIGN_CENTER);
+#endif
 
 #if GTK_CHECK_VERSION(3, 0, 0)
   g_signal_connect(gtklocal->View, "draw",
@@ -308,17 +435,26 @@ gtkinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 		   G_CALLBACK(gtkevpaint), gtklocal);
 #endif
 
+  g_signal_connect(gtklocal->mainwin, "size-allocate",
+		   G_CALLBACK(size_allocate), gtklocal);
+
+#if GTK_CHECK_VERSION(3, 8, 0)
+  gtk_container_add(GTK_CONTAINER(scrolled_window), gtklocal->View);
+#else
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW
 					(scrolled_window), gtklocal->View);
+#endif
 
   gtk_widget_show_all(gtklocal->mainwin);
 
-  win = gtk_widget_get_window(gtklocal->View);
-
   gtklocal->surface= NULL;
-  gtklocal->window = win;
   gtklocal->redraw = TRUE;
 
+  gtk_widget_add_events(gtklocal->mainwin, GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_PRESS_MASK);
+  g_signal_connect(gtklocal->mainwin, "motion-notify-event", G_CALLBACK(cursor_moved), gtklocal);
+  g_signal_connect(gtklocal->mainwin, "button-release-event", G_CALLBACK(button_released), gtklocal);
+  g_signal_connect(gtklocal->mainwin, "scroll-event", G_CALLBACK(scrolled), gtklocal);
+
   if (chkobjfield(obj, "_evloop")) {
     goto errexit;
   }
@@ -366,8 +502,17 @@ gtkdone(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
   if (_getobj(obj, "_gtklocal", inst, &gtklocal))
     return 1;
 
+  if (gtklocal->blank_cursor) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    g_object_unref(gtklocal->blank_cursor);
+#else
+    gdk_cursor_unref(gtklocal->blank_cursor);
+#endif
+  }
+
   if (gtklocal->mainwin != NULL) {
     gtk_widget_destroy(gtklocal->mainwin);
+    gtklocal->mainwin = NULL;
     while (gtk_events_pending()) {
       gtk_main_iteration();
     }
@@ -397,6 +542,23 @@ clear_pixmap(struct gtklocal *local)
   cairo_destroy(cr);
 }
 
+static void
+redraw_window(struct gtklocal *local)
+{
+  GdkWindow *win;
+
+  if (local->View == NULL) {
+    return;
+  }
+
+  win = gtk_widget_get_window(local->View);
+  if (win == NULL) {
+    return;
+  }
+
+  gdk_window_invalidate_rect(win, NULL, TRUE);
+}
+
 static int
 gtkclear(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -409,8 +571,7 @@ gtkclear(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     return 1;
 
   clear_pixmap(local);
-
-  gdk_window_invalidate_rect(local->window, NULL, TRUE);
+  redraw_window(local);
 
   return 0;
 }
@@ -433,6 +594,29 @@ gtkpresent(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   return 0;
 }
 
+static int
+gtkfullscreen(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  struct gtklocal *local;
+  int state;
+
+  if (_exeparent(obj, argv[1], inst, rval, argc, argv))
+    return 1;
+
+  if (_getobj(obj, "_gtklocal", inst, &local))
+    return 1;
+
+  state = *(int *) argv[2];
+
+  if (state) {
+    gtk_window_fullscreen(GTK_WINDOW(local->mainwin));
+  } else {
+    gtk_window_unfullscreen(GTK_WINDOW(local->mainwin));
+  }
+
+  return 0;
+}
+
 static int
 gtkflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -444,7 +628,8 @@ gtkflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   if (_getobj(obj, "_gtklocal", inst, &local))
     return 1;
 
-  gdk_window_invalidate_rect(local->window, NULL, TRUE);
+  redraw_window(local);
+
   return 0;
 }
 
@@ -509,7 +694,7 @@ gtkredraw(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **ar
   if (_getobj(obj, "_gtklocal", inst, &gtklocal))
     return 1;
 
-  gdk_window_invalidate_rect(gtklocal->window, NULL, TRUE);
+  redraw_window(gtklocal);
   return 0;
 }
 
@@ -550,7 +735,7 @@ gtkchangedpi(struct gtklocal *gtklocal)
     pw = cairo_image_surface_get_width(pixmap);
 
     if (pw != width || ph != height) {
-      g_object_unref(G_OBJECT(pixmap));
+      cairo_surface_destroy(pixmap);
       pixmap = NULL;
     }
   }
@@ -581,7 +766,7 @@ gtkMakeRuler(cairo_t *cr, struct gtklocal *gtklocal)
   width = gtklocal->PaperWidth;
   height = gtklocal->PaperHeight;
 
-  if (width == 0 || height == 0 || gtklocal->surface == NULL)
+  if (width == 0 || height == 0 || gtklocal->surface == NULL || ! gtklocal->frame)
     return;
 
   cairo_save(cr);
@@ -627,13 +812,17 @@ gtk_output(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
 static int
 gtk_set_dpi(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
-  int dpi;
+  int dpi, fit;
   struct gtklocal *local;
 
   dpi = abs(*(int *) argv[2]);
 
   _getobj(obj, "_gtklocal", inst, &local);
 
+  fit = FALSE;
+  local->fit = FALSE;
+  _putobj(obj, "fit", inst, &fit);
+
   if (dpi < 1)
     dpi = 1;
   if (dpi > DPI_MAX)
@@ -649,6 +838,39 @@ gtk_set_dpi(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **
   return 0;
 }
 
+static int
+gtk_set_fit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  int fit;
+  struct gtklocal *local;
+
+  _getobj(obj, "_gtklocal", inst, &local);
+
+  fit = *(int *) argv[2];
+
+  local->fit = fit;
+
+  if (fit) {
+    GtkAllocation allocation;
+    gtk_widget_get_allocation(local->mainwin, &allocation);
+    size_allocate(local->mainwin, &allocation, local);
+  }
+
+  return 0;
+}
+
+static int
+gtk_set_frame(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  struct gtklocal *local;
+
+  _getobj(obj, "_gtklocal", inst, &local);
+
+  local->frame = *(int *) argv[2];
+
+  return 0;
+}
+
 static int
 gtk_set_size(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
@@ -678,6 +900,66 @@ gtk_set_size(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char *
   return 0;
 }
 
+static int
+gtkwait_action(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  struct gtklocal *local;
+  char *name;
+
+  _getobj(obj, "_gtklocal", inst, &local);
+
+  g_free(rval->str);
+  rval->str = NULL;
+
+  if (local->mainwin == NULL) {
+    return 1;
+  }
+
+  if (local->blank_cursor == NULL) {
+    local->blank_cursor = gdk_cursor_new(GDK_BLANK_CURSOR);
+    gdk_window_set_cursor(gtk_widget_get_window(local->mainwin), local->blank_cursor);
+  }
+
+  local->action.type = ACTION_TYPE_NONE;
+  local->action.val = 0;
+  while (local->action.type == ACTION_TYPE_NONE) {
+    gtk_evloop(NULL, NULL, NULL, 0, NULL);
+    msleep(100);
+  }
+
+  switch (local->action.type) {
+  case ACTION_TYPE_NONE:
+    break;
+  case ACTION_TYPE_KEY:
+    name = gdk_keyval_name(local->action.val);
+    if (name) {
+      rval->str = g_strdup(name);
+    }
+    break;
+  case ACTION_TYPE_BUTTON:
+    rval->str = g_strdup_printf("button%d", local->action.val);
+    break;
+  case ACTION_TYPE_SCROLL:
+    switch (local->action.val) {
+    case GDK_SCROLL_UP:
+      rval->str = g_strdup("scroll_up");
+      break;
+    case GDK_SCROLL_DOWN:
+      rval->str = g_strdup("scroll_down");
+      break;
+    case GDK_SCROLL_LEFT:
+      rval->str = g_strdup("scroll_left");
+      break;
+    case GDK_SCROLL_RIGHT:
+      rval->str = g_strdup("scroll_right");
+      break;
+    }
+    break;
+  }
+
+  return 0;
+}
+
 static struct objtable gra2gtk[] = {
   {"init", NVFUNC, NEXEC, gtkinit, NULL, 0},
   {"done", NVFUNC, NEXEC, gtkdone, NULL, 0},
@@ -687,13 +969,17 @@ static struct objtable gra2gtk[] = {
   {"dpiy", NINT, NREAD | NWRITE, gtk_set_dpi, NULL, 0},
   {"width", NINT, NREAD | NWRITE, gtk_set_size, NULL, 0},
   {"height", NINT, NREAD | NWRITE, gtk_set_size, NULL, 0},
+  {"fit", NBOOL, NREAD | NWRITE, gtk_set_fit, NULL, 0},
+  {"frame", NBOOL, NREAD | NWRITE, gtk_set_frame, NULL, 0},
   {"expose", NVFUNC, NREAD | NEXEC, gtkredraw, "", 0},
   {"flush", NVFUNC, NREAD | NEXEC, gtkflush, "", 0},
   {"clear", NVFUNC, NREAD | NEXEC, gtkclear, "", 0},
   {"present", NVFUNC, NREAD | NEXEC, gtkpresent, "", 0},
+  {"fullscreen", NVFUNC, NREAD | NEXEC, gtkfullscreen, "i", 0},
   {"BR", NINT, NREAD | NWRITE, gtkbr, NULL, 0},
   {"BG", NINT, NREAD | NWRITE, gtkbg, NULL, 0},
   {"BB", NINT, NREAD | NWRITE, gtkbb, NULL, 0},
+  {"wait_action",NSFUNC, NREAD | NEXEC, gtkwait_action, "", 0},
   {"_gtklocal", NPOINTER, 0, NULL, NULL, 0},
   {"_output", NVFUNC, 0, gtk_output, NULL, 0},
   {"_strwidth", NIFUNC, 0, gra2cairo_strwidth, NULL, 0},
