@@ -39,6 +39,7 @@
 
 #include "ngraph.h"
 #include "object.h"
+#include "odraw.h"
 #include "ioutil.h"
 #include "shell.h"
 #include "nstring.h"
@@ -154,7 +155,7 @@ static struct child_win_stat menu_config_axis_geometry = {
 };
 
 static struct child_win_stat menu_config_legend_geometry = {
-  (struct SubWin *) &NgraphApp.LegendWin,
+  &NgraphApp.LegendWin,
   {
     &Menulocal.legendx,
     &Menulocal.legendy,
@@ -176,7 +177,7 @@ static struct child_win_stat menu_config_merge_geometry = {
 };
 
 static struct child_win_stat menu_config_dialog_geometry = {
-  (struct SubWin *) &NgraphApp.InfoWin,
+  &NgraphApp.InfoWin,
   {
     &Menulocal.dialogx,
     &Menulocal.dialogy,
@@ -187,7 +188,7 @@ static struct child_win_stat menu_config_dialog_geometry = {
 };
 
 static struct child_win_stat menu_config_coord_geometry = {
-  (struct SubWin * ) &NgraphApp.CoordWin,
+  &NgraphApp.CoordWin,
   {
     &Menulocal.coordx,
     &Menulocal.coordy,
@@ -208,7 +209,6 @@ static struct menu_config MenuConfig[] = {
   {"script_console",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.scriptconsole},
   {"addin_console",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.addinconsole},
   {"show_tip",			MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.showtip},
-  {"expand_to_fullpath",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.expandtofullpath},
   {"character_map",		MENU_CONFIG_TYPE_CHARMAP, menu_config_set_char_map, &Menulocal.char_map},
   {NULL},
 };
@@ -224,23 +224,24 @@ static struct menu_config MenuConfigScript[] = {
 };
 
 static struct menu_config MenuConfigMisc[] = {
-  {"editor",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.editor},
-  {"browser",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.browser},
-  {"help_browser",	MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.help_browser},
-  {"coordwin_font",	MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.coordwin_font},
-  {"infowin_font",	MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.infowin_font},
-  {"file_preview_font",	MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.file_preview_font},
-  {"change_directory",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.changedirectory},
-  {"save_path",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savepath},
-  {"save_with_data",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savewithdata},
-  {"save_with_merge",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savewithmerge},
-  {"expand_dir",	MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.expanddir},
-  {"expand",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.expand},
-  {"ignore_path",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ignorepath},
-  {"history_size",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.hist_size},
-  {"infowin_size",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.info_size},
-  {"data_head_lines",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.data_head_lines},
-  {"use_opacity",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.use_opacity},
+  {"editor",			MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.editor},
+  {"browser",			MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.browser},
+  {"help_browser",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.help_browser},
+  {"coordwin_font",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.coordwin_font},
+  {"infowin_font",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.infowin_font},
+  {"file_preview_font",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.file_preview_font},
+  {"change_directory",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.changedirectory},
+  {"save_path",			MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savepath},
+  {"save_with_data",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savewithdata},
+  {"save_with_merge",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.savewithmerge},
+  {"expand_dir",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.expanddir},
+  {"expand",			MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.expand},
+  {"load_path",			MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.loadpath},
+  {"history_size",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.hist_size},
+  {"infowin_size",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.info_size},
+  {"data_head_lines",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.data_head_lines},
+  {"use_opacity",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.use_opacity},
+  {"select_data_on_export",	MENU_CONFIG_TYPE_BOOL,    NULL, &Menulocal.select_data},
   {NULL},
 };
 
@@ -258,7 +259,8 @@ static struct menu_config MenuConfigViewer[] = {
 
 static struct menu_config MenuConfigToggleView[] = {
   {"viewer_show_ruler",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ruler},
-  {"status_bar",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.statusb},
+  {"sidebar",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.sidebar},
+  {"status_bar",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.statusbar},
   {"scrollbar",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.scrollbar},
   {"command_toolbar",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ctoolbar},
   {"pointer_toolbar",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ptoolbar},
@@ -274,6 +276,19 @@ static struct menu_config MenuConfigOthers[] = {
   {"ps_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.ps_version},
   {"svg_version",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.svg_version},
   {"palette",		MENU_CONFIG_TYPE_STRING,  NULL, &Menulocal.Palette},
+  {"main_pane",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.main_pane_pos},
+  {"side_pane1",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.side_pane1_pos},
+  {"side_pane2",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.side_pane2_pos},
+  {"side_pane3",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.side_pane3_pos},
+  {"single_window_mode",MENU_CONFIG_TYPE_BOOL,    NULL, &Menulocal.single_window_mode},
+  {"file_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.file_tab},
+  {"axis_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.axis_tab},
+  {"merge_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.merge_tab},
+  {"path_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.path_tab},
+  {"rectangle_tab",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.rectangle_tab},
+  {"arc_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.arc_tab},
+  {"mark_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.mark_tab},
+  {"text_tab",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.text_tab},
   {NULL},
 };
 
@@ -396,6 +411,7 @@ add_child_geometry_to_array(struct menu_config *cfg, struct narray *conf)
   buf = (char *) g_malloc(BUF_SIZE);
   if (buf) {
     sub_window_save_geometry(stat->win);
+    sub_window_save_visibility(stat->win);
     snprintf(buf, BUF_SIZE, "%s=%d,%d,%d,%d,%d",
 	     cfg->name, *data[0], *data[1], *data[2], *data[3], *data[4]);
     arrayadd(conf, &buf);
@@ -406,10 +422,9 @@ static void
 add_geometry_to_array(struct menu_config *cfg, struct narray *conf)
 {
   char *buf;
-  GdkWindowState state;
   gint x, y, w, h;
 
-  get_window_geometry(TopLevel, &x, &y, &w, &h, &state);
+  get_window_geometry(TopLevel, &x, &y, &w, &h);
 
   Menulocal.menux = x;
   Menulocal.menuy = y;
@@ -1110,7 +1125,8 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
     = Menulocal.menuheight = Menulocal.menuwidth = DEFAULT_GEOMETRY;
   initwindowconfig();
   Menulocal.showtip = TRUE;
-  Menulocal.statusb = TRUE;
+  Menulocal.sidebar = TRUE;
+  Menulocal.statusbar = TRUE;
   Menulocal.ruler = TRUE;
   Menulocal.scrollbar = TRUE;
   Menulocal.ptoolbar = TRUE;
@@ -1127,7 +1143,7 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.exwin_use_external = TRUE;
   Menulocal.expand = 1;
   Menulocal.expanddir = g_strdup("./");
-  Menulocal.expandtofullpath = TRUE;
+  Menulocal.loadpath = SAVE_PATH_FULL;
   Menulocal.ngpfilelist = gtk_recent_manager_get_default();
   Menulocal.GRAobj = chkobject("gra");
   Menulocal.hist_size = 1000;
@@ -1136,6 +1152,19 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.bg_g = 1.0;
   Menulocal.bg_b = 1.0;
   Menulocal.focus_frame_type = N_LINE_TYPE_DOT;
+  Menulocal.main_pane_pos = 600;
+  Menulocal.side_pane1_pos = 500;
+  Menulocal.side_pane2_pos = 300;
+  Menulocal.side_pane3_pos = 200;
+  Menulocal.single_window_mode = FALSE;
+  Menulocal.file_tab = 0;
+  Menulocal.axis_tab = 100;
+  Menulocal.merge_tab = 101;
+  Menulocal.path_tab = 1;
+  Menulocal.rectangle_tab = 2;
+  Menulocal.arc_tab = 3;
+  Menulocal.mark_tab = 4;
+  Menulocal.text_tab = 5;
 
   arrayinit(&(Menulocal.drawrable), sizeof(char *));
   menuadddrawrable(chkobject("draw"), &(Menulocal.drawrable));
@@ -1146,11 +1175,12 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.grid = 200;
   Menulocal.data_head_lines = 20;
   Menulocal.use_opacity = FALSE;
+  Menulocal.select_data = TRUE;
   Menulocal.local = local;
 
   Menulocal.png_dpi = 72;
 #ifdef WINDOWS
-  Menulocal.emf_dpi = 72;
+  Menulocal.emf_dpi = 576;
 #endif
   Menulocal.ps_version = 0;
   Menulocal.svg_version = 0;
@@ -1402,6 +1432,22 @@ mx_data_head_lines(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
   return 0;
 }
 
+void
+main_window_redraw(void)
+{
+  GdkWindow *win;
+  if (NgraphApp.Viewer.Win == NULL) {
+    return;
+  }
+
+  win = gtk_widget_get_window(NgraphApp.Viewer.Win);
+  if(win == NULL) {
+    return;
+  }
+
+  gdk_window_invalidate_rect(win, NULL, FALSE);
+}
+
 void
 mx_redraw(struct objlist *obj, N_VALUE *inst)
 {
@@ -1421,9 +1467,7 @@ mx_redraw(struct objlist *obj, N_VALUE *inst)
   mxflush(obj, inst, NULL, 0, NULL);
   draw_paper_frame();
 
-  if (NgraphApp.Viewer.gdk_win) {
-    gdk_window_invalidate_rect(NgraphApp.Viewer.gdk_win, NULL, TRUE);
-  }
+  main_window_redraw();
 }
 
 void
@@ -1477,23 +1521,28 @@ mxdpi(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
     Menulocal.local->pixel_dot_y = dpi / (DPI_MAX * 1.0);
   *(int *) argv[2] = dpi;
 
-  if (NgraphApp.Viewer.gdk_win) {
-    gdk_window_invalidate_rect(NgraphApp.Viewer.gdk_win, NULL, TRUE);
-  }
+  main_window_redraw();
+
   return 0;
 }
 
 static int
 mxflush(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
 {
+  cairo_surface_t *surface;
+
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
   }
 
-  if (Menulocal.local->linetonum && Menulocal.local->cairo) {
-    cairo_stroke(Menulocal.local->cairo);
-    Menulocal.local->linetonum = 0;
+  if (Menulocal.local->cairo) {
+    gra2cairo_draw_path(Menulocal.local);
+
+    surface = cairo_get_target(Menulocal.local->cairo);
+    if (surface) {
+      cairo_surface_flush(surface);
+    }
   }
 
   return 0;
@@ -1539,11 +1588,9 @@ mxclear(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv
 
   mx_clear(NULL);
 
-  if (NgraphApp.Viewer.gdk_win) {
-    gdk_window_invalidate_rect(NgraphApp.Viewer.gdk_win, NULL, TRUE);
-  }
+  main_window_redraw();
 
-return 0;
+  return 0;
 }
 
 static int
@@ -1739,6 +1786,10 @@ mx_show_win(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **
 {
   unsigned int win;
 
+  if (Menulocal.single_window_mode) {
+    return 0;
+  }
+
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
@@ -1756,6 +1807,10 @@ mx_hide_win(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **
 {
   unsigned int win;
 
+  if (Menulocal.single_window_mode) {
+    return 0;
+  }
+
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
@@ -1773,6 +1828,10 @@ mx_toggle_win(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
 {
   int win;
 
+  if (Menulocal.single_window_mode) {
+    return 0;
+  }
+
   if (TopLevel == NULL) {
     error(obj, ERR_MENU_GUI);
     return 1;
@@ -1821,7 +1880,7 @@ mx_get_accel_map(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, ch
   return 0;
 }
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
 #include <readline/readline.h>
 #endif
 
@@ -1883,7 +1942,7 @@ mx_show_lib_version(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc,
 	       h,
 	       pango_version_string());
 
-#ifdef HAVE_LIBREADLINE
+#ifdef HAVE_READLINE_READLINE_H
   printfstdout("\n");
   printfstdout("%sreadline\n"
 	       "%s compile: %d.%d\n"
@@ -1959,6 +2018,8 @@ mxmodified(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **a
   modified = * (int *) argv[2];
 
   SetCaption(modified);
+  set_modified_state(modified);
+
   return 0;
 }
 
@@ -2147,8 +2208,6 @@ graph_modified_sub(int a)
     return;
 
   putobj(Menulocal.obj, "modified", 0, &a);
-  SetCaption(a);
-  set_modified_state(a);
 }
 
 void
