@@ -53,18 +53,18 @@
 #include "x11commn.h"
 
 static n_list_store Alist[] = {
-  {"",         G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden",    FALSE},
-  {"#",        G_TYPE_INT,     TRUE, FALSE, "id",        FALSE},
-  {N_("name"), G_TYPE_STRING,  TRUE, FALSE, "group",     FALSE},
-  {N_("min"),  G_TYPE_STRING,  TRUE, TRUE,  "min",       FALSE},
-  {N_("max"),  G_TYPE_STRING,  TRUE, TRUE,  "max",       FALSE},
-  {N_("inc"),  G_TYPE_STRING,  TRUE, TRUE,  "inc",       FALSE},
-  {N_("type"), G_TYPE_ENUM,    TRUE, TRUE,  "type",      FALSE},
-  {"x",        G_TYPE_DOUBLE,  TRUE, TRUE,  "x",         FALSE, - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {"y",        G_TYPE_DOUBLE,  TRUE, TRUE,  "y",         FALSE, - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {N_("dir"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "direction", FALSE,                0,          36000, 100, 1500},
-  {N_("len"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "length",    FALSE, - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {"^#",       G_TYPE_INT,     TRUE, FALSE, "oid",       FALSE},
+  {" ",        G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden"},
+  {"#",        G_TYPE_INT,     TRUE, FALSE, "id"},
+  {N_("name"), G_TYPE_STRING,  TRUE, FALSE, "group"},
+  {N_("min"),  G_TYPE_STRING,  TRUE, TRUE,  "min"},
+  {N_("max"),  G_TYPE_STRING,  TRUE, TRUE,  "max"},
+  {N_("inc"),  G_TYPE_STRING,  TRUE, TRUE,  "inc"},
+  {N_("type"), G_TYPE_PARAM,   TRUE, TRUE,  "type"},
+  {"x",        G_TYPE_DOUBLE,  TRUE, TRUE,  "x", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {"y",        G_TYPE_DOUBLE,  TRUE, TRUE,  "y", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("dir"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "direction",        0,          36000, 100, 1500},
+  {N_("len"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "length", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {"^#",       G_TYPE_INT,     TRUE, FALSE, "oid"},
 };
 
 #define AXIS_WIN_COL_NUM (sizeof(Alist)/sizeof(*Alist))
@@ -91,7 +91,6 @@ static struct subwin_popup_list Popup_list[] = {
   {GTK_STOCK_DELETE,      G_CALLBACK(axis_delete_popup_func), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, 0, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {N_("_Focus"),          G_CALLBACK(list_sub_window_focus), FALSE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
-  {N_("_Show"),           G_CALLBACK(list_sub_window_hide), FALSE, NULL, POP_UP_MENU_ITEM_TYPE_CHECK},
   {GTK_STOCK_CLEAR,       G_CALLBACK(axiswin_scale_clear), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {GTK_STOCK_PROPERTIES,  G_CALLBACK(list_sub_window_update), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, 0, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
@@ -103,17 +102,15 @@ static struct subwin_popup_list Popup_list[] = {
 
 #define POPUP_ITEM_NUM (sizeof(Popup_list) / sizeof(*Popup_list))
 
-#define POPUP_ITEM_HIDE 4
-#define POPUP_ITEM_TOP 8
-#define POPUP_ITEM_UP 9
-#define POPUP_ITEM_DOWN 10
-#define POPUP_ITEM_BOTTOM 11
+#define POPUP_ITEM_TOP 7
+#define POPUP_ITEM_UP 8
+#define POPUP_ITEM_DOWN 9
+#define POPUP_ITEM_BOTTOM 10
 
-#define CB_BUF_SIZE 128
 #define ID_BUF_SIZE 16
 #define TITLE_BUF_SIZE 128
 
-static void axis_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row);
+static void axis_list_set_val(struct obj_list_data *d, GtkTreeIter *iter, int row);
 static int check_axis_history(struct objlist *obj);
 
 #define TIME_FORMAT_STR N_(						\
@@ -166,10 +163,14 @@ AxisCB(struct objlist *obj, int id)
 
   getobj(obj, "direction", id, 0, NULL, &dir);
   getobj(obj, "group", id, 0, NULL, &name);
+
+  s = NULL;
   name = CHK_STR(name);
   sgetobjfield(obj, id, "type", NULL, &valstr, FALSE, FALSE, FALSE);
-  s = g_strdup_printf("%-10s %.6s %s:%.2f", name, _(valstr), _("dir"), dir / 100.0);
-  g_free(valstr);
+  if (valstr) {
+    s = g_strdup_printf("%-10s %.6s %s:%.2f", name, _(valstr), _("dir"), dir / 100.0);
+    g_free(valstr);
+  }
 
   return s;
 }
@@ -177,8 +178,7 @@ AxisCB(struct objlist *obj, int id)
 char *
 AxisHistoryCB(struct objlist *obj, int id)
 {
-  char *s, *valstr, *name;
-  int dir, num;
+  int num;
   struct narray *array;
 
   getobj(obj, "scale_history", id, 0, NULL, &array);
@@ -187,15 +187,7 @@ AxisHistoryCB(struct objlist *obj, int id)
   if (num == 0)
     return NULL;
 
-  getobj(obj, "group", id, 0, NULL, &name);
-  getobj(obj, "direction", id, 0, NULL, &dir);
-
-  name = CHK_STR(name);
-  sgetobjfield(obj, id, "type", NULL, &valstr, FALSE, FALSE, FALSE);
-  s = g_strdup_printf("%-10s %.6s %s:%.2f", name, _(valstr), _("dir"), dir / 100.0);
-  g_free(valstr);
-
-  return s;
+  return AxisCB(obj, id);
 }
 
 static char *
@@ -246,20 +238,24 @@ GridDialogSetupItem(GtkWidget *w, struct GridDialog *d, int id)
   }
 
   sgetobjfield(d->Obj, id, "axis_x", NULL, &valstr, FALSE, FALSE, FALSE);
-  for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
-  if (valstr[i] == ':') {
-    i++;
+  if (valstr) {
+    for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
+    if (valstr[i] == ':') {
+      i++;
+    }
+    combo_box_entry_set_text(d->axisx, valstr + i);
+    g_free(valstr);
   }
-  combo_box_entry_set_text(d->axisx, valstr + i);
-  g_free(valstr);
 
   sgetobjfield(d->Obj, id, "axis_y", NULL, &valstr, FALSE, FALSE, FALSE);
-  for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
-  if (valstr[i] == ':') {
-    i++;
+  if (valstr) {
+    for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
+    if (valstr[i] == ':') {
+      i++;
+    }
+    combo_box_entry_set_text(d->axisy, valstr + i);
+    g_free(valstr);
   }
-  combo_box_entry_set_text(d->axisy, valstr + i);
-  g_free(valstr);
 
   SetWidgetFromObjField(d->draw_x, d->Obj, id, "grid_x");
   SetWidgetFromObjField(d->draw_y, d->Obj, id, "grid_y");
@@ -529,7 +525,7 @@ SectionDialogAxisX(GtkWidget *w, gpointer client_data)
 
   d = (struct SectionDialog *) client_data;
   if (d->IDX >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDX, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDX, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -541,7 +537,7 @@ SectionDialogAxisY(GtkWidget *w, gpointer client_data)
 
   d = (struct SectionDialog *) client_data;
   if (d->IDY >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDY, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDY, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -553,7 +549,7 @@ SectionDialogAxisU(GtkWidget *w, gpointer client_data)
 
   d = (struct SectionDialog *) client_data;
   if (d->IDU >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDU, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDU, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -565,7 +561,7 @@ SectionDialogAxisR(GtkWidget *w, gpointer client_data)
 
   d = (struct SectionDialog *) client_data;
   if (d->IDR >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDR, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDR, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -855,7 +851,7 @@ CrossDialogAxisX(GtkWidget *w, gpointer client_data)
 
   d = (struct CrossDialog *) client_data;
   if (d->IDX >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDX, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDX, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -867,7 +863,7 @@ CrossDialogAxisY(GtkWidget *w, gpointer client_data)
 
   d = (struct CrossDialog *) client_data;
   if (d->IDY >= 0) {
-    AxisDialog(&DlgAxis, d->Obj, d->IDY, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, d->IDY, -1);
     DialogExecute(d->widget, &DlgAxis);
   }
 }
@@ -1154,13 +1150,14 @@ scale_tab_setup_item(struct AxisDialog *d, int id)
   }
 
   sgetobjfield(d->Obj, id, "reference", NULL, &valstr, FALSE, FALSE, FALSE);
-  for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
-  if (valstr[i] == ':') {
-    i++;
+  if (valstr) {
+    for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
+    if (valstr[i] == ':') {
+      i++;
+    }
+    combo_box_entry_set_text(d->ref, valstr + i);
+    g_free(valstr);
   }
-  combo_box_entry_set_text(d->ref, valstr + i);
-  g_free(valstr);
-
   SetWidgetFromObjField(d->margin, d->Obj, id, "auto_scale_margin");
 }
 
@@ -2279,12 +2276,14 @@ position_tab_setup_item(struct AxisDialog *axis, int id)
   }
 
   sgetobjfield(axis->Obj, id, "adjust_axis", NULL, &valstr, FALSE, FALSE, FALSE);
-  for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
-  if (valstr[i] == ':')
-    i++;
+  if (valstr) {
+    for (i = 0; (valstr[i] != '\0') && (valstr[i] != ':'); i++);
+    if (valstr[i] == ':')
+      i++;
 
-  combo_box_entry_set_text(d->adjust, valstr + i);
-  g_free(valstr);
+    combo_box_entry_set_text(d->adjust, valstr + i);
+    g_free(valstr);
+  }
 
   SetWidgetFromObjField(d->adjustpos, axis->Obj, id, "adjust_position");
 }
@@ -2508,15 +2507,15 @@ AxisDialogClose(GtkWidget *w, void *data)
 }
 
 void
-AxisDialog(void *data, struct objlist *obj, int id, int sub_id)
+AxisDialog(struct obj_list_data *data, int id, int user_data)
 {
   struct AxisDialog *d;
 
-  d = (struct AxisDialog *) data;
+  d = (struct AxisDialog *) data->dialog;
 
   d->SetupWindow = AxisDialogSetup;
   d->CloseWindow = AxisDialogClose;
-  d->Obj = obj;
+  d->Obj = data->obj;
   d->Id = id;
 }
 
@@ -2573,7 +2572,7 @@ CmAxisNewFrame(GtkAction *w, gpointer client_data)
   } else {
     set_graph_modified();
   }
-  AxisWinUpdate(TRUE);
+  AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
 }
 
 void
@@ -2640,7 +2639,7 @@ CmAxisNewSection(GtkAction *w, gpointer client_data)
   } else {
     set_graph_modified();
   }
-  AxisWinUpdate(TRUE);
+  AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
 }
 
 void
@@ -2682,7 +2681,7 @@ CmAxisNewCross(GtkAction *w, gpointer client_data)
     delobj(obj, idx);
   } else
     set_graph_modified();
-  AxisWinUpdate(TRUE);
+  AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
 }
 
 void
@@ -2696,13 +2695,13 @@ CmAxisNewSingle(GtkAction *w, gpointer client_data)
   if ((obj = chkobject("axis")) == NULL)
     return;
   if ((id = newobj(obj)) >= 0) {
-    AxisDialog(&DlgAxis, obj, id, -1);
+    AxisDialog(NgraphApp.AxisWin.data.data, id, -1);
     ret = DialogExecute(TopLevel, &DlgAxis);
     if ((ret == IDDELETE) || (ret == IDCANCEL)) {
       delobj(obj, id);
     } else
       set_graph_modified();
-    AxisWinUpdate(TRUE);
+    AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
   }
 }
 
@@ -2725,8 +2724,8 @@ CmAxisDel(GtkAction *w, gpointer client_data)
   if (DialogExecute(TopLevel, &DlgCopy) == IDOK && DlgCopy.sel >= 0) {
     AxisDel(DlgCopy.sel);
     set_graph_modified();
-    AxisWinUpdate(TRUE);
-    FileWinUpdate(TRUE);
+    AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
   }
 }
 
@@ -2750,12 +2749,12 @@ CmAxisUpdate(GtkAction *w, gpointer client_data)
   } else {
     return;
   }
-  AxisDialog(&DlgAxis, obj, i, -1);
+  AxisDialog(NgraphApp.AxisWin.data.data, i, -1);
   if ((ret = DialogExecute(TopLevel, &DlgAxis)) == IDDELETE) {
     AxisDel(i);
   }
-  AxisWinUpdate(TRUE);
-  FileWinUpdate(TRUE);
+  AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
+  FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
 }
 
 void
@@ -2798,7 +2797,7 @@ CmAxisZoom(GtkAction *w, gpointer client_data)
 	  set_graph_modified();
 	}
       }
-      AxisWinUpdate(TRUE);
+      AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
     }
     arraydel(&farray);
   }
@@ -2807,9 +2806,9 @@ CmAxisZoom(GtkAction *w, gpointer client_data)
 static void
 axiswin_scale_clear(GtkMenuItem *item, gpointer user_data)
 {
-  struct SubWin *d;
+  struct obj_list_data *d;
   struct objlist *obj;
-  int sel;
+  int sel, num;
 
   if (Menulock || Globallock)
     return;
@@ -2818,17 +2817,18 @@ axiswin_scale_clear(GtkMenuItem *item, gpointer user_data)
   if (obj == NULL)
     return;
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
 
-  sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  sel = list_store_get_selected_int(d->text, AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if ((sel >= 0) && (sel <= d->num)) {
-    d->setup_dialog(d->dialog, d->obj, sel, -1);
+  if ((sel >= 0) && (sel <= num)) {
+    d->setup_dialog(d, sel, -1);
     d->select = sel;
     axis_scale_push(obj, sel);
     exeobj(obj, "clear", sel, 0, NULL);
     set_graph_modified();
-    d->update(FALSE);
+    d->update(d, FALSE);
   }
 }
 
@@ -2855,7 +2855,7 @@ CmAxisClear(GtkAction *w, gpointer client_data)
       exeobj(obj, "clear", array[i], 0, NULL);
       set_graph_modified();
     }
-    AxisWinUpdate(TRUE);
+    AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
   }
   arraydel(&farray);
 }
@@ -2938,15 +2938,12 @@ CmAxisGridUpdate(GtkAction *w, gpointer client_data)
 }
 
 void
-AxisWinUpdate(int clear)
+AxisWinUpdate(struct obj_list_data *d, int clear)
 {
-  struct SubWin *d;
-
   if (Menulock || Globallock)
     return;
 
-  d = &(NgraphApp.AxisWin);
-  if (d->text == NULL)
+  if (d == NULL)
     return;
 
   if (list_sub_window_must_rebuild(d)) {
@@ -2961,7 +2958,13 @@ AxisWinUpdate(int clear)
 }
 
 static void
-axis_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row)
+AxisDelCB(struct obj_list_data *data, int id)
+{
+  AxisDel(id);
+}
+
+static void
+axis_list_set_val(struct obj_list_data *d, GtkTreeIter *iter, int row)
 {
   int cx;
   unsigned int i;
@@ -2996,8 +2999,10 @@ axis_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row)
       break;
     case AXIS_WIN_COL_TYPE:
       sgetobjfield(d->obj, row, "type", NULL, &valstr, FALSE, FALSE, FALSE);
-      list_store_set_string(GTK_WIDGET(d->text), iter, i, _(valstr));
-      g_free(valstr);
+      if (valstr) {
+	list_store_set_string(GTK_WIDGET(d->text), iter, i, _(valstr));
+	g_free(valstr);
+      }
       break;
     case AXIS_WIN_COL_INC:
       getobj(d->obj, "inc", row, 0, NULL, &inc);
@@ -3072,7 +3077,7 @@ CmAxisScaleUndo(GtkAction *w, gpointer client_data)
     }
     n = check_axis_history(obj);
     set_axis_undo_button_sensitivity(n > 0);
-    AxisWinUpdate(TRUE);
+    AxisWinUpdate(NgraphApp.AxisWin.data.data, TRUE);
   }
   arraydel(&farray);
 }
@@ -3081,114 +3086,36 @@ static void
 popup_show_cb(GtkWidget *widget, gpointer user_data)
 {
   unsigned int i;
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
 
   sel = d->select;
+  num = chkobjlastinst(d->obj);
   for (i = 0; i < POPUP_ITEM_NUM; i++) {
     switch (i) {
     case POPUP_ITEM_TOP:
     case POPUP_ITEM_UP:
-      gtk_widget_set_sensitive(d->popup_item[i], sel > 0 && sel <= d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel > 0 && sel <= num);
       break;
     case POPUP_ITEM_DOWN:
     case POPUP_ITEM_BOTTOM:
-      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel < d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel < num);
       break;
-    case POPUP_ITEM_HIDE:
-      if (sel >= 0 && sel <= d->num) {
-	int hidden;
-	getobj(d->obj, "hidden", sel, 0, NULL, &hidden);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(d->popup_item[i]), ! hidden);
-      }
     default:
-      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel <= d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel <= num);
     }
   }
 }
 
-static void
-select_type(GtkComboBox *w, gpointer user_data)
-{
-  int j, type, sel;
-  struct SubWin *d;
-
-  d = (struct SubWin *) user_data;
-
-  sel = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "user-data"));
-  if (sel < 0)
-    return;
-
-  getobj(d->obj, "type", sel, 0, NULL, &type);
-
-  j = combo_box_get_active(GTK_WIDGET(w));
-  if (j < 0 || j == type)
-    return;
-
-  if (putobj(d->obj, "type", sel, &j) >= 0)
-    d->select = sel;
-}
-
-static void
-start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path, gpointer user_data)
-{
-  GtkTreeView *view;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  struct SubWin *d;
-  GtkComboBox *cbox;
-  int sel, type;
-
-  menu_lock(TRUE);
-
-  d = (struct SubWin *) user_data;
-
-  view = GTK_TREE_VIEW(d->text);
-  model = gtk_tree_view_get_model(view);
-
-  if (! gtk_tree_model_get_iter_from_string(model, &iter, path))
-    return;
-
-  list_store_select_iter(GTK_WIDGET(view), &iter);
-  sel = list_store_get_selected_int(GTK_WIDGET(view), AXIS_WIN_COL_ID);
-
-  cbox = GTK_COMBO_BOX(editable);
-  g_object_set_data(G_OBJECT(cbox), "user-data", GINT_TO_POINTER(sel));
-
-  SetWidgetFromObjField(GTK_WIDGET(cbox), d->obj, sel, "type");
-
-  getobj(d->obj, "type", sel, 0, NULL, &type);
-  combo_box_set_active(GTK_WIDGET(cbox), type);
-
-  d->select = -1;
-  g_signal_connect(cbox, "changed", G_CALLBACK(select_type), d);
-}
-
-static void
-edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
-{
-  struct SubWin *d;
-
-  menu_lock(FALSE);
-
-  d = (struct SubWin *) user_data;
-
-  if (str == NULL || d->select < 0)
-    return;
-
-  d->update(FALSE);
-  set_graph_modified();
-}
-
 enum CHANGE_DIR {
   CHANGE_DIR_X,
   CHANGE_DIR_Y,
 };
 
 static void
-pos_edited_common(struct SubWin *d, int id, char *str, enum CHANGE_DIR dir)
+pos_edited_common(struct obj_list_data *d, int id, char *str, enum CHANGE_DIR dir)
 {
   int x, y, pos1, pos2, man, ecode;
   double val;
@@ -3236,19 +3163,19 @@ pos_edited_common(struct SubWin *d, int id, char *str, enum CHANGE_DIR dir)
     exeobj(d->obj, "move", man, 2, argv);
 
     set_graph_modified();
-    AxisWinUpdate(TRUE);
+    AxisWinUpdate(d, TRUE);
   }
 }
 
 static void
 pos_x_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
 {
-  struct SubWin *d;
+  struct obj_list_data *d;
   int sel;
 
   menu_lock(FALSE);
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
 
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
 
@@ -3258,12 +3185,12 @@ pos_x_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer u
 static void
 pos_y_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
 {
-  struct SubWin *d;
+  struct obj_list_data *d;
   int sel;
 
   menu_lock(FALSE);
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
 
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
 
@@ -3271,15 +3198,16 @@ pos_y_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer u
 }
 
 static void
-axis_prm_edited_common(struct SubWin *d, char *field, gchar *str)
+axis_prm_edited_common(struct obj_list_data *d, char *field, gchar *str)
 {
-  int sel;
+  int sel, num;
 
   menu_lock(FALSE);
 
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if (sel < 0 || sel > d->num)
+  if (sel < 0 || sel > num)
     return;
 
   axis_scale_push(d->obj, sel);
@@ -3288,41 +3216,370 @@ axis_prm_edited_common(struct SubWin *d, char *field, gchar *str)
     return;
 
   d->select = sel;
-  d->update(FALSE);
+  d->update(d, FALSE);
 }
 
 static void
 min_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
 {
-  axis_prm_edited_common((struct SubWin *) user_data, "min", str);
+  axis_prm_edited_common((struct obj_list_data *) user_data, "min", str);
 }
 
 static void
 max_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
 {
-  axis_prm_edited_common((struct SubWin *) user_data, "max", str);
+  axis_prm_edited_common((struct obj_list_data *) user_data, "max", str);
 }
 
 static void
 inc_edited(GtkCellRenderer *cell_renderer, gchar *path, gchar *str, gpointer user_data)
 {
-  axis_prm_edited_common((struct SubWin *) user_data, "inc", str);
+  axis_prm_edited_common((struct obj_list_data *) user_data, "inc", str);
+}
+
+enum AXIS_COMBO_ITEM {
+  AXIS_COMBO_ITEM_SCALE,
+
+  AXIS_COMBO_ITEM_BASE_DRAW,
+  AXIS_COMBO_ITEM_BASE_COLOR,
+  AXIS_COMBO_ITEM_BASE_STYLE,
+  AXIS_COMBO_ITEM_BASE_ARROW,
+  AXIS_COMBO_ITEM_BASE_WAVE,
+
+  AXIS_COMBO_ITEM_GAUGE_POS,
+  AXIS_COMBO_ITEM_GAUGE_COLOR,
+  AXIS_COMBO_ITEM_GAUGE_STYLE,
+
+  AXIS_COMBO_ITEM_NUM_POS,
+  AXIS_COMBO_ITEM_NUM_ALIGN,
+  AXIS_COMBO_ITEM_NUM_DIR,
+  AXIS_COMBO_ITEM_NUM_LOG,
+  AXIS_COMBO_ITEM_NUM_NO_ZERO,
+  AXIS_COMBO_ITEM_NUM_COLOR,
+  AXIS_COMBO_ITEM_NUM_FONT,
+  AXIS_COMBO_ITEM_NUM_BOLD,
+  AXIS_COMBO_ITEM_NUM_ITALIC,
+};
+
+static void
+create_base_combo_item(GtkTreeStore *list, GtkTreeIter *parent, struct objlist *obj, int id)
+{
+  GtkTreeIter iter, child;
+
+  gtk_tree_store_append(list, &iter, parent);
+  gtk_tree_store_set(list, &iter,
+		     OBJECT_COLUMN_TYPE_STRING, _("Baseline"),
+		     OBJECT_COLUMN_TYPE_PIXBUF, NULL,
+		     OBJECT_COLUMN_TYPE_INT, -1, 
+		     OBJECT_COLUMN_TYPE_TOGGLE_VISIBLE, FALSE,
+		     OBJECT_COLUMN_TYPE_PIXBUF_VISIBLE, FALSE,
+		     -1);
+
+#if GTK_CHECK_VERSION(3, 0, 0)
+  add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_BASE_DRAW, obj, "baseline", id, _("Draw"));
+#endif
+  add_line_style_item_to_cbox(list, &iter, AXIS_COMBO_ITEM_BASE_STYLE, obj, "style", id);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Arrow"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_BASE_ARROW, obj, "arrow", id);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Wave"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_BASE_WAVE, obj, "wave", id);
+
+  add_text_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_BASE_COLOR, -1, _("Color"), TOGGLE_NONE, FALSE);
+#if ! GTK_CHECK_VERSION(3, 0, 0)
+  add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_BASE_DRAW, obj, "baseline", id, _("Draw"));
+#endif
+
 }
 
 static void
-axiswin_delete_axis(struct SubWin *d)
+create_gauge_combo_item(GtkTreeStore *list, GtkTreeIter *parent, struct objlist *obj, int id)
 {
+  GtkTreeIter iter, child;
+
+  gtk_tree_store_append(list, &iter, parent);
+  gtk_tree_store_set(list, &iter,
+		     OBJECT_COLUMN_TYPE_STRING, _("Gauge"),
+		     OBJECT_COLUMN_TYPE_PIXBUF, NULL,
+		     OBJECT_COLUMN_TYPE_INT, -1, 
+		     OBJECT_COLUMN_TYPE_TOGGLE_VISIBLE, FALSE,
+		     OBJECT_COLUMN_TYPE_PIXBUF_VISIBLE, FALSE,
+		     -1);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Position"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_GAUGE_POS, obj, "gauge", id);
+
+  add_line_style_item_to_cbox(list, &iter, AXIS_COMBO_ITEM_GAUGE_STYLE, obj, "gauge_style", id);
+  add_text_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_GAUGE_COLOR, -1, _("Color"), TOGGLE_NONE, FALSE);
+}
+
+static void
+create_num_combo_item(GtkTreeStore *list, GtkTreeIter *parent, struct objlist *obj, int id)
+{
+  GtkTreeIter iter, child;
+
+  gtk_tree_store_append(list, &iter, parent);
+  gtk_tree_store_set(list, &iter,
+		     OBJECT_COLUMN_TYPE_STRING, _("Numbering"),
+		     OBJECT_COLUMN_TYPE_PIXBUF, NULL,
+		     OBJECT_COLUMN_TYPE_INT, -1,
+		     OBJECT_COLUMN_TYPE_TOGGLE_VISIBLE, FALSE,
+		     OBJECT_COLUMN_TYPE_PIXBUF_VISIBLE, FALSE,
+		     -1);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Position"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_POS, obj, "num", id);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Align"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_ALIGN, obj, "num_align", id);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Direction"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_DIR, obj, "num_direction", id);
+
+  add_text_combo_item_to_cbox(list, &child, &iter, -1, -1, _("Font"), TOGGLE_NONE, FALSE);
+  add_font_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_FONT, obj, "num_font", id);
+
+  add_separator_combo_item_to_cbox(list, NULL, &child);
+
+  add_font_style_combo_item_to_cbox(list, NULL, &child, AXIS_COMBO_ITEM_NUM_BOLD, AXIS_COMBO_ITEM_NUM_ITALIC, obj, "num_font_style", id);
+
+  add_text_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_COLOR, -1, _("Color"), TOGGLE_NONE, FALSE);
+
+  add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_LOG, obj, "num_log_pow", id, _("Log power"));
+  add_bool_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_NUM_NO_ZERO, obj, "num_no_zero", id, _("No zero"));
+}
+
+static void
+create_type_combo_box(GtkWidget *cbox, struct objlist *obj, int id)
+{
+  GtkTreeStore *list;
+  GtkTreeIter iter;
+
+  list = GTK_TREE_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(cbox)));
+  gtk_tree_store_clear(list);
+
+  add_text_combo_item_to_cbox(list, &iter, NULL, -1, -1, _("Scale"), TOGGLE_NONE, FALSE);
+  add_enum_combo_item_to_cbox(list, NULL, &iter, AXIS_COMBO_ITEM_SCALE, obj, "type", id);
+
+  create_base_combo_item(list, NULL, obj, id);
+  create_gauge_combo_item(list, NULL, obj, id);
+  create_num_combo_item(list, NULL, obj, id);
+}
+
+static void
+select_type(GtkComboBox *w, gpointer user_data)
+{
+  int sel, col_type, type, enum_id, found, active, style;
+  struct objlist *obj;
+  struct obj_list_data *d;
+  GtkTreeStore *list;
+  GtkTreeIter iter;
+  char *font, *ptr;
+
+  menu_lock(FALSE);
+
+  d = (struct obj_list_data *) user_data;
+
+  sel = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "user-data"));
+  if (sel < 0)
+    return;
+
+  obj = getobject("axis");
+
+  list = GTK_TREE_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(w)));
+  found = gtk_combo_box_get_active_iter(w, &iter);
+  if (! found)
+    return;
+
+  gtk_tree_model_get(GTK_TREE_MODEL(list), &iter,
+		     OBJECT_COLUMN_TYPE_INT, &col_type,
+		     OBJECT_COLUMN_TYPE_ENUM, &enum_id,
+		     -1);
+
+  switch (col_type) {
+  case AXIS_COMBO_ITEM_SCALE:
+    getobj(d->obj, "type", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "type", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_BASE_DRAW:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    active = ! active;
+    putobj(d->obj, "baseline", sel, &active);
+    break;
+  case AXIS_COMBO_ITEM_BASE_COLOR:
+    if (select_obj_color(obj, sel, OBJ_FIELD_COLOR_TYPE_AXIS_BASE)) {
+      return;
+    }
+    break;
+  case AXIS_COMBO_ITEM_BASE_STYLE:
+    if (enum_id < 0 || enum_id >= FwNumStyleNum) {
+      return;
+    }
+    if (chk_sputobjfield(d->obj, sel, "style", FwLineStyle[enum_id].list) != 0) {
+      return;
+    }
+    if (! get_graph_modified()) {
+      return;
+    }
+    break;
+  case AXIS_COMBO_ITEM_BASE_ARROW:
+    getobj(d->obj, "arrow", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "arrow", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_BASE_WAVE:
+    getobj(d->obj, "wave", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "wave", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_GAUGE_POS:
+    getobj(d->obj, "gauge", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "gauge", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_GAUGE_COLOR:
+    if (select_obj_color(obj, sel, OBJ_FIELD_COLOR_TYPE_AXIS_GAUGE)) {
+      return;
+    }
+    break;
+  case AXIS_COMBO_ITEM_GAUGE_STYLE:
+    if (enum_id < 0 || enum_id >= FwNumStyleNum) {
+      return;
+    }
+    if (chk_sputobjfield(d->obj, sel, "gauge_style", FwLineStyle[enum_id].list) != 0) {
+      return;
+    }
+    if (! get_graph_modified()) {
+      return;
+    }
+    break;
+  case AXIS_COMBO_ITEM_NUM_POS:
+    getobj(d->obj, "num", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "num", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_NUM_ALIGN:
+    getobj(d->obj, "num_align", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "num_align", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_NUM_DIR:
+    getobj(d->obj, "num_direction", sel, 0, NULL, &type);
+    if (type == enum_id) {
+      return;
+    }
+    putobj(d->obj, "num_direction", sel, &enum_id);
+    break;
+  case AXIS_COMBO_ITEM_NUM_LOG:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    active = ! active;
+    putobj(d->obj, "num_log_pow", sel, &active);
+    break;
+  case AXIS_COMBO_ITEM_NUM_NO_ZERO:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    active = ! active;
+    putobj(d->obj, "num_no_zero", sel, &active);
+    break;
+  case AXIS_COMBO_ITEM_NUM_COLOR:
+    if (select_obj_color(obj, sel, OBJ_FIELD_COLOR_TYPE_AXIS_NUM)) {
+      return;
+    }
+    break;
+  case AXIS_COMBO_ITEM_NUM_FONT:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_STRING, &font, -1);
+    getobj(d->obj, "num_font", sel, 0, NULL, &ptr);
+    if (g_strcmp0(font, ptr) == 0) {
+      g_free(font);
+      return;
+    }
+    putobj(d->obj, "num_font", sel, font);
+    break;
+  case AXIS_COMBO_ITEM_NUM_BOLD:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    getobj(d->obj, "num_font_style", sel, 0, NULL, &type);
+    style = (type & GRA_FONT_STYLE_ITALIC) | (active ? 0 : GRA_FONT_STYLE_BOLD);
+    putobj(d->obj, "num_font_style", sel, &style);
+    break;
+  case AXIS_COMBO_ITEM_NUM_ITALIC:
+    gtk_tree_model_get(GTK_TREE_MODEL(list), &iter, OBJECT_COLUMN_TYPE_TOGGLE, &active, -1);
+    getobj(d->obj, "num_font_style", sel, 0, NULL, &type);
+    style = (type & GRA_FONT_STYLE_BOLD) | (active ? 0 : GRA_FONT_STYLE_ITALIC);
+    putobj(d->obj, "num_font_style", sel, &style);
+    break;
+  default:
+    return;
+  }
+
+  d->select = sel;
+  d->update(d, FALSE);
+  set_graph_modified();
+}
+
+static void
+start_editing_type(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path, gpointer user_data)
+{
+  GtkTreeIter iter;
+  struct obj_list_data *d;
+  GtkWidget *cbox;
   int sel;
+  struct objlist *obj;
+
+  menu_lock(TRUE);
+
+  d = (struct obj_list_data *) user_data;
+
+  sel = tree_view_get_selected_row_int_from_path(d->text, path, &iter, AXIS_WIN_COL_ID);
+  if (sel < 0) {
+    return;
+  }
+
+  cbox = GTK_WIDGET(editable);
+  g_object_set_data(G_OBJECT(cbox), "user-data", GINT_TO_POINTER(sel));
+
+  init_object_combo_box(cbox);
+
+  obj = getobject("axis");
+  if (obj == NULL) {
+    return;
+  }
+  create_type_combo_box(cbox, obj, sel);
+
+  g_signal_connect(cbox, "editing-done", G_CALLBACK(select_type), d);
+  gtk_widget_show(cbox);
+
+  return;
+}
+
+static void
+axiswin_delete_axis(struct obj_list_data *d)
+{
+  int sel, num;
 
   if (Menulock || Globallock)
     return;
 
+  UnFocus();
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if ((sel >= 0) && (sel <= d->num)) {
+  if ((sel >= 0) && (sel <= num)) {
     AxisDel(sel);
-    AxisWinUpdate(TRUE);
-    FileWinUpdate(TRUE);
+    AxisWinUpdate(d, TRUE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
     set_graph_modified();
     d->select = -1;
   }
@@ -3331,30 +3588,31 @@ axiswin_delete_axis(struct SubWin *d)
 static void
 axis_delete_popup_func(GtkMenuItem *w, gpointer client_data)
 {
-  struct SubWin *d;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) client_data;
+  d = (struct obj_list_data *) client_data;
   axiswin_delete_axis(d);
 }
 
 static void
 AxisWinAxisTop(GtkWidget *w, gpointer client_data)
 {
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) client_data;
+  d = (struct obj_list_data *) client_data;
 
   if (Menulock || Globallock) return;
   UnFocus();
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if ((sel  >=  0) && (sel <= d->num)) {
+  if ((sel  >=  0) && (sel <= num)) {
     movetopobj(d->obj, sel);
     d->select = 0;
     AxisMove(sel,0);
-    AxisWinUpdate(FALSE);
-    FileWinUpdate(FALSE);
+    AxisWinUpdate(d, FALSE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, FALSE);
     set_graph_modified();
   }
 }
@@ -3362,21 +3620,22 @@ AxisWinAxisTop(GtkWidget *w, gpointer client_data)
 static void
 AxisWinAxisLast(GtkWidget *w, gpointer client_data)
 {
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) client_data;
+  d = (struct obj_list_data *) client_data;
 
   if (Menulock || Globallock) return;
   UnFocus();
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if ((sel >= 0) && (sel <= d->num)) {
+  if ((sel >= 0) && (sel <= num)) {
     movelastobj(d->obj, sel);
-    d->select = d->num;
-    AxisMove(sel, d->num);
-    AxisWinUpdate(FALSE);
-    FileWinUpdate(FALSE);
+    d->select = num;
+    AxisMove(sel, num);
+    AxisWinUpdate(d, FALSE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, FALSE);
     set_graph_modified();
   }
 }
@@ -3384,21 +3643,22 @@ AxisWinAxisLast(GtkWidget *w, gpointer client_data)
 static void
 AxisWinAxisUp(GtkWidget *w, gpointer client_data)
 {
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) client_data;
+  d = (struct obj_list_data *) client_data;
 
   if (Menulock || Globallock) return;
   UnFocus();
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if ((sel >= 1) && (sel <= d->num)) {
+  if ((sel >= 1) && (sel <= num)) {
     moveupobj(d->obj, sel);
     d->select = sel - 1;
     AxisMove(sel, sel - 1);
-    AxisWinUpdate(FALSE);
-    FileWinUpdate(FALSE);
+    AxisWinUpdate(d, FALSE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, FALSE);
     set_graph_modified();
   }
 }
@@ -3406,21 +3666,22 @@ AxisWinAxisUp(GtkWidget *w, gpointer client_data)
 static void
 AxisWinAxisDown(GtkWidget *w, gpointer client_data)
 {
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) client_data;
+  d = (struct obj_list_data *) client_data;
 
   if (Menulock || Globallock) return;
   UnFocus();
   sel = list_store_get_selected_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID);
+  num = chkobjlastinst(d->obj);
 
-  if (sel >= 0 && sel <= d->num-1) {
+  if (sel >= 0 && sel <= num-1) {
     movedownobj(d->obj, sel);
     d->select = sel + 1;
     AxisMove(sel, sel + 1);
-    AxisWinUpdate(FALSE);
-    FileWinUpdate(FALSE);
+    AxisWinUpdate(d, FALSE);
+    FileWinUpdate(NgraphApp.FileWin.data.data, FALSE);
     set_graph_modified();
   }
 }
@@ -3428,7 +3689,7 @@ AxisWinAxisDown(GtkWidget *w, gpointer client_data)
 static gboolean
 axiswin_ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
 {
-  struct SubWin *d;
+  struct obj_list_data *d;
   GdkEventKey *e;
 
   g_return_val_if_fail(w != NULL, FALSE);
@@ -3437,7 +3698,7 @@ axiswin_ev_key_down(GtkWidget *w, GdkEvent *event, gpointer user_data)
   if (Menulock || Globallock)
     return TRUE;
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
   e = (GdkEventKey *)event;
 
   switch (e->keyval) {
@@ -3497,35 +3758,29 @@ CmAxisWindow(GtkToggleAction *action, gpointer client_data)
     return;
   }
 
-  d->update = AxisWinUpdate;
-  d->setup_dialog = AxisDialog;
-  d->dialog = &DlgAxis;
-  d->ev_key = axiswin_ev_key_down;
-  d->delete = AxisDel;
-
   list_sub_window_create(d, "Axis Window", AXIS_WIN_COL_NUM, Alist, Axiswin_xpm, Axiswin48_xpm);
 
-  d->obj = chkobject("axis");
-  d->num = chkobjlastinst(d->obj);
+  d->data.data->update = AxisWinUpdate;
+  d->data.data->setup_dialog = AxisDialog;
+  d->data.data->dialog = &DlgAxis;
+  d->data.data->ev_key = axiswin_ev_key_down;
+  d->data.data->delete = AxisDelCB;
+  d->data.data->obj = chkobject("axis");
 
-  sub_win_create_popup_menu(d, POPUP_ITEM_NUM,  Popup_list, G_CALLBACK(popup_show_cb));
-  set_combo_cell_renderer_cb(d, AXIS_WIN_COL_TYPE, Alist, G_CALLBACK(start_editing), G_CALLBACK(edited));
-  set_editable_cell_renderer_cb(d, AXIS_WIN_COL_X, Alist, G_CALLBACK(pos_x_edited));
-  set_editable_cell_renderer_cb(d, AXIS_WIN_COL_Y, Alist, G_CALLBACK(pos_y_edited));
-  set_editable_cell_renderer_cb(d, AXIS_WIN_COL_MIN, Alist, G_CALLBACK(min_edited));
-  set_editable_cell_renderer_cb(d, AXIS_WIN_COL_MAX, Alist, G_CALLBACK(max_edited));
-  set_editable_cell_renderer_cb(d, AXIS_WIN_COL_INC, Alist, G_CALLBACK(inc_edited));
+  sub_win_create_popup_menu(d->data.data, POPUP_ITEM_NUM,  Popup_list, G_CALLBACK(popup_show_cb));
+  set_editable_cell_renderer_cb(d->data.data, AXIS_WIN_COL_X, Alist, G_CALLBACK(pos_x_edited));
+  set_editable_cell_renderer_cb(d->data.data, AXIS_WIN_COL_Y, Alist, G_CALLBACK(pos_y_edited));
+  set_editable_cell_renderer_cb(d->data.data, AXIS_WIN_COL_MIN, Alist, G_CALLBACK(min_edited));
+  set_editable_cell_renderer_cb(d->data.data, AXIS_WIN_COL_MAX, Alist, G_CALLBACK(max_edited));
+  set_editable_cell_renderer_cb(d->data.data, AXIS_WIN_COL_INC, Alist, G_CALLBACK(inc_edited));
 
-  list_store_set_align(GTK_WIDGET(d->text), AXIS_WIN_COL_MIN, 1.0);
-  list_store_set_align(GTK_WIDGET(d->text), AXIS_WIN_COL_MAX, 1.0);
-  list_store_set_align(GTK_WIDGET(d->text), AXIS_WIN_COL_INC, 1.0);
+  set_combo_cell_renderer_cb(d->data.data, AXIS_WIN_COL_TYPE, Alist, G_CALLBACK(start_editing_type), NULL);
 
-  gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->text), TRUE);
-  gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->text), AXIS_WIN_COL_NAME);
-  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(d->text), AXIS_WIN_COL_NAME);
+  list_store_set_align(GTK_WIDGET(d->data.data->text), AXIS_WIN_COL_MIN, 1.0);
+  list_store_set_align(GTK_WIDGET(d->data.data->text), AXIS_WIN_COL_MAX, 1.0);
+  list_store_set_align(GTK_WIDGET(d->data.data->text), AXIS_WIN_COL_INC, 1.0);
 
-  sub_window_show_all(d);
-  sub_window_set_geometry(d, TRUE);
-
-  AxisWinUpdate(TRUE);
+  gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->data.data->text), TRUE);
+  gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->data.data->text), AXIS_WIN_COL_NAME);
+  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(d->data.data->text), AXIS_WIN_COL_NAME);
 }
