@@ -53,7 +53,6 @@
 #include "x11commn.h"
 #include "x11info.h"
 
-#define MESSAGE_BUF_SIZE 4096
 #define COMMENT_BUF_SIZE 1024
 
 static GtkWidget *ProgressDialog = NULL;
@@ -72,14 +71,14 @@ OpenGRA(void)
   int i, id, otherGC;
   char *device, *name, *name_str = "viewer";
   struct narray *drawrable;
+  N_VALUE *gra_inst;
 
-  if ((Menulocal.GRAinst =
-       chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid)) != NULL)
+  gra_inst = chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid);
+  if (gra_inst) {
     return;
+  }
 
-  /* search closed gra object */
-  if (chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid) == NULL)
-    CloseGRA();
+  CloseGRA();
 
   for (i = chkobjlastinst(Menulocal.GRAobj); i >= 0; i--) {
     getobj(Menulocal.GRAobj, "GC", i, 0, NULL, &otherGC);
@@ -91,8 +90,8 @@ OpenGRA(void)
     /* closed gra object is not found. generate new gra object */
 
     id = newobj(Menulocal.GRAobj);
-    Menulocal.GRAinst = chkobjinst(Menulocal.GRAobj, id);
-    _getobj(Menulocal.GRAobj, "oid", Menulocal.GRAinst, &(Menulocal.GRAoid));
+    gra_inst = chkobjinst(Menulocal.GRAobj, id);
+    _getobj(Menulocal.GRAobj, "oid", gra_inst, &(Menulocal.GRAoid));
 
     /* set page settings */
     putobj(Menulocal.GRAobj, "paper_width", id, &(Menulocal.PaperWidth));
@@ -105,8 +104,8 @@ OpenGRA(void)
     /* find closed gra object */
 
     id = i;
-    Menulocal.GRAinst = chkobjinst(Menulocal.GRAobj, id);
-    _getobj(Menulocal.GRAobj, "oid", Menulocal.GRAinst, &(Menulocal.GRAoid));
+    gra_inst = chkobjinst(Menulocal.GRAobj, id);
+    _getobj(Menulocal.GRAobj, "oid", gra_inst, &(Menulocal.GRAoid));
 
     /* get page settings */
     CheckPage();
@@ -131,15 +130,22 @@ OpenGRA(void)
 void
 CheckPage(void)
 {
-  _getobj(Menulocal.GRAobj, "paper_width", Menulocal.GRAinst,
+  N_VALUE *gra_inst;
+
+  gra_inst = chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid);
+  if (gra_inst == NULL) {
+    return;
+  }
+
+  _getobj(Menulocal.GRAobj, "paper_width", gra_inst,
 	  &(Menulocal.PaperWidth));
-  _getobj(Menulocal.GRAobj, "paper_height", Menulocal.GRAinst,
+  _getobj(Menulocal.GRAobj, "paper_height", gra_inst,
 	  &(Menulocal.PaperHeight));
-  _getobj(Menulocal.GRAobj, "left_margin", Menulocal.GRAinst,
+  _getobj(Menulocal.GRAobj, "left_margin", gra_inst,
 	  &(Menulocal.LeftMargin));
-  _getobj(Menulocal.GRAobj, "top_margin", Menulocal.GRAinst,
+  _getobj(Menulocal.GRAobj, "top_margin", gra_inst,
 	  &(Menulocal.TopMargin));
-  _getobj(Menulocal.GRAobj, "zoom", Menulocal.GRAinst,
+  _getobj(Menulocal.GRAobj, "zoom", gra_inst,
 	  &(Menulocal.PaperZoom));
 
   set_paper_type(Menulocal.PaperWidth, Menulocal.PaperHeight);
@@ -166,18 +172,17 @@ void
 CloseGRA(void)
 {
   int id;
+  N_VALUE *gra_inst;
 
-  if ((Menulocal.GRAinst =
-       chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid)) == NULL)
+  gra_inst = chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid);
+  if (gra_inst == NULL) {
     return;
-
-  if (chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid) != NULL) {
-    _getobj(Menulocal.GRAobj, "id", Menulocal.GRAinst, &id);
-    exeobj(Menulocal.GRAobj, "close", id, 0, NULL);
-    delobj(Menulocal.GRAobj, id);
-    Menulocal.GRAinst = NULL;
-    Menulocal.GRAoid = -1;
   }
+
+  _getobj(Menulocal.GRAobj, "id", gra_inst, &id);
+  exeobj(Menulocal.GRAobj, "close", id, 0, NULL);
+  delobj(Menulocal.GRAobj, id);
+  Menulocal.GRAoid = -1;
 }
 
 void
@@ -194,8 +199,7 @@ ChangeGRA(void)
 
   if (i == -1) {
     /* closed gra is not find. maintain present gra object */
-    if ((Menulocal.GRAinst =
-	 chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid)) == NULL) {
+    if (chkobjinstoid(Menulocal.GRAobj, Menulocal.GRAoid) == NULL) {
       ChangePage();
     }
   } else {
@@ -241,10 +245,11 @@ SetPageSettingsToGRA(void)
 
     num = arraynum(&(Menulocal.drawrable));
 
-    for (j = 0; j < num; j++)
+    for (j = 0; j < num; j++) {
       arrayadd2(drawrable, (char **) arraynget(&(Menulocal.drawrable), j));
+    }
 
-    _putobj(obj, "draw_obj", inst, &drawrable);
+    _putobj(obj, "draw_obj", inst, drawrable);
   }
 }
 
@@ -747,15 +752,17 @@ FitClear(void)
     return;
   for (i = 0; i <= chkobjlastinst(obj); i++) {
     getobj(obj, "fit", i, 0, NULL, &fit);
-    if (fit != NULL) {
-      arrayinit(&iarray, sizeof(int));
-      if (getobjilist(fit, &fitobj, &iarray, FALSE, NULL) == 0) {
-	anum = arraynum(&iarray);
-	if (anum >= 1) {
-	  id = arraylast_int(&iarray);
-	  getobj(obj, "hidden", i, 0, NULL, &hidden);
-	  if (!hidden)
-	    putobj(fitobj, "equation", id, NULL);
+    if (fit == NULL) {
+      continue;
+    }
+    arrayinit(&iarray, sizeof(int));
+    if (getobjilist(fit, &fitobj, &iarray, FALSE, NULL) == 0) {
+      anum = arraynum(&iarray);
+      if (anum >= 1) {
+	id = arraylast_int(&iarray);
+	getobj(obj, "hidden", i, 0, NULL, &hidden);
+	if (! hidden) {
+	  putobj(fitobj, "equation", id, NULL);
 	}
       }
       arraydel(&iarray);
@@ -1096,7 +1103,7 @@ ToBasename(void)
 
 
 void
-LoadNgpFile(char *file, int ignorepath, int expand, char *exdir,
+LoadNgpFile(char *file, int loadpath, int expand, char *exdir,
 	    int console, char *option)
 {
   struct objlist *sys;
@@ -1213,10 +1220,13 @@ LoadNgpFile(char *file, int ignorepath, int expand, char *exdir,
     AddNgpFileList(name);
     reset_graph_modified();
 
-    if (ignorepath) {
+    switch (loadpath) {
+    case LOAD_PATH_BASE:
       ToBasename();
-    } else if (Menulocal.expandtofullpath) {
+      break;
+    case LOAD_PATH_FULL:
       ToFullPath();
+      break;
     }
     InfoWinClear();
   }
@@ -1498,40 +1508,57 @@ SetFileHidden(void)
   struct objlist *fobj;
   int lastinst;
   struct narray farray, ifarray;
-  int i, a, num, *array;
+  int i, a, r, num, inum, *array;
 
-  if ((fobj = chkobject("file")) == NULL)
+  fobj = chkobject("file");
+  if (fobj == NULL) {
     return 1;
-  lastinst = chkobjlastinst(fobj);
-  if (lastinst >= 0) {
-    arrayinit(&ifarray, sizeof(int));
-    for (i = 0; i <= lastinst; i++) {
-      getobj(fobj, "hidden", i, 0, NULL, &a);
-      if (!a)
-	arrayadd(&ifarray, &i);
-    }
-    SelectDialog(&DlgSelect, fobj, FileCB, (struct narray *) &farray,
-		 (struct narray *) &ifarray);
-    if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
-      a = TRUE;
-      for (i = 0; i <= lastinst; i++) {
-	putobj(fobj, "hidden", i, &a);
-      }
-      num = arraynum(&farray);
-      array = arraydata(&farray);
-      a = FALSE;
-      for (i = 0; i < num; i++) {
-	putobj(fobj, "hidden", array[i], &a);
-      }
-    } else {
-      arraydel(&ifarray);
-      arraydel(&farray);
-      return 0;
-    }
-    arraydel(&ifarray);
-    arraydel(&farray);
   }
-  return 1;
+
+  lastinst = chkobjlastinst(fobj);
+  if (lastinst < 0) {
+    return 1;
+  }
+
+  arrayinit(&ifarray, sizeof(int));
+  for (i = 0; i <= lastinst; i++) {
+    getobj(fobj, "hidden", i, 0, NULL, &a);
+    if (!a) {
+      arrayadd(&ifarray, &i);
+    }
+  }
+
+  r = 0;
+  SelectDialog(&DlgSelect, fobj, FileCB, &farray, &ifarray);
+  if (DialogExecute(TopLevel, &DlgSelect) == IDOK) {
+    a = TRUE;
+    for (i = 0; i <= lastinst; i++) {
+      putobj(fobj, "hidden", i, &a);
+    }
+    num = arraynum(&farray);
+    array = arraydata(&farray);
+    a = FALSE;
+    for (i = 0; i < num; i++) {
+      putobj(fobj, "hidden", array[i], &a);
+    }
+
+    inum = arraynum(&ifarray);
+    if (inum != num) {
+      set_graph_modified();
+    } else {
+      for (i = 0; i < num; i++) {
+	if (arraynget_int(&ifarray, i) != array[i]) {
+	  set_graph_modified();
+	  break;
+	}
+      }
+    }
+    r = 1;
+  }
+
+  arraydel(&ifarray);
+  arraydel(&farray);
+  return r;
 }
 
 int
