@@ -27,10 +27,11 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 
+#include "ngraph.h"
 #include "object.h"
 #include "ogra2cairo.h"
+#include "x11menu.h"
 
-#define MARK_TYPE_NUM 90
 #define MATH_FNC_NUM 5
 
 struct line_style {
@@ -54,7 +55,7 @@ void SetTextFromObjPoints(GtkWidget *w, struct objlist *Obj, int Id, char *field
 int SetObjFieldFromStyle(GtkWidget *w, struct objlist *Obj, int Id, char *field);
 void SetStyleFromObjField(GtkWidget *w, struct objlist *Obj, int Id, char *field);
 int SetObjAxisFieldFromWidget(GtkWidget *w, struct objlist *obj, int id, char *field);
-struct compatible_font_info *SetFontListFromObj(GtkWidget *w, struct objlist *obj, int id, char *name);
+struct compatible_font_info *SetFontListFromObj(GtkWidget *w, struct objlist *obj, int id, const char *name);
 void SetObjFieldFromFontList(GtkWidget *w, struct objlist *obj, int id, char *name);
 void set_color(GtkWidget *w, struct objlist *obj, int id, char *prefix);
 void set_color2(GtkWidget *w, struct objlist *obj, int id);
@@ -121,19 +122,20 @@ struct FileDialog
   GtkWidget *file, *load_settings, *fit, *xcol, *xaxis, *ycol, *yaxis,
     *type, *mark_btn, *curve, *col1, *col2,
     *clip, *style, *size, *miter, *join,
-    *comment_box, *file_box, *fit_table, *width, *apply_all, *comment_view;
+    *comment_box, *file_box, *fit_table, *width, *apply_all, *comment_view, *comment_table;
   GtkNotebook *tab;
   struct objlist *Obj;
   int Id;
   struct MarkDialog mark;
-  int R, G, B, R2, G2, B2, alpha, multi_open, fit_row;
+  int R, G, B, R2, G2, B2, alpha, multi_open, fit_row, initialized;
   struct FileMath math;
   struct FileLoad load;
   struct FileMask mask;
   struct FileMove move;
+  char *head_lines;
 };
 
-void FileDialog(void *data, struct objlist *obj, int id, int multi);
+void FileDialog(struct obj_list_data *data, int id, int multi);
 
 struct EvalDialog
 {
@@ -289,7 +291,7 @@ struct AxisDialog
   struct AxisGauge gauge;
   struct AxisNumbering numbering;
 };
-void AxisDialog(void *data, struct objlist *obj, int id, int sub_id);
+void AxisDialog(struct obj_list_data *data, int id, int sub_id);
 
 #define GRID_DIALOG_STYLE_NUM 3
 
@@ -322,7 +324,7 @@ struct MergeDialog
   struct objlist *Obj;
   int Id;
 };
-void MergeDialog(void *data, struct objlist *obj, int id, int Sub_id);
+void MergeDialog(struct obj_list_data *data, int id, int Sub_id);
 
 struct LegendDialog
 {
@@ -404,10 +406,10 @@ struct LoadDialog
 {
   DIALOG_PROTOTYPE;
   /****** local member *******/
-  GtkWidget *expand_file, *ignore_path, *dir;
+  GtkWidget *expand_file, *load_path, *dir;
   int expand;
   char *exdir;
-  int ignorepath;
+  int loadpath;
   int Id;
 };
 void LoadDialog(struct LoadDialog *data);
@@ -527,9 +529,9 @@ struct MiscDialog
   DIALOG_PROTOTYPE;
   /****** local member *******/
   GtkWidget *editor, *directory, *path, *datafile,
-    *expand, *expanddir, *ignorepath, *mergefile, *coordwin_font, *infowin_font,
+    *expand, *expanddir, *loadpath, *mergefile, *coordwin_font, *infowin_font,
     *file_preview_font, *hist_size, *info_size, *data_head_lines, *help_browser,
-    *browser, *use_opacity;
+    *browser, *use_opacity, *select_data;
   struct objlist *Obj;
   int Id;
 };
