@@ -467,6 +467,51 @@ DialogRadio(GtkWidget *parent, const char *title, const char *caption, struct na
   return data;
 }
 
+int
+DialogButton(GtkWidget *parent, const char *title, const char *caption, struct narray *array, int *x, int *y)
+{
+  GtkWidget *dlg;
+  gint res_id;
+  char **d;
+  int i, anum;
+
+  d = arraydata(array);
+  anum = arraynum(array);
+
+  if (anum < 1) {
+    return -1;
+  }
+
+  dlg = gtk_dialog_new();
+  for (i = 0; i < anum; i++) {
+    if (d[i] && g_utf8_validate(d[i], -1, NULL)) {
+      gtk_dialog_add_button(GTK_DIALOG(dlg), d[i], i);
+    }
+  }
+
+  if (title && g_utf8_validate(title, -1, NULL)) {
+    gtk_window_set_title(GTK_WINDOW(dlg), title);
+  }
+
+  if (caption && g_utf8_validate(caption, -1, NULL)) {
+    GtkWidget *box, *label;
+    box = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
+    label = gtk_label_new(caption);
+    gtk_box_pack_start(GTK_BOX(box), label, TRUE, TRUE, 4);
+  }
+
+  gtk_window_set_resizable(GTK_WINDOW(dlg), FALSE);
+
+  set_dialog_position(dlg, x, y);
+  gtk_widget_show_all(dlg);
+  res_id = ndialog_run(dlg);
+  get_dialog_position(dlg, x, y);
+  gtk_widget_destroy(dlg);
+  reset_event();
+
+  return res_id;
+}
+
 int
 DialogCombo(GtkWidget *parent, const char *title, const char *caption, struct narray *array, int sel, char **r, int *x, int *y)
 {
@@ -878,12 +923,42 @@ check_overwrite(GtkWidget *parent, const char *filename)
   return r != IDYES;
 }
 
+static char *
+get_filename_with_ext(const char *basename, const char *ext)
+{
+  char *filename;
+  int len, ext_len, i;
+
+  if (basename == NULL) {
+    return NULL;
+  }
+
+  if (ext == NULL || ext[0] == '\0') {
+    return g_strdup(basename);
+  }
+
+  ext_len = strlen(ext);
+  len = strlen(basename);
+  i = len - ext_len;
+
+  if (i > 0 && g_strcmp0(basename + i, ext) == 0 && basename[i - 1] == '.') {
+    return g_strdup(basename);
+  }
+
+  filename = g_strdup_printf("%s%s%s",
+			     basename,
+			     (basename[len -1] == '.') ? "" : ".",
+			     ext);
+  return filename;
+}
+
 static int
 FileSelectionDialog(GtkWidget *parent, int type, char *stock)
 {
   struct nGetOpenFileData *data;
   GtkWidget *dlg, *rc;
   GtkFileFilter *filter;
+  char *fname;
 
   data = &FileSelection;
 
@@ -932,21 +1007,23 @@ FileSelectionDialog(GtkWidget *parent, int type, char *stock)
     gtk_widget_hide(data->chdir_cb);
   }
 
-  if (data->init_file) {
+  fname = get_filename_with_ext(data->init_file, data->ext);
+  if (fname) {
     if (type == GTK_FILE_CHOOSER_ACTION_SAVE) {
 #ifdef WINDOWS
       char *tmp;
-      tmp = g_locale_from_utf8(data->init_file, -1, NULL, NULL, NULL);
+      tmp = g_locale_from_utf8(fname, -1, NULL, NULL, NULL);
       if (tmp) {
 	file_dialog_set_current_neme(dlg, tmp);
 	g_free(tmp);
       }
 #else  /* WINDOWS */
-      file_dialog_set_current_neme(dlg, data->init_file);
+      file_dialog_set_current_neme(dlg, fname);
 #endif	/* WINDOWS */
     } else {
-      gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dlg), data->init_file);
+      gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dlg), fname);
     }
+    g_free(fname);
   }
 
   data->ret = IDCANCEL;
@@ -1071,12 +1148,8 @@ nGetSaveFileName(GtkWidget * parent,
 }
 
 void
-get_window_geometry(GtkWidget *win, gint *x, gint *y, gint *w, gint *h, GdkWindowState *state)
+get_window_geometry(GtkWidget *win, gint *x, gint *y, gint *w, gint *h)
 {
-  if (state) {
-    *state = gdk_window_get_state(gtk_widget_get_window(win));
-  }
-
   gtk_window_get_size(GTK_WINDOW(win), w, h);
   gtk_window_get_position(GTK_WINDOW(win), x, y);
 }
