@@ -34,37 +34,19 @@
 #include "x11menu.h"
 #include "x11info.h"
 
-static GtkWidget *
-create_win(void)
-{
-  struct SubWin *d;
-  GtkWidget *w;
-
-  d = &(NgraphApp.InfoWin);
-
-  w = text_sub_window_create((struct SubWin *)d, "Information Window", Infowin_xpm, Infowin48_xpm);
-
-  if (w) {
-    InfoWinSetFont(Menulocal.infowin_font);
-    sub_window_set_geometry((struct SubWin *) d, TRUE);
-  }
-
-  return w;
-}
-
 void
 InfoWinSetFont(char *font)
 {
-  if (NgraphApp.InfoWin.text && font) {
+  if (NgraphApp.InfoWin.data.text && font) {
     PangoFontDescription *desc;
 
     desc = pango_font_description_from_string(font);
 #if GTK_CHECK_VERSION(3, 0, 0)
-    gtk_widget_override_font(GTK_WIDGET(NgraphApp.InfoWin.text), NULL);
-    gtk_widget_override_font(GTK_WIDGET(NgraphApp.InfoWin.text), desc);
+    gtk_widget_override_font(GTK_WIDGET(NgraphApp.InfoWin.data.text), NULL);
+    gtk_widget_override_font(GTK_WIDGET(NgraphApp.InfoWin.data.text), desc);
 #else
-    gtk_widget_modify_font(GTK_WIDGET(NgraphApp.InfoWin.text), NULL);
-    gtk_widget_modify_font(GTK_WIDGET(NgraphApp.InfoWin.text), desc);
+    gtk_widget_modify_font(GTK_WIDGET(NgraphApp.InfoWin.data.text), NULL);
+    gtk_widget_modify_font(GTK_WIDGET(NgraphApp.InfoWin.data.text), desc);
 #endif
     pango_font_description_free(desc);
   }
@@ -82,14 +64,14 @@ InfoWinDrawInfoText(const char *str)
     return;
 
   if (NgraphApp.InfoWin.Win == NULL) {
-    create_win();
-  }
-
-  if (NgraphApp.InfoWin.text == NULL) {
     return;
   }
 
-  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(NgraphApp.InfoWin.text));
+  if (NgraphApp.InfoWin.data.text == NULL) {
+    return;
+  }
+
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(NgraphApp.InfoWin.data.text));
 
   gtk_text_buffer_get_end_iter(buf, &iter);
   gtk_text_buffer_insert(buf, &iter, str, -1);
@@ -106,7 +88,7 @@ InfoWinDrawInfoText(const char *str)
   gtk_text_buffer_get_end_iter(buf, &iter);
   gtk_text_buffer_place_cursor(buf, &iter);
   mark = gtk_text_buffer_get_selection_bound(buf);
-  gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(NgraphApp.InfoWin.text), mark);
+  gtk_text_view_scroll_mark_onscreen(GTK_TEXT_VIEW(NgraphApp.InfoWin.data.text), mark);
 }
 
 void
@@ -114,10 +96,10 @@ InfoWinClear(void)
 {
   GtkTextBuffer *buf;
 
-  if (NgraphApp.InfoWin.text == NULL)
+  if (NgraphApp.InfoWin.data.text == NULL)
     return;
 
-  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(NgraphApp.InfoWin.text));
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(NgraphApp.InfoWin.data.text));
   gtk_text_buffer_set_text(buf, "", 0);
 }
 
@@ -126,38 +108,11 @@ InfoWinUpdate(int clear)
 {
 }
 
-/*
-void
-InfoWinUnmap(Widget w, XtPointer client_data, XtPointer call_data)
-{
-  struct InfoWin *d;
-  Position x, y, x0, y0;
-  Dimension w0, h0;
-
-  d = &(NgraphApp.InfoWin);
-  if (d->Win != NULL) {
-    XtVaGetValues(d->Win, XmNx, &x, XmNy, &y,
-		  XmNwidth, &w0, XmNheight, &h0, NULL);
-    menulocal.dialogwidth = w0;
-    menulocal.dialogheight = h0;
-    XtTranslateCoords(TopLevel, 0, 0, &x0, &y0);
-    menulocal.dialogx = x - x0;
-    menulocal.dialogy = y - y0;
-    XtDestroyWidget(d->Win);
-    d->Win = NULL;
-    d->text = NULL;
-    XmToggleButtonSetState(XtNameToWidget
-			   (TopLevel, "*windowmenu.button_5"), False, False);
-  }
-}
-*/
-
 void
 CmInformationWindow(GtkToggleAction *action, gpointer data)
 {
   struct SubWin *d;
   int state;
-  GtkWidget *dlg;
 
   d = &(NgraphApp.InfoWin);
 
@@ -176,8 +131,6 @@ CmInformationWindow(GtkToggleAction *action, gpointer data)
     return;
   }
 
-  dlg = create_win();
-  if (dlg) {
-    sub_window_show_all((struct SubWin *) d);
-  }
+  text_sub_window_create(d, "Information Window", Infowin_xpm, Infowin48_xpm);
+  InfoWinSetFont(Menulocal.infowin_font);
 }
