@@ -24,7 +24,8 @@
 #ifndef GTK_MENU_HEADER
 #define GTK_MENU_HEADER
 
-#include "x11dialg.h"
+#include "ngraph.h"
+#include "gtk_liststore.h"
 #include "ogra2cairo.h"
 
 #define N2GTK_RULER_METRIC(v) ((v) / 100.0 * 72.0 / 25.4 * 10)
@@ -40,6 +41,7 @@ enum MenuID {
   MenuIdEditCopy,
   MenuIdEditPaste,
   MenuIdEditDelete,
+  MenuIdEditDuplicate,
   MenuIdAlignLeft,
   MenuIdAlignVCenter,
   MenuIdAlignRight,
@@ -54,10 +56,13 @@ enum MenuID {
   MenuIdOutputPSFile,
   MenuIdOutputEPSFile,
   MenuIdOutputPNGFile,
+  MenuIdOutputCairoEMFFile,
   MenuIdOutputEMFFile,
+  MenuIdOutputEMFClipboard,
   MenuIdOutputPDFFile,
   MenuIdOutputSVGFile,
-  MenuIdToggleStatusBar,
+  MenuIdToggleSidebar,
+  MenuIdToggleStatusbar,
   MenuIdToggleRuler,
   MenuIdToggleScrollbar,
   MenuIdToggleCToolbar,
@@ -112,6 +117,7 @@ enum MouseMode {
   MOUSEZOOM3,
   MOUSEZOOM4,
   MOUSECHANGE,
+  MOUSESCROLLE,
 };
 
 enum pop_up_menu_item_type {
@@ -125,8 +131,7 @@ enum pop_up_menu_item_type {
 struct Viewer
 {
   GtkWidget *Win;
-  GdkWindow *gdk_win;
-  GtkWidget *menu, *VScroll, *HScroll, *popup, *VRuler, *HRuler;
+  GtkWidget *menu, *VScroll, *HScroll, *popup, *VRuler, *HRuler, *side_pane1, *side_pane2, *side_pane3, *main_pane;
   int ShowFrame, ShowLine, ShowRect;
   int Capture, MoveData, KeyMask;
   enum MouseMode MouseMode;
@@ -151,35 +156,35 @@ enum SubWinType {
   TypeInfoWin,
 };
 
-#define SUBWIN_PROTOTYPE enum SubWinType type;				\
-  GtkWidget *Win, *popup, **popup_item;					\
-  GdkWindowState window_state;						\
-  GObject *text;							\
-  int select, num, can_focus;						\
-  void (* update)(int);							\
-  void (* delete)(int);							\
-  void (* setup_dialog)(void *data, struct objlist *obj, int id, int sub_id); \
-  void *dialog;								\
-  gboolean (* ev_key) (GtkWidget *, GdkEvent *, gpointer);		\
-  gboolean (* ev_button) (GtkWidget *, GdkEventButton *, gpointer);	\
+struct SubWin;
 
+struct obj_list_data
+{
+  GtkWidget *popup, **popup_item;
+  GtkWidget *text;
+  int select, can_focus;
+  void (* update)(struct obj_list_data *data, int);
+  void (* delete)(struct obj_list_data *data, int);
+  void (* setup_dialog)(struct obj_list_data *data, int id, int user_data);
+  void *dialog;
+  gboolean (* ev_key) (GtkWidget *, GdkEvent *, gpointer);
+  gboolean (* ev_button) (GtkWidget *, GdkEventButton *, gpointer);
+  struct objlist *obj;
+  struct SubWin *parent;
+  n_list_store *list;
+  int list_col_num;
+  struct obj_list_data *next;
+};
 
 struct SubWin
 {
-  SUBWIN_PROTOTYPE;
-  struct objlist *obj;
-};
-
-
-#define LEGENDNUM 5
-
-struct LegendWin
-{
-  SUBWIN_PROTOTYPE;
-  struct objlist *obj[LEGENDNUM];
-  /* Private member */
-  int legend_type;
-  int legend[LEGENDNUM];
+  enum SubWinType type;
+  GtkWidget *Win;
+  GdkWindowState window_state;
+  union {
+    struct obj_list_data *data;
+    GtkWidget *text;
+  } data;
 };
 
 #define MENU_HISTORY_NUM 10
@@ -202,7 +207,7 @@ struct NgraphApp
   struct Viewer Viewer;
   struct SubWin FileWin;
   struct SubWin AxisWin;
-  struct LegendWin LegendWin;
+  struct SubWin LegendWin;
   struct SubWin MergeWin;
   struct SubWin CoordWin;
   struct SubWin InfoWin;
@@ -238,6 +243,7 @@ void set_draw_lock(int lock);
 int find_gra2gdk_inst(struct objlist **o, N_VALUE **i, struct objlist **ro, int *routput, struct gra2cairo_local **rlocal);
 void set_axis_undo_button_sensitivity(int state);
 void set_modified_state(int state);
+void set_focus_insensitive(const struct Viewer *d);
 void set_focus_sensitivity(const struct Viewer *d);
 void window_action_set_active(enum SubWinType type, int state);
 void window_action_toggle(enum SubWinType type);
