@@ -48,13 +48,13 @@
 #include "x11view.h"
 
 static n_list_store Mlist[] = {
-  {"",         G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden",      FALSE},
-  {"#",        G_TYPE_INT,     TRUE, FALSE, "id",          FALSE},
-  {N_("file"), G_TYPE_STRING,  TRUE, TRUE,  "file",        FALSE},
-  {N_("top"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "top_margin",  FALSE, - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {N_("left"), G_TYPE_DOUBLE,  TRUE, TRUE,  "left_margin", FALSE, - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
-  {N_("zoom"), G_TYPE_DOUBLE,  TRUE, TRUE,  "zoom",        FALSE,                0, SPIN_ENTRY_MAX, 100, 1000},
-  {"^#",       G_TYPE_INT,     TRUE, FALSE, "oid",         FALSE},
+  {" ",        G_TYPE_BOOLEAN, TRUE, TRUE,  "hidden"},
+  {"#",        G_TYPE_INT,     TRUE, FALSE, "id"},
+  {N_("file"), G_TYPE_STRING,  TRUE, TRUE,  "file"},
+  {N_("top"),  G_TYPE_DOUBLE,  TRUE, TRUE,  "top_margin",  - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("left"), G_TYPE_DOUBLE,  TRUE, TRUE,  "left_margin", - SPIN_ENTRY_MAX, SPIN_ENTRY_MAX, 100, 1000},
+  {N_("zoom"), G_TYPE_DOUBLE,  TRUE, TRUE,  "zoom",                       0, SPIN_ENTRY_MAX, 100, 1000},
+  {"^#",       G_TYPE_INT,     TRUE, FALSE, "oid"},
 };
 
 #define MERG_WIN_COL_NUM (sizeof(Mlist)/sizeof(*Mlist))
@@ -63,7 +63,7 @@ static n_list_store Mlist[] = {
 #define MERG_WIN_COL_ID     1
 #define MERG_WIN_COL_FILE   2
 
-static void merge_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row);
+static void merge_list_set_val(struct obj_list_data *d, GtkTreeIter *iter, int row);
 
 static struct subwin_popup_list Popup_list[] = {
   {GTK_STOCK_ADD,         G_CALLBACK(CmMergeOpen), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
@@ -71,7 +71,6 @@ static struct subwin_popup_list Popup_list[] = {
   {GTK_STOCK_CLOSE,       G_CALLBACK(list_sub_window_delete), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, 0, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {"_Focus",              G_CALLBACK(list_sub_window_focus), FALSE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
-  {N_("_Show"),           G_CALLBACK(list_sub_window_hide), FALSE, NULL, POP_UP_MENU_ITEM_TYPE_CHECK},
   {GTK_STOCK_PREFERENCES, G_CALLBACK(list_sub_window_update), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
   {NULL, NULL, 0, NULL, POP_UP_MENU_ITEM_TYPE_SEPARATOR},
   {GTK_STOCK_GOTO_TOP,    G_CALLBACK(list_sub_window_move_top), TRUE, NULL, POP_UP_MENU_ITEM_TYPE_NORMAL},
@@ -81,11 +80,10 @@ static struct subwin_popup_list Popup_list[] = {
 };
 
 #define POPUP_ITEM_NUM (sizeof(Popup_list) / sizeof(*Popup_list))
-#define POPUP_ITEM_HIDE 5
-#define POPUP_ITEM_TOP 8
-#define POPUP_ITEM_UP 9
-#define POPUP_ITEM_DOWN 10
-#define POPUP_ITEM_BOTTOM 11
+#define POPUP_ITEM_TOP 7
+#define POPUP_ITEM_UP 8
+#define POPUP_ITEM_DOWN 9
+#define POPUP_ITEM_BOTTOM 10
 
 
 static void
@@ -191,15 +189,15 @@ MergeDialogClose(GtkWidget *w, void *data)
 }
 
 void
-MergeDialog(void *data, struct objlist *obj, int id, int sub_id)
+MergeDialog(struct obj_list_data *data, int id, int user_data)
 {
   struct MergeDialog *d;
 
-  d = (struct MergeDialog *) data;
+  d = (struct MergeDialog *) data->dialog;
 
   d->SetupWindow = MergeDialogSetup;
   d->CloseWindow = MergeDialogClose;
-  d->Obj = obj;
+  d->Obj = data->obj;
   d->Id = id;
 }
 
@@ -224,7 +222,7 @@ CmMergeOpen(GtkAction *w, gpointer client_data)
   if (id >= 0) {
     changefilename(name);
     putobj(obj, "file", id, name);
-    MergeDialog(&DlgMerge, obj, id, -1);
+    MergeDialog(NgraphApp.MergeWin.data.data, id, -1);
     ret = DialogExecute(TopLevel, &DlgMerge);
     if ((ret == IDDELETE) || (ret == IDCANCEL)) {
       delobj(obj, id);
@@ -234,7 +232,7 @@ CmMergeOpen(GtkAction *w, gpointer client_data)
   } else {
     g_free(name);
   }
-  MergeWinUpdate(TRUE);
+  MergeWinUpdate(NgraphApp.MergeWin.data.data, TRUE);
 }
 
 void
@@ -259,7 +257,7 @@ CmMergeClose(GtkAction *w, gpointer client_data)
       delobj(obj, array[i]);
       set_graph_modified();
     }
-    MergeWinUpdate(TRUE);
+    MergeWinUpdate(NgraphApp.MergeWin.data.data, TRUE);
   }
   arraydel(&farray);
 }
@@ -283,7 +281,7 @@ CmMergeUpdate(GtkAction *w, gpointer client_data)
     num = arraynum(&farray);
     array = arraydata(&farray);
     for (i = 0; i < num; i++) {
-      MergeDialog(&DlgMerge, obj, array[i], -1);
+      MergeDialog(NgraphApp.MergeWin.data.data, array[i], -1);
       if (DialogExecute(TopLevel, &DlgMerge) == IDDELETE) {
 	delobj(obj, array[i]);
 	set_graph_modified();
@@ -291,18 +289,15 @@ CmMergeUpdate(GtkAction *w, gpointer client_data)
 	  array[j]--;
       }
     }
-    MergeWinUpdate(TRUE);
+    MergeWinUpdate(NgraphApp.MergeWin.data.data, TRUE);
   }
   arraydel(&farray);
 }
 
 void
-MergeWinUpdate(int clear)
+MergeWinUpdate(struct obj_list_data *d, int clear)
 {
-  struct SubWin *d;
-
-  d = &(NgraphApp.MergeWin);
-  if (GTK_WIDGET(d->text) == NULL)
+  if (d == NULL)
     return;
 
   if (list_sub_window_must_rebuild(d)) {
@@ -317,7 +312,7 @@ MergeWinUpdate(int clear)
 }
 
 static void
-merge_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row)
+merge_list_set_val(struct obj_list_data *d, GtkTreeIter *iter, int row)
 {
   int cx;
   unsigned int i = 0;
@@ -356,30 +351,25 @@ static void
 popup_show_cb(GtkWidget *widget, gpointer user_data)
 {
   unsigned int i;
-  int sel;
-  struct SubWin *d;
+  int sel, num;
+  struct obj_list_data *d;
 
-  d = (struct SubWin *) user_data;
+  d = (struct obj_list_data *) user_data;
 
   sel = d->select;
+  num = chkobjlastinst(d->obj);
   for (i = 1; i < POPUP_ITEM_NUM; i++) {
     switch (i) {
     case POPUP_ITEM_TOP:
     case POPUP_ITEM_UP:
-      gtk_widget_set_sensitive(d->popup_item[i], sel > 0 && sel <= d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel > 0 && sel <= num);
       break;
     case POPUP_ITEM_DOWN:
     case POPUP_ITEM_BOTTOM:
-      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel < d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel < num);
       break;
-    case POPUP_ITEM_HIDE:
-      if (sel >= 0 && sel <= d->num) {
-	int hidden;
-	getobj(d->obj, "hidden", sel, 0, NULL, &hidden);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(d->popup_item[i]), ! hidden);
-      }
     default:
-      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel <= d->num);
+      gtk_widget_set_sensitive(d->popup_item[i], sel >= 0 && sel <= num);
     }
   }
 }
@@ -409,7 +399,7 @@ init_dnd(struct SubWin *d)
     {"text/uri-list", 0, DROP_TYPE_FILE},
   };
 
-  widget = d->Win;
+  widget = d->data.data->text;
 
   gtk_drag_dest_set(widget, GTK_DEST_DEFAULT_ALL, target, sizeof(target) / sizeof(*target), GDK_ACTION_COPY);
   g_signal_connect(widget, "drag-data-received", G_CALLBACK(drag_drop_cb), NULL);
@@ -438,25 +428,18 @@ CmMergeWindow(GtkToggleAction *action, gpointer client_data)
     return;
   }
 
-  d->update = MergeWinUpdate;
-  d->setup_dialog = MergeDialog;
-  d->dialog = &DlgMerge;
-
   list_sub_window_create(d, "Merge Window", MERG_WIN_COL_NUM, Mlist, Mergewin_xpm, Mergewin48_xpm);
 
-  d->obj = chkobject("merge");
-  d->num = chkobjlastinst(d->obj);
+  d->data.data->update = MergeWinUpdate;
+  d->data.data->setup_dialog = MergeDialog;
+  d->data.data->dialog = &DlgMerge;
+  d->data.data->obj = chkobject("merge");
 
-  sub_win_create_popup_menu(d, POPUP_ITEM_NUM,  Popup_list, G_CALLBACK(popup_show_cb));
+  sub_win_create_popup_menu(d->data.data, POPUP_ITEM_NUM,  Popup_list, G_CALLBACK(popup_show_cb));
 
   init_dnd(d);
 
-  gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->text), TRUE);
-  gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->text), MERG_WIN_COL_FILE);
-  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(d->text), MERG_WIN_COL_FILE);
-
-  sub_window_show_all(d);
-  sub_window_set_geometry(d, TRUE);
-
-  MergeWinUpdate(TRUE);
+  gtk_tree_view_set_enable_search(GTK_TREE_VIEW(d->data.data->text), TRUE);
+  gtk_tree_view_set_search_column(GTK_TREE_VIEW(d->data.data->text), MERG_WIN_COL_FILE);
+  gtk_tree_view_set_tooltip_column(GTK_TREE_VIEW(d->data.data->text), MERG_WIN_COL_FILE);
 }
