@@ -53,7 +53,6 @@
 #include "x11cood.h"
 #include "x11info.h"
 
-#define BUF_SIZE 64
 #define MESSAGE_BUF_SIZE 4096
 
 #define WIN_SIZE_MIN 100
@@ -108,6 +107,8 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->external_driver), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->addin_script), FALSE);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->misc), FALSE);
+
+  gtk_widget_set_sensitive(d->child_geometry, ! Menulocal.single_window_mode);
 }
 
 static int
@@ -277,7 +278,6 @@ SetScriptDialogSetupItem(GtkWidget *w, struct SetScriptDialog *d)
   }
 }
 
-#if USE_ENTRY_ICON
 static void
 SetScriptDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
@@ -289,9 +289,6 @@ SetScriptDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *even
   }
   g_free(file);
 }
-#else
-GCallback SetScriptDialogBrowse = NULL;
-#endif
 
 static void
 SetScriptDialogSetup(GtkWidget *wi, void *data, int makewidget)
@@ -472,9 +469,9 @@ PrefScriptDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
   struct PrefScriptDialog *d;
   n_list_store list[] = {
-    {N_("name"), G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
-    {N_("file"), G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
-    {N_("description"), G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
+    {N_("name"), G_TYPE_STRING, TRUE, FALSE, NULL},
+    {N_("file"), G_TYPE_STRING, TRUE, FALSE, NULL},
+    {N_("description"), G_TYPE_STRING, TRUE, FALSE, NULL},
   };
 
   d = (struct PrefScriptDialog *) data;
@@ -519,7 +516,6 @@ SetDriverDialogSetupItem(GtkWidget *w, struct SetDriverDialog *d)
   gtk_entry_set_text(GTK_ENTRY(d->ext), CHK_STR(d->Driver->ext));
 }
 
-#if USE_ENTRY_ICON
 static void
 SetDriverDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
@@ -533,9 +529,6 @@ SetDriverDialogBrowse(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *even
   }
   g_free(file);
 }
-#else
-GCallback SetDriverDialogBrowse = NULL;
-#endif
 
 static void
 SetDriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
@@ -681,7 +674,7 @@ PrefDriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
   struct PrefDriverDialog *d;
   n_list_store list[] = {
-    {N_("Driver"), G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
+    {N_("Driver"), G_TYPE_STRING, TRUE, FALSE, NULL},
   };
 
   d = (struct PrefDriverDialog *) data;
@@ -907,7 +900,7 @@ FontSettingDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   if (makewidget) {
     n_list_store list[] = {
-      {N_("Font name"),  G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
+      {N_("Font name"),  G_TYPE_STRING, TRUE, FALSE, NULL},
     };
 
     int j;
@@ -1179,9 +1172,9 @@ PrefFontDialogSetup(GtkWidget *wi, void *data, int makewidget)
   GtkWidget *vbox;
   struct PrefFontDialog *d;
   n_list_store list[] = {
-    {N_("alias"), G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
-    {N_("name"),  G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
-    {N_("alternative fonts"),  G_TYPE_STRING, TRUE, FALSE, NULL, FALSE},
+    {N_("alias"), G_TYPE_STRING, TRUE, FALSE, NULL},
+    {N_("name"),  G_TYPE_STRING, TRUE, FALSE, NULL},
+    {N_("alternative fonts"),  G_TYPE_STRING, TRUE, FALSE, NULL},
   };
 
   d = (struct PrefFontDialog *) data;
@@ -1243,7 +1236,7 @@ MiscDialogSetupItem(GtkWidget *w, struct MiscDialog *d)
   if (Menulocal.expanddir)
     gtk_entry_set_text(GTK_ENTRY(d->expanddir), Menulocal.expanddir);
 
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->ignorepath), Menulocal.ignorepath);
+  combo_box_set_active(d->loadpath, Menulocal.loadpath);
 
   spin_entry_set_val(d->hist_size, Menulocal.hist_size);
   spin_entry_set_val(d->info_size, Menulocal.info_size);
@@ -1262,9 +1255,10 @@ MiscDialogSetupItem(GtkWidget *w, struct MiscDialog *d)
   }
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), Menulocal.use_opacity);
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->select_data), Menulocal.select_data);
 }
 
-#if USE_ENTRY_ICON
 static void
 set_file_in_entry(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
@@ -1278,9 +1272,6 @@ set_file_in_entry(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, g
   }
   g_free(file);
 }
-#else
-GCallback set_file_in_entry = NULL;
-#endif
 
 static void
 MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
@@ -1339,10 +1330,10 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     i = 0;
     w = combo_box_create();
     add_widget_to_table(table, w, _("_Path:"), FALSE, i++);
-    d->path = w;
     for (j = 0; pathchar[j]; j++) {
-      combo_box_append_text(d->path, _(pathchar[j]));
+      combo_box_append_text(w, _(pathchar[j]));
     }
+    d->path = w;
 
     w = gtk_check_button_new_with_mnemonic(_("include _Data file"));
     d->datafile = w;
@@ -1373,9 +1364,12 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, _("_Expand directory:"), TRUE, i++);
     d->expanddir = w;
 
-    w = gtk_check_button_new_with_mnemonic(_("_Ignore file path"));
-    add_widget_to_table(table, w, NULL, FALSE, i++);
-    d->ignorepath = w;
+    w = combo_box_create();
+    add_widget_to_table(table, w, _("_Path:"), FALSE, i++);
+    for (j = 0; LoadPathStr[j]; j++) {
+      combo_box_append_text(w, _(LoadPathStr[j]));
+    }
+    d->loadpath = w;
 
     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_box_pack_start(GTK_BOX(vbox2), frame, TRUE, TRUE, 4);
@@ -1402,7 +1396,7 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     d->hist_size = w;
 
     w = create_spin_entry(1, INFOWIN_SIZE_MAX, 1, FALSE, TRUE);
-    add_widget_to_table(table, w, _("_Length of information window:"), FALSE, i++);
+    add_widget_to_table(table, w, _("_Length of information view:"), FALSE, i++);
     d->info_size = w;
 
     w = create_spin_entry(0, SPIN_ENTRY_MAX, 1, FALSE, TRUE);
@@ -1422,11 +1416,11 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     i = 0;
     w = gtk_font_button_new();
-    add_widget_to_table(table, w, _("_Coordinate window:"), FALSE, i++);
+    add_widget_to_table(table, w, _("_Coordinate view:"), FALSE, i++);
     d->coordwin_font = w;
 
     w = gtk_font_button_new();
-    add_widget_to_table(table, w, _("_Information window:"), FALSE, i++);
+    add_widget_to_table(table, w, _("_Information view:"), FALSE, i++);
     d->infowin_font = w;
 
     w = gtk_font_button_new();
@@ -1452,6 +1446,10 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, NULL, FALSE, i++);
     d->use_opacity = w;
 
+    w = gtk_check_button_new_with_mnemonic(_("_Show select data dialog on exporting"));
+    add_widget_to_table(table, w, NULL, FALSE, i++);
+    d->select_data = w;
+
     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_box_pack_start(GTK_BOX(vbox2), frame, TRUE, TRUE, 4);
 
@@ -1549,8 +1547,7 @@ MiscDialogClose(GtkWidget *w, void *data)
     }
   }
 
-  Menulocal.ignorepath =
-    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->ignorepath));
+  Menulocal.loadpath = combo_box_get_active(d->loadpath);
 
   a = spin_entry_get_val(d->hist_size);
   if (a <= HIST_SIZE_MAX && a > 0)
@@ -1576,6 +1573,8 @@ MiscDialogClose(GtkWidget *w, void *data)
   Menulocal.use_opacity = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->use_opacity));
   putobj(d->Obj, "use_opacity", d->Id, &Menulocal.use_opacity);
 
+  Menulocal.select_data = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->select_data));
+
   d->ret = ret;
 
   if (d->ret == IDSAVE) {
