@@ -116,7 +116,6 @@ DriverDialogSelectCB(GtkWidget *wi, gpointer client_data)
   }
 }
 
-#if USE_ENTRY_ICON
 static void
 DriverDialogBrowseCB(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event, gpointer user_data)
 {
@@ -134,9 +133,6 @@ DriverDialogBrowseCB(GtkEntry *w, GtkEntryIconPosition icon_pos, GdkEvent *event
   }
   g_free(file);
 }
-#else
-GCallback DriverDialogBrowseCB = NULL;
-#endif
 
 static void
 DriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
@@ -320,7 +316,6 @@ OutputImageDialogSetupItem(GtkWidget *w, struct OutputImageDialog *d)
     combo_box_append_text(d->version, "--------");
 
     set_widget_sensitivity_with_label(d->dpi, TRUE);
-
     gtk_widget_set_sensitive(d->version, FALSE);
     gtk_widget_set_sensitive(vlabel, FALSE);
     gtk_widget_set_sensitive(d->t2p, FALSE);
@@ -336,7 +331,6 @@ OutputImageDialogSetupItem(GtkWidget *w, struct OutputImageDialog *d)
     set_widget_sensitivity_with_label(d->dpi, FALSE);
     gtk_widget_set_sensitive(d->version, FALSE);
     gtk_widget_set_sensitive(vlabel, FALSE);
-
     gtk_widget_set_sensitive(d->t2p, TRUE);
 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), 72);
@@ -349,7 +343,6 @@ OutputImageDialogSetupItem(GtkWidget *w, struct OutputImageDialog *d)
     }
 
     set_widget_sensitivity_with_label(d->dpi, FALSE);
-
     gtk_widget_set_sensitive(d->version, TRUE);
     gtk_widget_set_sensitive(vlabel, TRUE);
     gtk_widget_set_sensitive(d->t2p, TRUE);
@@ -361,13 +354,12 @@ OutputImageDialogSetupItem(GtkWidget *w, struct OutputImageDialog *d)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->use_opacity), Menulocal.use_opacity);
     break;
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
+  case MenuIdOutputCairoEMFFile:
     combo_box_append_text(d->version, "--------");
 
+    set_widget_sensitivity_with_label(d->dpi, TRUE);
     gtk_widget_set_sensitive(d->version, FALSE);
     gtk_widget_set_sensitive(vlabel, FALSE);
-
-    set_widget_sensitivity_with_label(d->dpi, TRUE);
     gtk_widget_set_sensitive(d->t2p, TRUE);
 
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(d->dpi), Menulocal.emf_dpi);
@@ -422,7 +414,7 @@ OutputImageDialogSetup(GtkWidget *wi, void *data, int makewidget)
     title = N_("Cairo SVG Output");
     break;
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
+  case MenuIdOutputCairoEMFFile:
     title = N_("Cairo EMF Output");
     break;
 #endif	/* CAIRO_HAS_WIN32_SURFACE */
@@ -482,7 +474,7 @@ OutputImageDialogClose(GtkWidget *w, void *data)
     Menulocal.png_dpi = d->Dpi;
     break;
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
+  case MenuIdOutputCairoEMFFile:
     d->Version = TYPE_EMF;
     Menulocal.emf_dpi = d->Dpi;
     break;
@@ -533,9 +525,7 @@ draw_gra(struct objlist *graobj, int id, char *msg, int close)
 
   ProgressDialogFinalize();
   ResetStatusBar();
-  if (NgraphApp.Viewer.gdk_win) {
-    gdk_window_invalidate_rect(NgraphApp.Viewer.gdk_win, NULL, FALSE);
-  }
+  main_window_redraw();
 }
 
 static void
@@ -701,8 +691,11 @@ CmOutputPrinter(int select_file, int show_dialog)
 
   delobj(graobj, id);
   delobj(g2wobj, g2wid);
-}
 
+  if (select_file) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
+  }
+}
 
 void
 CmOutputDriver(void)
@@ -716,7 +709,7 @@ CmOutputDriver(void)
   if (Menulock || Globallock)
     return;
 
-  if (! SetFileHidden())
+  if (Menulocal.select_data && ! SetFileHidden())
     return;
 
   FileAutoScale();
@@ -750,6 +743,10 @@ CmOutputDriver(void)
     ResetStatusBar();
   }
   delobj(g2wobj, g2wid);
+
+  if (Menulocal.select_data) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
+  }
 }
 
 void
@@ -862,7 +859,7 @@ CmPrintGRAFile(void)
     return;
   }
 
-  if (! SetFileHidden())
+  if (Menulocal.select_data && ! SetFileHidden())
     return;
 
   FileAutoScale();
@@ -892,6 +889,10 @@ CmPrintGRAFile(void)
   draw_gra(graobj, id, _("Making GRA file."), TRUE);
   delobj(graobj, id);
   delobj(g2wobj, g2wid);
+
+  if (Menulocal.select_data) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
+  }
 }
 
 static void
@@ -929,8 +930,8 @@ CmOutputImage(int type)
     ext_str = "svg";
     break;
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
-    title = "Save as Scalable Vector Graphics (EMF)";
+  case MenuIdOutputCairoEMFFile:
+    title = "Save as Windows Enhanced Metafile (EMF)";
     ext_str = "emf";
     break;
 #endif	/* CAIRO_HAS_WIN32_SURFACE */
@@ -943,8 +944,9 @@ CmOutputImage(int type)
   tmp = get_base_ngp_name();
   ret = nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp,
 			 &file, FALSE, Menulocal.changedirectory);
-  if (tmp)
+  if (tmp) {
     g_free(tmp);
+  }
 
   if (ret != IDOK) {
     return;
@@ -961,7 +963,7 @@ CmOutputImage(int type)
     return;
   }
 
-  if (! SetFileHidden())
+  if (Menulocal.select_data && ! SetFileHidden())
     return;
 
   FileAutoScale();
@@ -996,7 +998,7 @@ CmOutputImage(int type)
   case MenuIdOutputSVGFile:
   case MenuIdOutputPDFFile:
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
+  case MenuIdOutputCairoEMFFile:
 #endif	/* CAIRO_HAS_WIN32_SURFACE */
     putobj(g2wobj, "text2path", g2wid, &DlgImageOut.text2path);
     break;
@@ -1012,8 +1014,86 @@ CmOutputImage(int type)
   draw_gra(graobj, id, _("Drawing."), TRUE);
   delobj(graobj, id);
   delobj(g2wobj, g2wid);
+
+  if (Menulocal.select_data) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
+  }
 }
 
+#ifdef WINDOWS
+static void
+CmOutputEMF(int type)
+{
+  struct objlist *graobj, *g2wobj;
+  int id, g2wid, g2woid;
+  N_VALUE *g2winst;
+  int ret;
+  char *title, *ext_str;
+  char *file, *tmp;
+
+  if (Menulock || Globallock)
+    return;
+
+  title = "Save as Windows Enhanced Metafile (EMF)";
+
+  file = NULL;
+  if (type == MenuIdOutputEMFFile) {
+    ext_str = "emf";
+    tmp = get_base_ngp_name();
+    ret = nGetSaveFileName(TopLevel, title, ext_str, NULL, tmp,
+			   &file, FALSE, Menulocal.changedirectory);
+    if (tmp) {
+      g_free(tmp);
+    }
+
+    if (ret != IDOK) {
+      return;
+    }
+
+    if (file == NULL) {
+      return;
+    }
+  }
+
+  if (Menulocal.select_data && ! SetFileHidden())
+    return;
+
+  FileAutoScale();
+  AdjustAxis();
+
+  graobj = chkobject("gra");
+  if (graobj == NULL) {
+    g_free(file);
+    return;
+  }
+
+  g2wobj = chkobject("gra2emf");
+  if (g2wobj == NULL) {
+    g_free(file);
+    return;
+  }
+
+  g2wid = newobj(g2wobj);
+  if (g2wid < 0) {
+    g_free(file);
+    return;
+  }
+
+  g2winst = chkobjinst(g2wobj, g2wid);
+  _getobj(g2wobj, "oid", g2winst, &g2woid);
+  id = newobj(graobj);
+  putobj(g2wobj, "file", g2wid, file);
+  init_graobj(graobj, id, "gra2emf", g2woid);
+  draw_gra(graobj, id, _("Drawing."), TRUE);
+  delobj(graobj, id);
+  delobj(g2wobj, g2wid);
+
+  if (Menulocal.select_data) {
+    FileWinUpdate(NgraphApp.FileWin.data.data, TRUE);
+  }
+}
+#endif
+
 void
 CmOutputPrinterB(GtkAction *wi, gpointer client_data)
 {
@@ -1033,9 +1113,15 @@ CmOutputMenu(GtkAction *wi, gpointer client_data)
   case MenuIdOutputPNGFile:
   case MenuIdOutputSVGFile:
 #ifdef CAIRO_HAS_WIN32_SURFACE
-  case MenuIdOutputEMFFile:
+  case MenuIdOutputCairoEMFFile:
 #endif	/* CAIRO_HAS_WIN32_SURFACE */
     CmOutputImage(GPOINTER_TO_INT(client_data));
     break;
+#ifdef WINDOWS
+  case MenuIdOutputEMFFile:
+  case MenuIdOutputEMFClipboard:
+    CmOutputEMF(GPOINTER_TO_INT(client_data));
+    break;
+#endif
   }
 }
