@@ -60,6 +60,7 @@
 #define ERRNEGATIVEWEIGHT 112
 #define ERRCONVERGE 113
 #define ERR_INCONSISTENT_DATA_NUM 114
+#define ERRINTERRUPT 115
 
 static char *fiterrorlist[]={
   "syntax error.",
@@ -93,7 +94,7 @@ static char *fittypechar[]={
 struct fitlocal {
   int id, oid;
   MathEquation *codedf[10];
-  MathEquation *codef;
+  MathEquation *codef, *result_code;
   int dim;
   double coe[11];
   int num;
@@ -122,6 +123,7 @@ fitinit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 
   if ((fitlocal=g_malloc(sizeof(struct fitlocal)))==NULL) return 1;
   fitlocal->codef=NULL;
+  fitlocal->result_code = NULL;
   for (i=0;i<10;i++) fitlocal->codedf[i]=NULL;
   fitlocal->equation=NULL;
   fitlocal->oid = oid;
@@ -142,11 +144,37 @@ fitdone(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   _getobj(obj,"_local",inst,&fitlocal);
   g_free(fitlocal->equation);
   math_equation_free(fitlocal->codef);
+  math_equation_free(fitlocal->result_code);
   for (i = 0; i < 10; i++)
     math_equation_free(fitlocal->codedf[i]);
   return 0;
 }
 
+static int
+set_equation(struct objlist *obj, N_VALUE *inst, struct fitlocal *fitlocal, char *equation)
+{
+  int r;
+  char *old_equation;
+
+  _getobj(obj, "equation", inst, &old_equation);
+
+  r = _putobj(obj, "equation", inst, equation);
+  if (r) {
+    return 1;
+  }
+
+  if (old_equation) {
+    g_free(old_equation);
+  }
+
+  if (fitlocal && fitlocal->result_code) {
+    math_equation_free(fitlocal->result_code);
+    fitlocal->result_code = NULL;
+  }
+
+  return 0;
+}
+
 static void
 show_eqn_error(struct objlist *obj, MathEquation *code, char *math, char *field, int rcode)
 {
@@ -161,6 +189,21 @@ show_eqn_error(struct objlist *obj, MathEquation *code, char *math, char *field,
   }
 }
 
+static int 
+fitequation(struct objlist *obj,N_VALUE *inst,N_VALUE *rval, int argc,char **argv)
+{
+  struct fitlocal *fitlocal;
+
+  _getobj(obj,"_local",inst,&fitlocal);
+
+  if (fitlocal->result_code) {
+    math_equation_free(fitlocal->result_code);
+    fitlocal->result_code = NULL;
+  }
+
+  return 0;
+}
+
 static int 
 fitput(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
            int argc,char **argv)
@@ -168,7 +211,6 @@ fitput(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
   char *field;
   char *math;
   struct fitlocal *fitlocal;
-  char *equation;
   MathEquation *code;
   int rcode;
 
@@ -183,17 +225,8 @@ fitput(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
     if (math!=NULL) {
       MathEquationParametar *prm;
 
-      code = math_equation_basic_new();
-      if (code == NULL)
-	return 1;
-
-      if (math_equation_add_parameter(code, 0, 1, 2, MATH_EQUATION_PARAMETAR_USE_ID)) {
-	math_equation_free(code);
-	return 1;
-      }
-
-      if (math_equation_add_var(code, "X") != 0) {
-	math_equation_free(code);
+      code = ofile_create_math_equation(NULL, 2, FALSE, FALSE, FALSE, FALSE, TRUE);
+      if (code == NULL) {
 	return 1;
       }
 
@@ -231,9 +264,7 @@ fitput(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
       fitlocal->codedf[field[10] - '0'] = code;
     }
   }
-  _getobj(obj,"equation",inst,&equation);
-  if (_putobj(obj,"equation",inst,NULL)) return 1;
-  g_free(equation);
+  if (set_equation(obj, inst, fitlocal, NULL)) return 1;
   return 0;
 }
 
@@ -257,7 +288,7 @@ fit_put_weight_func(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
     return 0;
   }
 
-  code = ofile_create_math_equation(NULL, TRUE, FALSE, TRUE, FALSE, FALSE);
+  code = ofile_create_math_equation(NULL, 3, FALSE, TRUE, FALSE, FALSE, TRUE);
   if (code == NULL)
     return 1;
 
@@ -284,7 +315,7 @@ enum FitError {
 };
 
 static void
-display_equation(char *equation)
+display_equation(const char *equation)
 {
   if (equation == NULL)
     return;
@@ -294,7 +325,93 @@ display_equation(char *equation)
   ndisplaydialog("\n");
 }
 
-enum FitError 
+static int
+show_poly_equation(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe)
+{
+  GString *equation;
+  int i;
+
+  equation = g_string_new("");
+  if (equation == NULL) {
+    return 1;
+  }
+
+  switch (type) {
+  case FIT_TYPE_POLY:
+    for (i = fitlocal->dim - 1; i > 0; i--) {
+      g_string_append_printf(equation,
+			     (i == fitlocal->dim - 1) ? "%.7e*X^%d" : "%+.7e*X^%d",
+			     coe[i], i);
+    }
+    g_string_append_printf(equation, "%+.7e", coe[0]);
+    break;
+  case FIT_TYPE_POW:
+    g_string_printf(equation, "exp(%.7e)*X^%.7e", coe[0], coe[1]);
+    break;
+  case FIT_TYPE_EXP:
+    g_string_printf(equation, "exp(%.7e*X%+.7e)", coe[1], coe[0]);
+    break;
+  case  FIT_TYPE_LOG:
+    g_string_printf(equation, "%.7e*Ln(X)%+.7e", coe[1], coe[0]);
+    break;
+  case FIT_TYPE_USER:
+    /* never reached */
+    break;
+  }
+  fitlocal->equation = g_string_free(equation, FALSE);
+
+  return 0;
+}
+
+static int
+show_poly_result(struct fitlocal *fitlocal, enum FIT_OBJ_TYPE type, vector coe, int num)
+{
+  int j;
+  GString *info;
+
+  info = g_string_new("");
+  if (info == NULL) {
+    return 1;
+  }
+
+  g_string_append_printf(info, "--------\nfit:%d (^%d)\n", fitlocal->id, fitlocal->oid);
+  switch (type) {
+  case FIT_TYPE_POLY:
+    g_string_append_printf(info,"Eq: %%0i*X^i (i=0-%d)\n\n", fitlocal->dim - 1);
+    break;
+  case FIT_TYPE_POW:
+    g_string_append(info,"Eq: exp(%00)*X^%01\n\n");
+    break;
+  case FIT_TYPE_EXP:
+    g_string_append(info,"Eq: exp(%01*X+%00)\n\n");
+    break;
+  case FIT_TYPE_LOG:
+    g_string_append(info,"Eq: %01*Ln(X)+%00\n\n");
+    break;
+  case FIT_TYPE_USER:
+    /* never reached */
+    break;
+  }
+  for (j = 0; j < fitlocal->dim; j++) {
+    g_string_append_printf(info, "       %%0%d = %+.7e\n", j, coe[j]);
+  }
+  g_string_append_c(info, '\n');
+  g_string_append_printf(info, "    points = %d\n", fitlocal->num);
+  g_string_append_printf(info, "    <DY^2> = %+.7e\n", fitlocal->derror);
+  if (fitlocal->correlation >= 0) {
+    g_string_append_printf(info, "|r| or |R| = %+.7e\n", fitlocal->correlation);
+  } else {
+    g_string_append(info, "|r| or |R| = -------------\n");
+  }
+  ndisplaydialog(info->str);
+  g_string_free(info, TRUE);
+
+  display_equation(fitlocal->equation);
+
+  return 0;
+}
+
+static enum FitError
 fitpoly(struct fitlocal *fitlocal,
 	enum FIT_OBJ_TYPE type,int dimension,int through,double x0,double y0,
 	double *data,int num,int disp,int weight,double *wdata)
@@ -311,8 +428,6 @@ fitpoly(struct fitlocal *fitlocal,
   double yy,y1,y2,derror,sy,correlation,wt,sum;
   vector b,x1,x2,coe;
   matrix m;
-  char *equation;
-  char buf[1024];
 
   if (type == FIT_TYPE_POLY) dim=dimension+1;
   else dim=2;
@@ -375,74 +490,111 @@ fitpoly(struct fitlocal *fitlocal,
   fitlocal->correlation=correlation;
   fitlocal->num=num;
 
-#define EQUATION_BUF_SIZE 512
-  if ((equation=g_malloc(EQUATION_BUF_SIZE))==NULL) return FitError_Fatal;
-  equation[0]='\0';
-  j=0;
-
-  switch (type) {
-  case FIT_TYPE_POLY:
-    for (i = dim - 1; i > 0; i--) 
-      j += snprintf(equation + j, EQUATION_BUF_SIZE - j,
-		    (i == dim - 1) ? "%.15e*X^%d" : "%+.15e*X^%d",
-		    coe[i], i);
-    snprintf(equation + j, EQUATION_BUF_SIZE - j, "%+.15e", coe[0]);
-    break;
-  case FIT_TYPE_POW:
-    sprintf(equation, "exp(%.15e)*X^%.15e", coe[0], coe[1]);
-    break;
-  case FIT_TYPE_EXP:
-    sprintf(equation, "exp(%.15e*X%+.15e)", coe[1], coe[0]);
-    break;
-  case  FIT_TYPE_LOG:
-    sprintf(equation, "%.15e*Ln(X)%+.15e", coe[1], coe[0]);
-    break;
-  case FIT_TYPE_USER:
-    /* never reached */
-    break;
+  if (show_poly_equation(fitlocal, type, coe)) {
+    return FitError_Fatal;
   }
-  fitlocal->equation = equation;
 
-  if (disp) {
-    i=0;
-    i += sprintf(buf + i, "--------\nfit:%d (^%d)\n", fitlocal->id, fitlocal->oid);
-    switch (type) {
-    case FIT_TYPE_POLY:
-      i += sprintf(buf+i,"Eq: %%0i*X^i (i=0-%d)\n\n",dim-1);
-      break;
-    case FIT_TYPE_POW:
-      i += sprintf(buf + i,"Eq: exp(%%00)*X^%%01\n\n");
-      break;
-    case FIT_TYPE_EXP:
-      i += sprintf(buf + i,"Eq: exp(%%01*X+%%00)\n\n");
-      break;
-    case FIT_TYPE_LOG:
-      i += sprintf(buf + i,"Eq: %%01*Ln(X)+%%00\n\n");
-      break;
-    case FIT_TYPE_USER:
-      /* never reached */
-      break;
-    }
-    for (j=0;j<dim;j++)
-      i+=sprintf(buf+i,"       %%0%d = %+.7e\n",j,coe[j]);
-    i+=sprintf(buf+i,"\n");
-    i+=sprintf(buf+i,"    points = %d\n",num);
-    i+=sprintf(buf+i,"    <DY^2> = %+.7e\n",derror);
-    if (correlation>=0)
-      i+=sprintf(buf+i,"|r| or |R| = %+.7e\n",correlation);
-    else
-      i+=sprintf(buf+i,"|r| or |R| = -------------\n");
-    ndisplaydialog(buf);
-
-    display_equation(equation);
+  if (disp && show_poly_result(fitlocal, type, coe, derror)) {
+    return FitError_Fatal;
   }
 
 
   return FitError_Success;
 }
 
-enum FitError 
-fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
+static int
+show_user_result(struct fitlocal *fitlocal, const char *func, int dim, int *tbl, MathValue *par, int n, double dxxc, double derror, double correlation)
+{
+  GString *info;
+  int j;
+
+  info = g_string_sized_new(1024);
+  if (info == NULL) {
+    return 1;
+  }
+  g_string_append_printf(info, "--------\nfit:%d (^%d)\n", fitlocal->id, fitlocal->oid);
+  g_string_append(info, "Eq: User defined\n");
+  g_string_append_printf(info, "    %s\n\n", func);
+  for (j = 0; j < dim; j++) {
+    g_string_append_printf(info, "       %%0%d = %+.7e\n", tbl[j],  par[tbl[j]].val);
+  }
+  g_string_append_c(info, '\n');
+  g_string_append_printf(info, "    points = %d\n", n);
+  g_string_append_printf(info, "     delta = %+.7e\n", dxxc);
+  g_string_append_printf(info, "    <DY^2> = %+.7e\n", derror);
+  if (correlation >= 0) {
+    g_string_append_printf(info, "|r| or |R| = %+.7e\n", correlation);
+  } else {
+    g_string_append(info, "|r| or |R| = -------------\n");
+  }
+  ndisplaydialog(info->str);
+  g_string_free(info, TRUE);
+  return 0;
+}
+
+static int
+show_user_equation(struct fitlocal *fitlocal, const char *func, MathValue *par, int disp)
+{
+  int i, prev_char;
+  GString *equation;
+
+  equation = g_string_sized_new(256);
+  if (equation == NULL) {
+    return 1;
+  }
+
+  prev_char = '\0';
+  for (i = 0; func[i] != '\0'; i++) {
+    double val;
+    char *format;
+    int prm_index;
+
+    switch (func[i]) {
+    case '%':
+      if (isdigit(func[i + 1]) && isdigit(func[i + 2]) && isdigit(func[i + 3])) {
+	prm_index = (func[i + 1] - '0') * 100 + (func[i + 2] - '0') * 10 + (func[i + 3] - '0');
+	i += 3;
+      } else if (isdigit(func[i + 1]) && isdigit(func[i + 2])) {
+	prm_index = (func[i + 1] - '0') * 10 + (func[i + 2] - '0');
+	i += 2;
+      } else {
+	prm_index = (func[i + 1] - '0');
+	i += 1;
+      }
+
+      val = par[prm_index].val;
+      switch (prev_char) {
+      case '-':
+	val = - val;
+	/* fall-through */
+      case '+':
+	g_string_truncate(equation, equation->len - 1);
+	format = "%+.7e";
+	break;
+      default:
+	format = "%.7e";
+      }
+      prev_char = '\0';
+      g_string_append_printf(equation, format, val);
+      break;
+    case ' ':
+      break;
+    default:
+      prev_char = func[i];
+      g_string_append_c(equation, toupper(prev_char));
+    }
+  }
+
+  fitlocal->equation = g_string_free(equation, FALSE);
+  if (disp) {
+    display_equation(fitlocal->equation);
+  }
+
+  return 0;
+}
+
+static enum FitError 
+fituser(struct objlist *obj,struct fitlocal *fitlocal, const char *func,
 	int deriv,double converge,double *data,int num,int disp,
 	int weight,double *wdata)
 /*
@@ -458,7 +610,7 @@ fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
          FitError_Fatal
 */
 {
-  int ecode, prev_char, equation_length;
+  int ecode;
   int *needdata;
   int tbl[10],dim,n,count,rcode,err,err2,err3;
   double yy,y,y1,y2,y3,sy,spx,spy,dxx,dxxc,xx,derror,correlation;
@@ -466,8 +618,7 @@ fituser(struct objlist *obj,struct fitlocal *fitlocal,char *func,
   MathValue par[10], par2[10], var;
   MathEquationParametar *prm;
   matrix m;
-  int i,j,k,pnum;
-  char *equation;
+  int i,j,k;
   char buf[1024];
   double wt,sum;
 /*
@@ -708,89 +859,21 @@ repeat:
 
   } while ((dxxc>xx*converge/100) && ((xx>1e-6) || (dxxc>1e-6*converge/100)));
 
-  if (disp) {
-    i=0;
-    i += sprintf(buf + i, "--------\nfit:%d (^%d)\n", fitlocal->id, fitlocal->oid);
-    i+=sprintf(buf+i,"Eq: User defined\n");
-    i+=sprintf(buf+i,"    %s\n\n", func);
-    for (j=0;j<dim;j++) {
-      i += sprintf(buf+i,"       %%0%d = %+.7e\n",tbl[j], par[tbl[j]].val);
-    }
-    i+=sprintf(buf+i,"\n");
-    i+=sprintf(buf+i,"    points = %d\n",n);
-    i+=sprintf(buf+i,"     delta = %+.7e\n",dxxc);
-    i+=sprintf(buf+i,"    <DY^2> = %+.7e\n",derror);
-    if (correlation>=0)
-      i+=sprintf(buf+i,"|r| or |R| = %+.7e\n",correlation);
-    else
-      i+=sprintf(buf+i,"|r| or |R| = -------------\n");
-    ndisplaydialog(buf);
+  if (disp && show_user_result(fitlocal, func, dim, tbl, par, n, dxxc, derror, correlation)) {;
+    return FitError_Fatal;
   }
 
 errexit:
   if ((ecode==FitError_Success) || (ecode==FitError_Range)) {
-    for (i = 0; i < 10; i++) fitlocal->coe[i] = par[i].val;
+    for (i = 0; i < 10; i++) {
+      fitlocal->coe[i] = par[i].val;
+    }
     fitlocal->dim=dim;
     fitlocal->derror=derror;
     fitlocal->correlation=correlation;
     fitlocal->num=n;
-    pnum=0;
-    for (i=0;func[i]!='\0';i++) {
-      if (func[i]=='%') {
-        pnum++;
-        i+=2;
-      }
-    }
-    equation_length = strlen(func) + 25 * pnum + 1;
-    equation=g_malloc(equation_length);
-    if (equation == NULL)
-      return FitError_Fatal;
-    j = 0;
-    prev_char = '\0';
-    for (i = 0; func[i] != '\0'; i++) {
-      double val;
-      char *format;
-
-      switch (func[i]) {
-      case '%':
-        if (isdigit(func[i + 1]) && isdigit(func[i + 2]) && isdigit(func[i + 3])) {
-          pnum = (func[i + 1] - '0') * 100 + (func[i + 2] - '0') * 10 + (func[i + 3] - '0');
-          i += 3;
-        } else if (isdigit(func[i + 1]) && isdigit(func[i + 2])) {
-          pnum = (func[i + 1] - '0') * 10 + (func[i + 2] - '0');
-          i += 2;
-        } else {
-          pnum = (func[i + 1] - '0');
-          i += 1;
-        }
-
-	val = par[pnum].val;
-	switch (prev_char) {
-	case '-':
-	  val = - val;
-	  /* fall-through */
-	case '+':
-	  j--;
-	  format = "%+.15e";
-	  break;
-	default:
-	  format = "%.15e";
-	}
-	prev_char = '\0';
-	j += snprintf(equation + j, equation_length - j, format, val);
-	break;
-      case ' ':
-	break;
-      default:
-        equation[j] = toupper(func[i]);
-	prev_char = func[i];
-        j++;
-      }
-    }
-    equation[j] = '\0';
-    fitlocal->equation = equation;
-    if (disp) {
-      display_equation(equation);
+    if (show_user_equation(fitlocal, func, par, disp)) {
+      ecode = FitError_Fatal;
     }
   }
   return ecode;
@@ -813,12 +896,11 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   int weight,anum;
 
   if (_exeparent(obj,(char *)argv[1],inst,rval,argc,argv)) return 1;
-  _getobj(obj,"equation",inst,&equation);
-  g_free(equation);
-  if (_putobj(obj,"equation",inst,NULL)) return 1;
+  _getobj(obj,"_local",inst,&fitlocal);
+  if (set_equation(obj, inst, fitlocal, NULL)) return 1;
   equation = g_strdup("undef");
   if (equation == NULL) return 1;
-  if (_putobj(obj,"equation",inst,equation)) {
+  if (set_equation(obj, inst, fitlocal, equation)) {
     g_free(equation);
     return 1;
   }
@@ -835,7 +917,6 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     if (_putobj(obj, prm, inst, &pp)) return 1;
   }
 
-  _getobj(obj,"_local",inst,&fitlocal);
   _getobj(obj,"type",inst,&type);
   _getobj(obj,"through_point",inst,&through);
   _getobj(obj,"point_x",inst,&x0);
@@ -985,6 +1066,9 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
       ecode = ERRCONVERGE;
       break;
     */
+  case FitError_Interrupt:
+    ecode = ERRINTERRUPT;
+    break;
   default:
     ecode = 0;
   }
@@ -1001,62 +1085,59 @@ fitfit(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
     if (_putobj(obj, prm, inst, &(fitlocal->coe[i]))) return 1;
   }
 
-  _getobj(obj,"equation",inst,&equation);
-  if (_putobj(obj,"equation",inst,fitlocal->equation)) return 1;
-  g_free(equation);
-  fitlocal->equation=NULL;
+  if (set_equation(obj, inst, fitlocal, fitlocal->equation)) return 1;
+  fitlocal->equation = NULL;
+
   return 0;
 }
 
 static int 
 fitcalc(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
-  MathEquation *eq;
   MathValue val;
   int r;
-  char *equation, *ptr, buf[32];
+  char *equation;
   double x;
+  struct fitlocal *fitlocal;
 
   if (_exeparent(obj, argv[1], inst, rval, argc, argv)) return 1;
 
-  g_free(rval->str);
-  rval->str=NULL;
+  rval->d = 0;
 
   x = * (double *) argv[2];
 
-  _getobj(obj, "equation", inst, &equation);
-  if (equation == NULL)
-    return 0;
+  _getobj(obj,"_local",inst,&fitlocal);
+  if (fitlocal->result_code == NULL) {
+    MathEquation *eq;
+    _getobj(obj, "equation", inst, &equation);
+    if (equation == NULL) {
+      return 0;
+    }
 
-  eq = math_equation_basic_new();
-  if (eq == NULL) {
-    return 1;
-  }
+    eq = ofile_create_math_equation(NULL, 0, FALSE, FALSE, FALSE, FALSE, TRUE);
+    if (eq == NULL) {
+      return 1;
+    }
 
-  if (math_equation_add_var(eq, "X") != 0) {
-    math_equation_free(eq);
-    return 1;
+    if (math_equation_parse(eq, equation)) {
+      math_equation_free(eq);
+      return 1;
+    }
+
+    math_equation_optimize(eq);
+    fitlocal->result_code = eq;
   }
 
   val.val = x;
   val.type = MATH_VALUE_NORMAL;
-  math_equation_set_var(eq, 0, &val);
+  math_equation_set_var(fitlocal->result_code, 0, &val);
 
-  if (math_equation_parse(eq, equation)) {
-    math_equation_free(eq);
-    return 1;
-  }
- 
-  r = math_equation_calculate(eq, &val);
-  math_equation_free(eq);
-  
+  r = math_equation_calculate(fitlocal->result_code, &val);
   if (r) {
     return 1;
   }
 
-  snprintf(buf, sizeof(buf), "%.15e", val.val);
-  ptr = g_strdup(buf);
-  rval->str = ptr;
+  rval->d = val.val;
 
   return 0;
 }
@@ -1076,7 +1157,7 @@ static struct objtable fit[] = {
   {"through_point",NBOOL,NREAD|NWRITE,fitput,NULL,0},
   {"point_x",NDOUBLE,NREAD|NWRITE,fitput,NULL,0},
   {"point_y",NDOUBLE,NREAD|NWRITE,fitput,NULL,0},
-  {"equation",NSTR,NREAD|NWRITE,NULL,NULL,0},
+  {"equation",NSTR,NREAD|NWRITE,fitequation,NULL,0},
 
   {"poly_dimension",NINT,NREAD|NWRITE,fitput,NULL,0},
 
@@ -1120,7 +1201,7 @@ static struct objtable fit[] = {
   {"display",NBOOL,NREAD|NWRITE,NULL,NULL,0},
 
   {"fit",NVFUNC,NREAD|NEXEC,fitfit,"da",0},
-  {"calc",NSFUNC,NREAD|NEXEC,fitcalc,"d",0},
+  {"calc",NDFUNC,NREAD|NEXEC,fitcalc,"d",0},
   {"_local",NPOINTER,0,NULL,NULL,0},
 };
 
