@@ -76,7 +76,7 @@ static int
 markdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
   int GC;
-  int x,y,type,size,width,fr,fg,fb,fa,br,bg,bb,ba,tm,lm,w,h;
+  int x,y,type,size,width,fr,fg,fb,fa,br,bg,bb,ba,w,h;
   struct narray *style;
   int snum,*sdata;
   int clip,zoom;
@@ -101,7 +101,7 @@ markdraw(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
   _getobj(obj,"clip",inst,&clip);
   snum=arraynum(style);
   sdata=arraydata(style);
-  GRAregion(GC,&lm,&tm,&w,&h,&zoom);
+  GRAregion(GC,&w,&h,&zoom);
   GRAview(GC,0,0,w*10000.0/zoom,h*10000.0/zoom,clip);
   GRAlinestyle(GC,snum,sdata,width,0,0,1000);
   GRAmark(GC,type,x,y,size,fr,fg,fb,fa,br,bg,bb,ba);
@@ -490,21 +490,6 @@ markgeometry(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
   return 0;
 }
 
-static int 
-marktype(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,
-	     int argc,char **argv)
-{
-  int type;
-
-  type = * (int *) (argv[2]);
-
-  if (type < 0 || type > 89) {
-    * (int *) (argv[2]) = 0;
-  }
-
-  return 0;
-}
-
 static int 
 markmatch(struct objlist *obj,N_VALUE *inst,N_VALUE *rval,int argc,char **argv)
 {
@@ -552,7 +537,7 @@ static struct objtable mark[] = {
   {"y",NINT,NREAD|NWRITE,markgeometry,NULL,0},
   {"size",NINT,NREAD|NWRITE,markgeometry,NULL,0},
 
-  {"type",NINT,NREAD|NWRITE,marktype,NULL,0},
+  {"type",NINT,NREAD|NWRITE,oputmarktype,NULL,0},
   {"R2",NINT,NREAD|NWRITE,oputcolor,NULL,0},
   {"G2",NINT,NREAD|NWRITE,oputcolor,NULL,0},
   {"B2",NINT,NREAD|NWRITE,oputcolor,NULL,0},
