@@ -353,86 +353,72 @@ print_func(struct nhash *h, void *data)
   return 0;
 }
 
-int 
+int
 cmset(struct nshell *nshell,int argc,char **argv)
 {
 #if USE_HASH
   char *s;
-  unsigned int n;
-  int j,ops;
-  char **argv2;
-  int argc2;
+  int j, r, ops;
 
   if (argc < 2) {
     nhash_each(nshell->valroot, print_val, NULL);
     nhash_each(nshell->valroot, print_func, NULL);
-  } else {
-    for (j = 1; j < argc; j++) {
-      s = argv[j];
-      if (s[0] == '-' && s[1] == '-') {
-	n = strlen(argv[j]);
-	memmove(argv[j], argv[j] + 1, sizeof(**argv) * n);
-        break;
-      } else if (s[0] == '-' || s[0] == '+') {
-	if (s[1] == '\0' || strchr("efvx", s[1]) == NULL) {
-	  sherror3(argv[0], ERRILOPS, s);
-	  return ERRILOPS;
+    return 0;
+  }
+  for (j = 1; j < argc; j++) {
+    s = argv[j];
+    if (s[0] == '-' && s[1] == '-' && s[2] == '\0') {
+      j++;
+      if (j == argc) {
+	r = set_shell_args(nshell, j, nshell->argv[0], argc, argv);
+	if (r) {
+	  r = ERR;
 	}
-      } else {
-	break;
+	return r;
       }
+      break;
+    } else if (s[0] == '-' && s[1] == '\0') {
+      nshell->optionv = FALSE;
+      nshell->optionx = FALSE;
+      j++;
+      break;
+    } else if (s[0] == '-' || s[0] == '+') {
+      if (s[1] == '\0' || strchr("efvx", s[1]) == NULL) {
+	sherror3(argv[0], ERRILOPS, s);
+	return ERRILOPS;
+      }
+    } else {
+      break;
     }
-    if (j != argc) { 
-      argv2 = NULL;
-      s = g_strdup(nshell->argv[0]);
-      if (s == NULL)
-	return ERR;
-
-      if (arg_add(&argv2, s) == NULL) {
-        g_free(s);
-        arg_del(argv2);
-        return ERR;
-      }
-
-      for (; j < argc; j++) {
-	s = g_strdup(argv[j]);
-        if (s == NULL)
-	  return ERR;
-
-        if (arg_add(&argv2, s) == NULL) {
-          g_free(s);
-          arg_del(argv2);
-          return ERR;
-        }
-      }
-      argc2 = getargc(argv2);
-      arg_del(nshell->argv);
-      nshell->argv = argv2;
-      nshell->argc = argc2;
+  }
+  if (j != argc) {
+    r = set_shell_args(nshell, j, nshell->argv[0], argc, argv);
+    if (r) {
+      return ERR;
     }
-    for (j = 1 ; j < argc; j++) {
-      s = argv[j];
-      if (s[0] == '-') {
-	ops=TRUE;
-      } else if (s[0]=='+') {
-	ops=FALSE;
-      } else {
-	break;
-      }
-      switch (s[1]) {
-      case 'e':
-        nshell->optione = ops;
-        break;
-      case 'f':
-        nshell->optionf = ops;
-        break;
-      case 'v':
-        nshell->optionv = ops;
-        break;
-      case 'x':
-        nshell->optionx = ops;
-        break;
-      }
+  }
+  for (j = 1 ; j < argc; j++) {
+    s = argv[j];
+    if (s[0] == '-') {
+      ops=TRUE;
+    } else if (s[0]=='+') {
+      ops=FALSE;
+    } else {
+      break;
+    }
+    switch (s[1]) {
+    case 'e':
+      nshell->optione = ops;
+      break;
+    case 'f':
+      nshell->optionf = ops;
+      break;
+    case 'v':
+      nshell->optionv = ops;
+      break;
+    case 'x':
+      nshell->optionx = ops;
+      break;
     }
   }
 #else
@@ -1031,8 +1017,10 @@ cmget(struct nshell*nshell,int argc,char **argv)
           }
           if (multi && !noid) printfstdout("%d: ",id);
           if (!nofield) printfstdout("%.256s:",field);
-          putstdout(valstr);
-          g_free(valstr);
+	  if (valstr) {
+	    putstdout(valstr);
+	    g_free(valstr);
+	  }
         }
       }
     } else {
@@ -1049,8 +1037,10 @@ cmget(struct nshell*nshell,int argc,char **argv)
             printfstdout("%.256s:",field);
             g_free(field);
           }
-          putstdout(valstr);
-          g_free(valstr);
+	  if (valstr) {
+	    putstdout(valstr);
+	    g_free(valstr);
+	  }
         }
       }
     }
